unit GrepWin;

interface
{$M 20000,8192}

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls,
  FileCtrl, Buttons, ExtCtrls, ShellApi, IniFiles,
  Options, AboutDlg, Menus, FmxPlus, Grids, Outline;

type
  TGrepForm = class(TForm)
    DirPanel: TPanel;
    DirectoryListBox1: TDirectoryListBox;
    DriveComboBox3: TDriveComboBox;
    DirLabel1: TLabel;
    FilterEdit: TEdit;
    DirLabel2: TLabel;
    Panel4: TPanel;
    StatLine: THeader;
    ListPanel: TPanel;
    AmlAboutDlg: TAmlAboutDlg;
    MainMenu1: TMainMenu;
    FileMenu: TMenuItem;
    FileCloseItem: TMenuItem;
    SearchItem: TMenuItem;
    OptionsItem: TMenuItem;
    HelpMeu: TMenuItem;
    HelpAboutItem: TMenuItem;
    Label1: TLabel;
    FilterPanel: TPanel;
    FilterLabel2: TLabel;
    ExpressionEdit1: TEdit;
    ExpressionLabel: TLabel;
    FilterComboBox1: TFilterComboBox;
    RecurseCheckBox: TCheckBox;
    CaseCheckBox: TCheckBox;
    RecurseLabel: TLabel;
    FileFilterLabel: TLabel;
    CaseLabel: TLabel;
    Panel1: TPanel;
    CloseBtn: TSpeedButton;
    SearchBtn: TSpeedButton;
    OptionsBtn: TSpeedButton;
    AboutBtn: TSpeedButton;
    StopBtn: TSpeedButton;
    Outline1: TOutline;
    OutlinePopup: TPopupMenu;
    ViewFileItem: TMenuItem;
    procedure FormResize(Sender: TObject);
    procedure FilterComboBox1Change(Sender: TObject);
    procedure SearchBtnClick(Sender: TObject);
    procedure ListBox1DblClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure OptionsBtnClick(Sender: TObject);
    procedure AboutBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure ExpressionEdit1Change(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure Outline1Click(Sender: TObject);
    procedure Outline1DblClick(Sender: TObject);
    procedure ViewFileItemClick(Sender: TObject);
  private
    GrepCount,LineCount: integer;
    procedure SearchFiles;
    function LogDir(const FileName: string; const SRec: TSearchRec): boolean;
    function LogFile(const FileName: string; const SRec: TSearchRec): boolean;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  GrepForm: TGrepForm;

implementation

uses RegExp;
{$R *.DFM}
const
  { const used to access Ini File }
  IniFileName    = 'AmlGrep.ini';
  DescSection    = 'Product';
  NameIdent      = 'Name';
  DefaultSection = 'Defaults';
  CaseIdent      = 'Case Match';
  RecurseIdent   = 'Recurse Subdirs';
  PreferenceSection = 'Preferences';
  BrowserIdent      = 'Browser';

type
  TRequest = record
    Expression: array[0..80] of Char;
    FileMask:   array[0..40] of Char;
    StartDir:   array[0..128] of Char;
    Browser:    array[0..128] of Char;
    CaseSensitive,
    UseSubDirs: Boolean;
  end;

var
  Browser: String;
  StopSearch: boolean;
type

TGrepEngine = class
  function ErrorText: string;
  constructor Create(SeaStr :String;
                     CaseSensitive: Boolean); virtual;
  destructor Destroy; override;
 public
  function GrepFile (Filename: TCaption; Outline: TOutline): Boolean;
 private
  Expression: HRegExp;
  ErrorCode: Integer;
  MatchCase: Boolean;
  procedure CompileExp(SeaStr :String);
end;

constructor TGrepEngine.Create(SeaStr :String;
                               CaseSensitive: Boolean);
begin
  Inherited Create;
  MatchCase := CaseSensitive;
  CompileExp(SeaStr);
end;

destructor TGrepEngine.Destroy;
begin
  RegFree(Expression);
  inherited Destroy;
end;

function TGrepEngine.ErrorText: String;
var
  ErrorText: array[0..255] of char;
begin
  FillChar(ErrorText,SizeOf(ErrorText),#0);
  RegError (Expression,ErrorCode,ErrorText);
  Result := StrPas(ErrorText);
end;

{Compile Expression}
procedure TGrepEngine.CompileExp (SeaStr :String);
var
  Exp: array[0..255] of Char;
begin
  ErrorCode := 0;
  FillChar(Exp,SizeOf(Exp),#0);
  StrPCopy(Exp, SeaStr);
  if not MatchCase then StrUpper(Exp);
  Expression := RegComp(Exp, ErrorCode);
end;

{Search specified file for expression}
function TGrepEngine.GrepFile (Filename: TCaption; Outline :TOutline): Boolean;
var
  InFile: TextFile;
  Line: array[0..255] of Char;
  Buff: String;
  Match: TRegMatch;
  Added :boolean;
  Index :integer;
  LineIndex :integer;
begin
  Added := false;
  GrepFile := False;
  LineIndex := 0;
  AssignFile(InFile, Filename);
  {$I-}
  Reset(InFile);
  while not Eof(InFile) do
  begin
    Inc(LineIndex);
    Readln(InFile, Line);
    Buff := StrPas(Line);
    if not MatchCase then StrUpper(Line);
    if RegExec(Expression, Line, Match) = 0 then
    begin
      GrepFile := True;
      if not Added then
      begin
        Index := Outline.Add(0,Filename);
        Added := true;
      end;
      Outline.AddChild(Index,Format('%5d: %s',[LineIndex,Buff]));
{      Break;}
    end;
  end;
  CloseFile(InFile);
end;

{ Execute a child program via ShellExecute API }
function ExecuteFile(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): THandle;
var
  zFileName, zParams, zDir: array[0..79] of Char;
begin
  Result := ShellExecute(Application.MainForm.Handle, nil,
  StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
  StrPCopy(zDir, DefaultDir), ShowCmd);
end;

var
  GrepEngine: TGrepEngine;

procedure TGrepForm.FilterComboBox1Change(Sender: TObject);
begin
  FilterEdit.Text := FilterComboBox1.Mask;
end;

function TGrepForm.LogFile(const FileName: string; const SRec: TSearchRec): boolean;
begin
  Application.Processmessages;
  if StopBtn.Enabled then
  begin
    Statline.Sections[0] := Format('Searching : %s',[FileName]);
    Statline.Update;
    if length(ExpressionEdit1.Text) > 0 then
    begin
      if GrepEngine.Grepfile(Filename,Outline1) then
        Inc(GrepCount);
    end
    else
      Outline1.Add(0,Filename);
  end;
  Result := StopBtn.Enabled;
end;

function TGrepForm.LogDir(const FileName: string; const SRec: TSearchRec): boolean;
begin
  Application.Processmessages;
  if StopBtn.Enabled then
  begin
    Statline.Sections[0] := Format('Recursing directory: %s',[FileName]);
    Statline.Update;
  end;
  Result := StopBtn.Enabled;
end;

procedure TGrepForm.SearchFiles;
var
  FileFound : integer;
begin {SearchFiles}
  FileFound := 0;
  GrepCount := 0;
  LineCount := 0;
  Outline1.Clear;
  if length(ExpressionEdit1.Text) = 0 then
   {MessageDlg('Cannot search an empty expression', mtError,[mbOK],0) }
    FileFound := ScanDir (DirectoryListBox1.Directory,
               FilterEdit.Text,
               RecurseCheckBox.Checked,
               LogDir,
               LogFile)
  else
  begin
{    CompileExp(Expression,ExpressionEdit1.Text,CaseCheckBox.Checked);}
    GrepEngine := TGrepEngine.Create(ExpressionEdit1.Text,CaseCheckBox.Checked);
    FileFound := ScanDir (DirectoryListBox1.Directory,
               FilterEdit.Text,
               RecurseCheckBox.Checked,
               LogDir,
               LogFile);
    GrepEngine.Destroy;
  end;
  StatLine.Sections[0] := Format('Searched %d file(s), found %d',[FileFound,GrepCount]);
end; {SearchFiles}

procedure TGrepForm.SearchBtnClick(Sender: TObject);
begin
  StopBtn.Enabled := true;
  SearchBtn.Enabled := false;
  SearchFiles;
  SearchBtn.Enabled := true;
  StopBtn.Enabled := false;
end;

procedure TGrepForm.FormResize(Sender: TObject);
begin
  if Width < 441 then Width := 441;
  if Height <363 then Height := 363;
{  ListPanel.SetBounds(160,89,ClientWidth-160,ClientHeight-117);}
  Outline1.SetBounds (8,24,ListPanel.Width -16,ListPanel.Height -31);
  DirectoryListBox1.Height := DirPanel.Height - 71;
end;

procedure TGrepForm.ListBox1DblClick(Sender: TObject);
begin
{  with ListBox1 do
  ExecuteFile(Browser,Items[ItemIndex],'',sw_ShowNormal);}
end;

procedure TGrepForm.FormActivate(Sender: TObject);
var
  IniFile: TIniFile;
  ProgDesc: String;
begin
  IniFile := TInifile.Create (IniFileName);
  try
    with IniFile do
    begin
      ProgDesc := ReadString(DescSection,NameIdent,'New');
      CaseCheckBox.Checked := ReadBool (DefaultSection,CaseIdent,False);
      RecurseCheckBox.Checked := ReadBool (DefaultSection,RecurseIdent,False);
      Browser := ReadString (PreferenceSection,BrowserIdent,'Notepad.exe');
      if ProgDesc = 'New' then
      begin
        WriteString(DescSection,NameIdent,Application.Title);
        MessageDlg ('Welcome to ' + Application.Title +
                    '. Press the main dialog''s About button to obtain' +
                    ' informations about application and license.',
                    mtInformation,[mbOK],0);
      end;
    end;
  finally
    IniFile.Free
  end;
end;

procedure TGrepForm.OptionsBtnClick(Sender: TObject);
var IniFile :TIniFile;
begin
  with OptionsForm do
  begin
    CaseDefault.Checked := CaseCheckBox.Checked;
    RecurseDefault.Checked := RecurseCheckBox.Checked;
    DefaultBrowser.Text := Browser;
    if OptionsForm.ShowModal = mrOK then
    begin
      CaseCheckBox.Checked := CaseDefault.Checked;
      RecurseCheckBox.Checked := RecurseDefault.Checked;
      Browser := DefaultBrowser.Text;
      IniFile := TIniFile.Create(IniFileName);
      try
        with IniFile do
        begin
          WriteBool (DefaultSection,CaseIdent,CaseCheckBox.Checked);
          WriteBool (DefaultSection,RecurseIdent,RecurseCheckBox.Checked);
          WriteString (PreferenceSection,BrowserIdent,Browser);
        end;
      finally
        IniFile.Free;
      end;
    end;
  end;
end;

procedure TGrepForm.AboutBtnClick(Sender: TObject);
begin
    AmlAboutDlg.Execute;
end;



procedure TGrepForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TGrepForm.ExpressionEdit1Change(Sender: TObject);
begin
  CaseCheckBox.Enabled := (length(ExpressionEdit1.Text) > 0);
end;

procedure TGrepForm.StopBtnClick(Sender: TObject);
begin
  StopBtn.Enabled := false;
end;

procedure TGrepForm.Outline1Click(Sender: TObject);
var
  i:integer;
  CP :TPoint;
begin
  with Outline1 do
  begin
    GetCursorPos(CP);
    CP := Outline1.ScreenToClient(CP);
    if CP.X < Outline1.ItemHeight then
    begin
    if Items[SelectedItem].Expanded then
      Items[SelectedItem].Collapse
    else
      Items[SelectedItem].Expand;
    end;
  end
end;

procedure TGrepForm.Outline1DblClick(Sender: TObject);
var item: string;
i:integer;
begin
  i :=Outline1.SelectedItem;
  with Outline1 do
  begin
    if Items[SelectedItem].Level = 1 then
      ExecuteFile(Browser,Items[SelectedItem].Text,'',sw_ShowNormal);
  end;
end;

procedure TGrepForm.ViewFileItemClick(Sender: TObject);
begin
  with Outline1 do
  begin
    if Items[SelectedItem].Level = 1 then
      ExecuteFile(Browser,Items[SelectedItem].Text,'',sw_ShowNormal);
  end;
end;

end.
