unit Touchwin;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, FileCtrl, ExtCtrls, Spin, AboutDlg,
  Menus;

type
  TTouchForm = class(TForm)
    Timer1: TTimer;
    AmlAboutDlg: TAmlAboutDlg;
    SpeedPanel: TPanel;
    CloseBtn: TSpeedButton;
    AboutBtn: TSpeedButton;
    MainMenu1: TMainMenu;
    FileMenu: TMenuItem;
    FileCloseItem: TMenuItem;
    ControlMenu: TMenuItem;
    ControlTouchItem: TMenuItem;
    ControlSetTimeItem: TMenuItem;
    HelpMenu: TMenuItem;
    HelpAboutItem: TMenuItem;
    DirPanel: TPanel;
    DirectoryListBox: TDirectoryListBox;
    DirLabel: TLabel;
    Label1: TLabel;
    FileListBox: TFileListBox;
    DriveComboBox: TDriveComboBox;
    DriveLabel: TLabel;
    FilterComboBox: TFilterComboBox;
    FilterLabel: TLabel;
    FileEdit: TEdit;
    Label2: TLabel;
    ListPanel: TPanel;
    SelectedListBox: TListBox;
    Label4: TLabel;
    SecSpinEdit: TSpinEdit;
    MinuteSpinEdit: TSpinEdit;
    HourSpinEdit: TSpinEdit;
    TimeStampLabel: TLabel;
    MovePanel: TPanel;
    AddFileBtn: TSpeedButton;
    RemoveFileBtn: TSpeedButton;
    AddAllBtn: TSpeedButton;
    RemoveAllBtn: TSpeedButton;
    StatusPanel: TPanel;
    Header: THeader;
    SetCurrentTimeBtn: TSpeedButton;
    TouchBtn: TSpeedButton;
    StopBtn: TSpeedButton;
    procedure FileListBoxDblClick(Sender: TObject);
    procedure AddFileBtnClick(Sender: TObject);
    procedure RemoveFileBtnClick(Sender: TObject);
    procedure SelectedListBoxDblClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure SetCurrentTimeBtnClick(Sender: TObject);
    procedure DirectoryListBoxChange(Sender: TObject);
    procedure AboutBtnClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure TouchBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure AddAllBtnClick(Sender: TObject);
    procedure RemoveAllBtnClick(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  TouchForm: TTouchForm;

implementation

{$R *.DFM}

var
  CurPath: TCaption;

procedure SetFileTimeStamp(const FileName,TimeStamp: string);
var
  FileHandle: integer;
begin
  FileHandle := FileOpen(FileName,fmShareDenyWrite);
  try
    FileSetDate(FileHandle, DateTimeToFileDate(StrToDateTime(TimeStamp)));
  finally
    FileClose(FileHandle);
  end;
end;

procedure TTouchForm.FileListBoxDblClick(Sender: TObject);
begin
  with FileListBox do
  begin
    SelectedListBox.Items.Add(ExtractFileName(FileName));
    Items.Delete(ItemIndex);
  end;
  TouchBtn.Enabled := true
end;

procedure TTouchForm.SelectedListBoxDblClick(Sender: TObject);
begin
  with SelectedListBox do
  begin
    FileListBox.Items.Add(ExtractFileName(Items[ItemIndex]));
    Items.Delete(ItemIndex);
  end;
  if SelectedListBox.Items.Count = 0 then
    TouchBtn.Enabled := false;
end;

procedure TTouchForm.AddFileBtnClick(Sender: TObject);
var I :integer;
begin
  Screen.Cursor := crHourGlass;
  try
    I := 0;
    with FileListBox do
      while I < Items.Count do
      if Selected[I] then
      begin
        SelectedListBox.Items.Add(ExtractFileName(Items[I]));
        Items.Delete(I);
      end
      else
        Inc(I);
    if SelectedListBox.Items.Count > 0 then
      TouchBtn.Enabled := true;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TTouchForm.AddAllBtnClick(Sender: TObject);
var I :integer;
begin
  Screen.Cursor := crHourGlass;
  try
    I := 0;
    with FileListBox do
      while I < Items.Count do
      begin
        SelectedListBox.Items.Add(ExtractFileName(Items[I]));
        Items.Delete(I);
      end;
    if SelectedListBox.Items.Count > 0 then
      TouchBtn.Enabled := true;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TTouchForm.RemoveFileBtnClick(Sender: TObject);
var I :integer;
begin
  Screen.Cursor := crHourGlass;
  try
    I := 0;
    with SelectedListBox do
      while I < Items.Count do
      if Selected[I] then
      begin
        FileListBox.Items.Add(ExtractFileName(Items[I]));
        Items.Delete(I);
      end
      else
        Inc(I);
    if SelectedListBox.Items.Count = 0 then
      TouchBtn.Enabled := false;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TTouchForm.RemoveAllBtnClick(Sender: TObject);
var I :integer;
begin
  Screen.Cursor := crHourGlass;
  try
    I := 0;
    with SelectedListBox do
      while I < Items.Count do
      begin
        FileListBox.Items.Add(ExtractFileName(Items[I]));
        Items.Delete(I);
      end;
    if SelectedListBox.Items.Count = 0 then
      TouchBtn.Enabled := false;
  finally
    Screen.Cursor := crDefault;
  end;
end;
procedure TTouchForm.Timer1Timer(Sender: TObject);
begin
   Header.Sections[0] := TimeToStr(Time);
end;

procedure TTouchForm.SetCurrentTimeBtnClick(Sender: TObject);
var Hour,Min,Sec,MSec :Word;
begin
  DecodeTime(Time,Hour,Min,Sec,MSec);
  HourSpinEdit.Value := Hour;
  MinuteSpinEdit.Value := Min;
  SecSpinEdit.Value := Sec;
end;

procedure TTouchForm.DirectoryListBoxChange(Sender: TObject);
begin
  SelectedListBox.Clear;
  TouchBtn.Enabled := false;
end;

procedure TTouchForm.AboutBtnClick(Sender: TObject);
begin
  AmlAboutDlg.Execute;
end;

procedure TTouchForm.FormResize(Sender: TObject);
begin
  If Height < 334 then Height := 334;
  If Width < 502 then Width := 502;
  DirectoryListBox.Height := DirPanel.Height - 96;
  FileListBox.Height := DirPanel.Height - 96;
  SelectedListBox.Height := ListPanel.Height - 96;
  SelectedListBox.Width := ListPanel.Width - 16;
end;

procedure TTouchForm.TouchBtnClick(Sender: TObject);
var I: integer;
begin
  StopBtn.Enabled := true;
  Screen.Cursor := crHourGlass;
  try
    I := 0;
    with SelectedListBox do
      while I < Items.Count do
      begin
        { Gives other Apps chance to process messages
          ad user to stop loop
        }
        Application.ProcessMessages;
        if not StopBtn.Enabled then Break;
        try
          setfiletimestamp(DirectoryListBox.Directory + '\' + Items[I],
                         DateToStr(Now) + ' ' + IntToStr(HourSpinEdit.Value) +
                         TimeSeparator + IntToStr(MinuteSpinEdit.Value) +
                         TimeSeparator + IntToStr(SecSpinEdit.Value));
        finally
          FileListBox.Items.Add(ExtractFileName(Items[I]));
          Items.Delete(I);
        end;
      end;
    if SelectedListBox.Items.Count = 0 then
      TouchBtn.Enabled := false;
  finally
    StopBtn.Enabled := false;
    Screen.Cursor := crDefault;
  end;
end;

procedure TTouchForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;



procedure TTouchForm.StopBtnClick(Sender: TObject);
begin
  StopBtn.Enabled := false;
end;

end.
