{ Demo BPW7 app to show how BDE ASCII driver works on
  comma separated fields  ...

  Author : Ian Hayes (Voice: UK 0171-602-3010,
                      Fax UK 0171-602-3027,
                      Email Compuserve 100010,1415)
  Date:    28th/9/95

  Notes : To run this app ....

          (a) Using the IDAPI config utility, setup a new alias
              with a type STANDARD, and driver 'ASCIIDRV'
          (b) copy the enclosed ASCII file 'PERIOD.TXT' into the
              path referenced by the new DB alias
          (c) Run this app with one command line param - the alias
              e.g. ASCIIDEM.EXE myalias

   Data file :

         Period.txt is a simple two field comma separated ASCII table :

               "01","January"
               "02","February"
               "03","March"
               "04","April"
               "05","May"
               "06","June"
               "07","July"
               "08","August"
               "09","September"
               "10","October"
               "11","November"
               "12","December"

   Points to note :

      (a)  The driver name variable 'TheDriverName' works as defined. If it
           is changed from ....

                          'ASCIIDRV-\"-,'

           to ....
                          'ASCIIDRV-\"-\,'

           ... the test app does'nt read secondary fields.

}

PROGRAM AsciiPas;

USES

   WinTypes,
   Strings,
   WinDos,
   WinCrt,
   DbiTypes,
   DbiErrs,
   Idapi;

CONST

   TheDriverName : PChar = 'ASCIIDRV-\"-,';
   TheTableName : PChar = 'Period';

VAR

   TheAlias : ARRAY[0..fsPathName] OF CHAR;
   TheDbH          : hDbiDB;     { database handle }
   TheTableCursor  : hDbiCur;    { table cursor handle }
   TheFldMapArray,
   TheTargetArray  : ARRAY[0..1] OF FLDDESC;
   TheDataBuffer : POINTER;
   ACurProps : CURPROPS;
   S : DBIRESULT;
   TheRecSize      : WORD;
   TheRecNo : LONGINT;
   TheFldNo : INTEGER;
   IsNull : BOOL;
   TheFieldStr : PChar;

   { ++++++++++++++++++++++++++++++++++++++++++++++ }

   PROCEDURE DisplayErrorMsg(E : DBIRESULT);
   VAR
      AStr : ARRAY[0..256] OF CHAR;
   BEGIN
      DbiGetErrorString(E,AStr);
      Writeln('    ',AStr);
      Writeln('    Hit ENTER key to continue ....');
      readln;
   END;

   { ++++++++++++++++++++++++++++++++++++++++++++++ }

BEGIN

   CmdShow := sw_ShowMaximized;

   GetMem(TheFieldStr,256);

   InitWinCrt;

   IF GetArgCount <> 1 THEN
   BEGIN
      Writeln('This test app must have 1 param - the DB alias!');
      readln;
   END
   ELSE
   BEGIN
      { 1st param is alias }
      GetArgStr(TheAlias,1,256);
      { init IDAPI }
      Writeln('Initialising IDAPI ....');
      S := DbiInit(NIL);
      IF S <> DBIERR_NONE THEN
      BEGIN
         Writeln('    FAILED!');
         DisplayErrorMsg(S);
      END
      ELSE
      BEGIN
         Writeln('    Ok');
         { init database }
         Writeln('Opening database ',TheAlias,' ....');
         S := DbiOpenDatabase(TheAlias,
                              nil,
                              dbiREADWRITE,
                              dbiOPENSHARED,
                              NIL,
                              0,
                              nil,
                              nil,
                              TheDbH);
         IF S <> DBIERR_NONE THEN
         BEGIN
            Writeln('    FAILED!');
            DisplayErrorMsg(S);
         END
         ELSE
         BEGIN
            { open table }
            Writeln('Opening table ',TheTableName,' ....');
            S := DbiOpenTable(TheDbH,          { Database handle }
                              TheTableName,      { Table name or file name }
                              TheDriverName,     { Driver type }
                              NIL,
                              NIL,
                              0,
                              dbiREADWRITE,
                              dbiOPENSHARED,
                              xltFIELD,        { Xlate mode }
                              FALSE,           { Uni or Bi directional }
                              NIL,             { Optional params /NULL }
                              TheTableCursor);
            IF S <> DBIERR_NONE THEN
            BEGIN
               Writeln('    FAILED!');
               DisplayErrorMsg(S);
            END
            ELSE
            BEGIN
               { set up field map array }
               FILLCHAR(TheFldMapArray,SIZEOF(TheFldMapArray),#0);
               WITH TheFldMapArray[0] DO
               BEGIN
                  iFldNum := 1;
                  StrCopy(szName,'PeriodCode');
                  iFldType := fldZSTRING;
                  iUnits1 := 4;
               END;
               WITH TheFldMapArray[1] DO
               BEGIN
                  iFldNum := 2;
                  StrCopy(szName,'PeriodName');
                  iFldType := fldZSTRING;
                  iUnits1 := 16;
               END;
               Writeln('Translating source field map ....');
               S := DbiTranslateRecordStructure(NIL,
                                                2,
                                                @TheFldMapArray,
                                                szASCII,
                                                NIL,
                                                @TheTargetArray);
               IF S <> DBIERR_NONE THEN
               BEGIN
                  Writeln('    FAILED!');
                  DisplayErrorMsg(S);
               END;
               { place field map over table }
               Writeln('Placing field map ....');
               S := DbiSetFieldMap(TheTableCursor,2,@TheTargetArray);
               IF S <> DBIERR_NONE THEN
               BEGIN
                  Writeln('    FAILED!');
                  DisplayErrorMsg(S);
               END
               ELSE
               BEGIN
                  { goto first rec }
                  S := DbiSetToBegin(TheTableCursor);
                  S := DbiGetCursorProps(TheTableCursor,ACurProps);
                  Writeln('Get Cursor properties ....');
                  IF S <> DBIERR_NONE THEN
                  BEGIN
                     Writeln('    FAILED!');
                     DisplayErrorMsg(S);
                  END;
                  TheRecSize := ACurProps.iRecBufSize * sizeof(BYTE);
                  GetMem(TheDataBuffer,TheRecSize);
                  { setup record buffer }
                  S := DbiInitRecord(TheTableCursor,TheDataBuffer);
                  Writeln('Initialising record buffer ....');
                  IF S <> DBIERR_NONE THEN
                  BEGIN
                     Writeln('    FAILED!');
                     DisplayErrorMsg(S);
                  END;
                  { read all records }
                  TheRecNo := 0;
                  REPEAT
                     INC(TheRecNo);
                     S := DbiGetNextRecord(TheTableCursor,
                                           dbiNOLOCK,
                                           TheDataBuffer,
                                           NIL);
                     IF S = DBIERR_NONE THEN
                     BEGIN
                        { display record contents to screen }
                        Writeln('Record number = ',TheRecNo);
                        FOR TheFldNo := 1 TO 2 DO
                        BEGIN
                           StrCopy(TheFieldStr,#0);
                           IsNull := FALSE;
                           S := DbiGetField(TheTableCursor,
                                            TheFldNo,
                                            TheDataBuffer,
                                            TheFieldStr,
                                            IsNull);
                           IF S <> DBIERR_NONE THEN
                           BEGIN
                              Writeln('    DbiGetField failed!');
                              DisplayErrorMsg(S);
                           END
                           ELSE
                           BEGIN
                              IF IsNull THEN
                                 Writeln('   Field number ',
                                         TheFldNo,
                                         ' : DbiGetField pbBlank = TRUE',
                                         ' (',TheFieldStr,')')
                              ELSE
                              BEGIN
                                 IF TheFieldStr[0] = #0 THEN
                                    Writeln('   Field number ',
                                            TheFldNo,
                                            ' : (NULL STRING)')
                                 ELSE
                                    Writeln('   Field number ',
                                            TheFldNo,' : ',
                                            TheFieldStr)
                              END
                           END
                        END
                     END;
                  UNTIL S <> DBIERR_NONE;

                  Writeln('Hit the ENTER key to continue ....');
                  Readln;
                  { close record buffer }
                  FreeMem(TheDataBuffer,TheRecSize);
               END;
               { close table }
               S := DbiCloseCursor(TheTableCursor);
            END;
            { close database }
            S := DbiCloseDatabase(TheDbH);
         END;
         { close down IDAPI }
         S := DbiExit;
      END
   END;

   DoneWinCrt;

   FreeMem(TheFieldStr,256);

END.