unit CurnEdit;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ClipBrd;

type
  TCurrencyEdit = class(TEdit)
  private
    FNumberOfDecimals: integer;
    FAsFloat: double;
    FMaxLength: integer;
    FormatString: string;
    ValueJustEntered: string;
    DistanceFromDecimal: integer;
    ForcedCursorPos: integer;
    JustEnteredWithMouse: boolean;
    FChange: TNotifyEvent;
    function GetAsFloat: double;
    procedure SetAsFloat(Value: double);
    procedure CMEnter(var Message: TCMGotFocus); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    procedure KeyDown(var Key: word; Shift: TShiftState); override;
    procedure KeyPress(var Key: char); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Change; override;
    procedure SetNumberOfDecimals(Value: integer);
    function  StripCommas(AString: string): string;
    procedure GetCursorPosition;
    procedure SetCursorPosition;
    procedure PasteText;
    procedure CutText;
  public
    constructor Create(AOwner: TComponent); override;
    property AsFloat: double read GetAsFloat write SetAsFloat;
  published
    property MaxLength: integer read FMaxLength write FMaxLength;
    property NumberOfDecimals: integer read FNumberOfDecimals write SetNumberOfDecimals;
    property OnChange: TNotifyEvent read FChange write FChange;
  end;

procedure Register;

implementation

{uses DebugLog;}

const
  MaxCurrencyLength: integer = 16; {set for 999,999,999,999. + decimals}

constructor TCurrencyEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FNumberOfDecimals := 2;
  FMaxLength := MaxCurrencyLength + FNumberOfDecimals;
  ForcedCursorPos := -99;
end;

function TCurrencyEdit.GetAsFloat: double;
begin
  Result := StrToFloat(StripCommas(Text));
end;

procedure TCurrencyEdit.SetAsFloat(Value: double);
begin
  if Value <> StrToFloat(StripCommas(Text)) then
  begin
    FAsFloat := Value;
    Text := FormatFloat(FormatString, Value);
    Change;
  end;
end;

procedure TCurrencyEdit.CMEnter(var Message: TCMGotFocus);
begin
  inherited;
  if Text <> '' then  {only format and set cursor pos if EditText has a value}
  begin
    Text := FormatFloat(FormatString, StrToFloat(StripCommas(Text)));
    DistanceFromDecimal := 1;
    SetCursorPosition;
    SelStart := 0;
    SelLength := Length(Text);
  end;
end;

procedure TCurrencyEdit.CMExit(var Message: TCMExit);
begin
  inherited;
  JustEnteredWithMouse := False;
end;

procedure TCurrencyEdit.SetNumberOfDecimals(Value: integer);
begin
    FNumberOfDecimals := Value;
    case FNumberOfDecimals of
    0:  FormatString := '#,##0';
    1:  FormatString := '#,##0.0';
    2:  FormatString := '#,##0.00';
    3:  FormatString := '#,##0.000';
    4:  FormatString := '#,##0.0000';
    5:  FormatString := '#,##0.00000';
    end;
    Text := FormatFloat(FormatString, StrToFloat(StripCommas(Text)));
    FMaxLength := MaxCurrencyLength + FNumberOfDecimals;
end;

procedure TCurrencyEdit.KeyDown(var Key: word; Shift: TShiftState);
begin

  if (ssShift in Shift) and (key = 45) then  {Shift-DEL}
    PasteText;
  if (ssShift in Shift) and (key = 46) then  {Shift-INS}
    CutText;
  if (ssCtrl in Shift) and (key = 45) then   {Ctrl-DEL}
    CopyToClipboard;

  if Key = 46 then
  begin
    GetCursorPosition;

    if SelLength > 0 then
    begin
      ValueJustEntered := Text;
      Delete(ValueJustEntered, SelStart + 1, SelLength);
      Text := ValueJustEntered;
    end
    else
    begin
      if (Text[SelStart + 1] = DecimalSeparator) or
         (Text[SelStart + 1] = ThousandSeparator) then
      begin
        ValueJustEntered := Text;
        Delete(ValueJustEntered, SelStart + 2, 1);
        DistanceFromDecimal := DistanceFromDecimal - 2;
        Text := ValueJustEntered;
      end
      else
      begin
        ValueJustEntered := Text;
        Delete(ValueJustEntered, SelStart + 1, 1);
        DistanceFromDecimal := DistanceFromDecimal - 1;
        Text := ValueJustEntered;
      end;
    end;
    Key := 0;
    SetCursorPosition;
  end
  else
    inherited KeyDown(Key, Shift);
end;

procedure TCurrencyEdit.KeyPress(var Key: char);
begin

GetCursorPosition;

  case Key of
  '0'..'9':
    begin
      ValueJustEntered := Text;
      if SelLength = Length(Text) then  {all text selected}
      begin
        Delete(ValueJustEntered, SelStart + 1, SelLength);
        Insert(Key, ValueJustEntered, SelStart + 1);
        ForcedCursorPos := 1;
      end
      else if SelLength > 0 then
      begin
        Delete(ValueJustEntered, SelStart + 1, SelLength);
        Insert(Key, ValueJustEntered, SelStart + 1);
      end
      else
      begin
        if DistanceFromDecimal < 1 then
        begin
          Insert(Key, ValueJustEntered, SelStart + 1);
            if Pos(DecimalSeparator, ValueJustEntered) > 0 then
              ValueJustEntered[Pos(DecimalSeparator, ValueJustEntered) +
                NumberOfDecimals + 1] := '0'
            else
              if NumberOfDecimals > 0 then
                ValueJustEntered[Length(ValueJustEntered)] := '0';
                {do nothing if no decimal point}
            ForcedCursorPos := DistanceFromDecimal - 1;
        end;
        if ((StrToFloat(StripCommas(Text)) < 1) and (DistanceFromDecimal = 1)) then
        begin
          ValueJustEntered := FloatToStr( (StrToFloat(Key)) + (StrToFloat( StripCommas(Text))) );
          Key := #0;  
        end;

        if (DistanceFromDecimal >= 1) and (StrToFloat( StripCommas(Text)) >= 1) then
        begin
          Insert(Key, ValueJustEntered, SelStart + 1);
          {Key := #0;}
        end;
      end;
    Key := #0;
    if Length(ValueJustEntered) <= FMaxLength then
       Text := ValueJustEntered
    else
       MessageBeep(0);
    end;
  ^H:
    begin
      ValueJustEntered := Text;
      if SelLength < 1 then
      begin
        if ValueJustEntered[SelStart] = ThousandSeparator then
        begin
          Delete(ValueJustEntered, SelStart, 1);
        end;
        if ValueJustEntered[SelStart] <> DecimalSeparator then
          Delete(ValueJustEntered, SelStart, 1)
        else
          ForcedCursorPos := 1;
        Text := FormatFloat( FormatString, StrToFloat(StripCommas(ValueJustEntered)));
      end
      else    {text block selected}
      begin
        Delete(ValueJustEntered, SelStart + 1, SelLength);
        Text := FormatFloat( FormatString, StrToFloat(StripCommas(ValueJustEntered)));
      end;
    Key := #0;
    SetCursorPosition;
    end;
  #27:
    begin
      SelectAll;
      ClearSelection;
      Key := #0;
    end;
  '.',',':
    begin
      if Key = ThousandSeparator then Key := #0;
      if Key = DecimalSeparator then
      begin
        ForcedCursorPos := 0;
        Key := #0;
      end;
    end;
  ^C:
    begin
     CopyToClipboard;
     Key := #0;
    end;
  ^V:
    begin
     PasteText;
     Key := #0;
    end;
  ^X:
    begin
     CutText;
     Key := #0;
    end;
  else
    begin
      MessageBeep(0);
      Key := #0;
    end;
  end;

SetCursorPosition;
end;

procedure TCurrencyEdit.Change;
begin
  if csDesigning in ComponentState then
    Text := Name
  else
  begin
    if Text = Name then
      Text := FormatFloat( FormatString, 0)
    else
      Text := FormatFloat( FormatString, StrToFloat(StripCommas(Text)));
  end;
  if assigned(FChange) then OnChange(Self);
end;

function TCurrencyEdit.StripCommas(AString: string): string;
begin
  if (AString = '') or (AString = Name) then
    Result := '0'
  else
  begin
    while (Pos(ThousandSeparator, AString) > 0 ) do
      delete(AString, Pos(ThousandSeparator, AString), 1);
    Result := AString;
  end;
end;

procedure TCurrencyEdit.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  DecimalPos: integer;
begin
  inherited MouseUp(Button, Shift, X, Y);
  if (Button = mbLeft) then
  begin
    if not JustEnteredWithMouse and (Text <> '') then  {only format and set cursor pos if EditText has a value}
      begin
        Text := FormatFloat( FormatString, StrToFloat(StripCommas(Text)));
        DecimalPos := Pos(DecimalSeparator, Text);
        SelStart := DecimalPos - 1 ;
        JustEnteredWithMouse := True;
      end;
  end;
end;

procedure TCurrencyEdit.GetCursorPosition;
var
  DecimalPos: integer;
begin
  DecimalPos := Pos(DecimalSeparator, Text);  {locate decimal point}

  if DecimalPos <= 0 then        {if no decimal point }
    DecimalPos := Length(Text);  {calculate cursor pos from end of string}

  if SelLength > 0 then          {text block selected}
    DistanceFromDecimal := DecimalPos - (SelStart + SelLength)  {calc from end of text block}
  else
    DistanceFromDecimal := DecimalPos - SelStart;    {calc num places to left of decimal}
end;

procedure TCurrencyEdit.SetCursorPosition;
var
  DecimalPos: integer;
begin
  DecimalPos := Pos(DecimalSeparator, Text);  {locate decimal point}

  if DecimalPos <= 0 then        {if no decimal point }
    DecimalPos := Length(Text) + 1;  {calculate cursor pos from end of string}

  if ForcedCursorPos <> -99 then
  begin
    SelStart := DecimalPos - ForcedCursorPos;
    ForcedCursorPos := -99;
  end
  else
  begin
    if DistanceFromDecimal = 0 then
      SelStart := DecimalPos + 1
    else
      SelStart := DecimalPos - DistanceFromDecimal;
  end;
end;

procedure TCurrencyEdit.PasteText;
var
  Original: string;
begin
  Original := Text;
  if SelLength > 0 then
  begin
    Delete(ValueJustEntered, SelStart + 1, SelLength);
  end;
  if StrToFloat( StripCommas(ValueJustEntered)) = 0 then
  begin
    ValueJustEntered := '';
  end;
  if Clipboard.HasFormat(CF_TEXT) then
    Insert(Clipboard.AsText, Original, SelStart + 1);
  Text := FormatFloat( FormatString, StrToFloat( StripCommas(Original)));
end;

procedure TCurrencyEdit.CutText;
begin
  Clipboard.AsText := SelText;
  ValueJustEntered := Text;
  Delete(ValueJustEntered, SelStart + 1, SelLength);
  Text := FormatFloat( FormatString, StrToFloat( StripCommas(ValueJustEntered)));
end;

procedure Register;
begin
  RegisterComponents('Samples', [TCurrencyEdit]);
end;

end.
