{COMMSPY Unit was developed from ModemSpy, which is described below.}
(**********************************************************************
 *                                                                    *
 * Modem Spy - Main Unit                                              *
 *                                                                    *
 * This program is supplied as is. I tried to do my best. Suggestions *
 * enhancements, positive critics are most welcome. If you like this  *
 * stuff, just drop me a few lines on a postcard with a nice stamp    *
 * and send it to:                                                    *
 *    Dr. Martin Mohnhaupt                                            *
 *    Mediterranean Shipping Company SA                               *
 *    18 Chemin Rieu                                                  *
 *    CH - 1208 Geneva (Switzerland)                                  *
 *                                                                    *
 * Bottles of (good) wine will not be refused!                        *
 *                                                                    *
 * Internet: mmohnhaupt@ping.ch                                       *
 *                                                                    *
 * E N J O Y !!!                                                      *
 **********************************************************************)
unit CommSpy;

interface

uses
  SysUtils;

const
   { Register offsets from the base I/O addresses }
   MODEMCONTROL = 4;
   LINESTATUS   = 5;
   MODEMSTATUS  = 6;

   { Some masks... }
   DCD : byte = $80; { Data Carrier Detect    }
   RI  : byte = $40; { Ring Indicator         }
   DSR : byte = $20; { Data Set Ready         }
   CTS : byte = $10; { Clear To Send          }
   RxD : byte = $01; { Received Data Ready    }
   TxD : byte = $40; { Data Transmitter Empty }
   RTS : byte = $02; { Request To Send        }
   DTR : byte = $01; { Data Terminal Ready    }

var
  Port, PortWord : Word;
  psDCD, psRI, psDSR, psCTS, psRxD, psTxD, psRTS, psDTR : Boolean;

function PeekW( Segm, Offs : word ) : Word;
function RWPort( port : word ) : byte;
procedure CommInit;
procedure CheckCommPortStatusRegister;
procedure CheckCommPortModemControl;
procedure CheckCommPortLineStatusRegister;

implementation

function PeekW( Segm, Offs : word ) : Word;
{ Read a word from a memoy location... }
begin
   asm { Assembler (...I like it!) }
      mov es, Segm
      mov bx, Offs
      mov ax, [es:bx]
      mov Result, ax
   end;
end {PeekW};

{---------------------------------------------------------------------}

function RWPort( port : word ) : byte;
{ This function reads an I/O port and immediately writes the value back }
{ because reading the port clears some bytes. No interrupts may occur!  }
begin
   asm { Assembler (...I like it!) }
      mov dx, port
      cli
      in  al, dx  { Read... }
      out dx, al  { ... rewrite.  }
      sti
      mov Result, al
   end;
end {REPort};

{---------------------------------------------------------------------}

procedure CommInit;
{ This is done once. }
var
   pp : Word;
   bAny: boolean;

begin
   { Initialize some items... }
   Port := 0;
   bAny := FALSE;

   { Test for the presence of COM ports by scanning the BDA (Bios Data Area). }
   { I do not know if it is a good method, but it works... There is just one  }
   { problem: if the mouse is for instance on COM1, windows clears the Bios   }
   { entry... This does not disturb since we want to spy the modem, not the   }
   { little beast called a mouse!                                             }
   { Note: instead of PeekW, we could have used the PASCAL Ptr(...) function! }

   pp := PeekW($40, $0);  { COM1 BDA }
   if pp <> 0 then begin
      Port := 1;
      PortWord := pp;
      bAny := TRUE;
      end;

   pp := PeekW($40, $2);  { COM2 BDA }
   if pp <> 0 then begin
      Port := 2;
      if not bAny then begin
         PortWord := pp;
         bAny := TRUE;
         end;
      end;

   pp := PeekW($40, $4);  { COM3 BDA }
   if pp <> 0 then begin
      Port := 3;
      if not bAny then begin
         PortWord := pp;
         bAny := TRUE;
         end;
      end;

   pp := PeekW($40, $6);  { COM4 BDA }
   if pp <> 0 then begin
      Port := 4;
      if not bAny then begin
         PortWord := pp;
         bAny := TRUE;
         end;
      end;
 end {CommInit};

{---------------------------------------------------------------------}

procedure CheckCommPortStatusRegister;
var stat : byte;
begin
   { Test the Modem Status Register }
   stat := RWPort( PortWord + MODEMSTATUS );

   { Turn on/of the lamps }
   if (stat and RI) = RI then
      psRI := True
   else
      psRI := False;

   if (stat and CTS) = CTS then
      psCTS := True
   else
      psCTS := False;

   if (stat and DSR) = DSR then
      psDSR := True
   else
      psDSR := False;

   if (stat and DCD) = DCD then
      psDCD := True
   else
      psDCD := False;
end {CheckComMPortStatusRegister};

procedure CheckComMPortModemControl;
var stat : byte;
begin
  { Read the Modem Control Register }
   stat := RWPort( PortWord + MODEMCONTROL );

   { Turn on/of the lamps }
   if (stat and DTR) = DTR then
      psDTR := True
   else
      psDTR := False;

   if (stat and RTS) = RTS then
      psRTS := True
   else
      psRTS := False;
end {CheckComMPortModemControl};

procedure CheckComMPortLineStatusRegister;
var stat : byte;
begin
   { Read the Line Status Register }
   stat := RWPort( PortWord + LINESTATUS );

   if (stat and RXD) = RXD then
      psRXD := True
   else
      psRXD := False;

   if (stat and TXD) <> TXD then
      psTXD := True
   else
      psTXD := False;
end {CheckComMPortLineStatusRegister};

initialization
end { Unit CommSpy }.
