unit DbCurrEd;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Mask, DBCtrls, DB, DBTables, ClipBrd;

type
  TDBCurrencyEdit = class(TDBEdit)
  private
    FNumberOfDecimals: integer;
    FCurrency: Boolean;
    FMaxLength: integer;
    ValueJustEntered: string;
    InKeyPress: boolean;
    JustDropped: boolean;
    DistanceFromDecimal: integer;
    ForcedCursorPos: integer;
    DeletedCommas: integer;
    FAsFloat: double;
    JustEnteredWithMouse: boolean;
    FChange: TNotifyEvent;
    procedure UpdateData(Sender: TObject);           {responds to DataSource's UpdateData event}
    procedure SetNumberOfDecimals(Value: integer);
    procedure CMEnter(var Message: TCMGotFocus); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    procedure KeyPress(var Key: Char); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    function  ValidateCurrency: Boolean;
    function  StripCommas(TempString: string ): string;
    function  GetFormatString(NumberOfDecimals: integer): string;
    procedure Change; override;
    procedure GetCursorPosition;
    procedure SetCursorPosition;
    procedure CutText;
    procedure PasteText;
    function GetAsFloat: double;
    procedure SetAsFloat(Value: double);
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure FormatFieldText(Sender: TField; var Text: string; DisplayText: boolean);
  public
    constructor Create(AOwner: TComponent); override;
    property AsFloat: double read GetAsFloat write FAsFloat;
  published
    property Currency: Boolean read FCurrency write FCurrency;
    property MaxLength: integer read FMaxLength write FMaxLength;
    property NumberOfDecimals: integer read FNumberOfDecimals write SetNumberOfDecimals;
    property OnChange: TNotifyEvent read FChange write FChange;
  end;

procedure Register;

implementation

{uses DebugLog;}

const
  MaxCurrencyAmount: extended = 999999999;
  MaxCurrencyLength: integer = 16; {set for 999,999,999,999. + decimals}

constructor TDBCurrencyEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  JustDropped := True;
  Currency := True;
  FNumberOfDecimals := 2;
  ForcedCursorPos := -99;
  FMaxLength := MaxCurrencyLength + FNumberOfDecimals;
  if not (csDesigning in ComponentState) then
    if DataSource <> nil then
    begin
      DataSource.OnUpdateData := UpdateData;
      (DataSource.DataSet.FieldByName(DataField) as TFloatField).OnGetText := FormatFieldText;
    end;
{  CurrencyDecimals := NumberOfDecimals; }
end;

procedure TDBCurrencyEdit.WMPaint(var Message: TWMPaint);
begin
  inherited;
end;

procedure TDBCurrencyEdit.SetNumberOfDecimals(Value: integer);
begin
  if Value <> FNumberOfDecimals then
  begin
    FNumberOfDecimals := Value;
    FMaxLength := MaxCurrencyLength + FNumberOfDecimals;
    if csDesigning in ComponentState then
      begin
        if Field <> nil then
          EditText := FormatFloat( GetFormatString(NumberOfDecimals), StrToFloat(StripCommas(EditText)))
        else
          EditText := Name
      end
    else  {not designing}
       if EditText = '' then
        EditText := ''
       else
        begin
          EditText := FormatFloat( GetFormatString(FNumberOfDecimals), StrToFloat(StripCommas(EditText)));
{          (DataSource.DataSet.FieldByName(DataField) as TCurrencyField).Currency := True;
          CurrencyDecimals := NumberOfDecimals;  }
         (DataSource.DataSet.FieldByName(DataField) as TFloatField).DisplayFormat :=
            GetFormatString(NumberOfDecimals);
        end;
  end;
end;

procedure TDBCurrencyEdit.Change;
var
  OldSelStart: integer;
  TmpChange: TNotifyEvent;
begin

 TmpChange := OnChange;
 if @TmpChange <> nil then
   OnChange(Self);

 if not (csDesigning in ComponentState)  then
   OldSelStart := SelStart;
 if not InKeyPress then
 begin
  if Currency and (Field <> nil) and (Field.DataType = ftFloat) then
   begin
    if (EditText <> '') and  (ValueJustEntered <> '') then
      EditText := FormatFloat( GetFormatString(NumberOfDecimals), StrToFloat(StripCommas(EditText)));
    DataSource.OnUpdateData := UpdateData;
    DataSource.DataSet.FieldByName(DataField).OnGetText := FormatFieldText;
    end;
 end;
 if not (csDesigning in ComponentState) then
   SelStart := OldSelStart;

 if csDesigning in ComponentState then
   if Field <> nil then
     EditText := FormatFloat( GetFormatString(NumberOfDecimals), StrToFloat(StripCommas(Field.DisplayText)))
   else
     EditText := Name;
 if assigned(FChange) then OnChange(self);
end;

procedure TDBCurrencyEdit.KeyDown(var Key: Word; Shift: TShiftState);
var
  OldSelStart: integer;
begin

  if (ssShift in Shift) and (key = 45) then
  begin
   PasteText;    {paste}
   Key := 0;
  end;
  if (ssShift in Shift) and (key = 46) then
  begin
   CutText;    {cut}
   Key := 0;
  end;
  if (ssCtrl in Shift) and (key = 45) then
  begin
   CopyToClipboard;     {copy}
   Key := 0;
  end;

  if Key = 46 then  {delete}
  begin

  GetCursorPosition;

    if SelLength > 0 then
    begin
      ValueJustEntered := EditText;
      Delete(ValueJustEntered, SelStart + 1, SelLength);
      DataSource.Edit;
      EditText := ValueJustEntered;
      Key := 0;
    end
    else
    begin
      if (EditText[SelStart + 1] = ',') or
         (EditText[SelStart + 1] = '.') then
      begin
        ValueJustEntered := EditText;
        Delete(ValueJustEntered, SelStart + 2, 1);
        DataSource.Edit;
        EditText := FormatFloat( GetFormatString(NumberOfDecimals), StrToFloat(StripCommas(ValueJustEntered)));
        DistanceFromDecimal := DistanceFromDecimal - 2;
        Key := 0;
      end
      else
      begin
        ValueJustEntered := EditText;
        Delete(ValueJustEntered, SelStart + 1, 1);
        DataSource.Edit;
        EditText := FormatFloat( GetFormatString(NumberOfDecimals), StrToFloat(StripCommas(ValueJustEntered)));
        DistanceFromDecimal := DistanceFromDecimal - 1;
        Key := 0;
      end;

   {   inherited KeyDown(Key, Shift);          {call inherited delete}
      {SelStart := OldSelStart;}                {put cursor back}
    end;

  SetCursorPosition;

  end
  else
    inherited KeyDown(Key, Shift);   {for all other keys}

end;

procedure TDBCurrencyEdit.KeyPress(var Key: Char);
var
  InvalidKey: boolean;
  OldSelStart, InsertIndex,Offset, BeforeLen, AfterLen,
  OldSelLength: Integer;
  RightOfDecimal: String;
  AFloat: double;
begin
InKeyPress := True;

GetCursorPosition;

if Currency and (Field <> nil) and (Field.DataType = ftFloat) then
begin

  case Key of
    '0'..'9':    { 0-9 }
     begin
     ValueJustEntered := EditText;
     OldSelStart := SelStart;
     OldSelLength := SelLength;
     BeforeLen := Length(ValueJustEntered);
     if SelLength = Length(EditText) then
       begin
         Delete(ValueJustEntered, SelStart + 1, SelLength);
         Insert(Key, ValueJustEntered, SelStart + 1);
         ForcedCursorPos := 1;
       end
     else if SelLength > 0 then
       begin
          Delete(ValueJustEntered, SelStart + 1, SelLength);
          Insert(Key, ValueJustEntered, SelStart + 1);
       end
     else
       begin

         {editing to right of decimal}
         if DistanceFromDecimal < 1 then
         begin
           Insert(Key, ValueJustEntered, SelStart + 1);
           if Pos(DecimalSeparator, ValueJustEntered) > 0 then
            ValueJustEntered[Pos(DecimalSeparator, ValueJustEntered) +
               NumberOfDecimals + 1]  := '0'
           else
             if NumberOfDecimals > 0 then
             begin
                ValueJustEntered[Length(ValueJustEntered)] := '0';
                ForcedCursorPos := DistanceFromDecimal - 1;
             end;
         end;
         {  X.99   }
         if ((StrToFloat(StripCommas(EditText)) < 1) and (DistanceFromDecimal = 1)) then
         begin
           ValueJustEntered := FloatToStr( (StrToFloat(Key)) + (StrToFloat(StripCommas(EditText))) );
           Key := #0;
         end;

         if DistanceFromDecimal >= 1 then
          begin
           Insert(Key, ValueJustEntered, SelStart + 1);
          Key := #0;
          end;

     end;
     Key := #0;
     if not ValidateCurrency then
       MessageBeep(0)
     else
     begin
       DataSource.Edit;
       EditText := ValueJustEntered;
     end;
     AfterLen := Length(ValueJustEntered);
     {kludge to handle case when entering a new number}
     if ((AfterLen - BeforeLen) = 4) and (OldSelLength = 0) then
       SelStart := 1
     {kludge #2 to handle case when entering decimal portion}
{     else if  (AfterLen = BeforeLen) and (OldSelLength = 0) then
       ForcedCursorPos := 1  }
     else
       if (OldSelLength > 0) then
        SelStart := OldSelStart
       else
        SelStart := OldSelStart + (AfterLen - BeforeLen) - 1;

    end;
    ^H:                          {Backspace}
     begin
        OldSelStart := SelStart;
        ValueJustEntered := EditText;
        BeforeLen := Length(ValueJustEntered);
        if SelLength < 1 then
        begin
          if ValueJustEntered[SelStart] = ThousandSeparator then
          begin
            Delete(ValueJustEntered, SelStart, 1);    {then delete char before comma}
          end;
          if ValueJustEntered[SelStart] <> DecimalSeparator then
            Delete(ValueJustEntered, (SelStart), 1)        {then delete char before caret}
          else ForcedCursorPos := 1;              {put cursor to left of decimal}
          {OldSelStart := OldSelStart - (BeforeLen - AfterLen); }
          OldSelStart := OldSelStart - 1;
          if not ValidateCurrency then          {validate and format text}
            MessageBeep(0)
        end
        else
        begin
          Delete(ValueJustEntered, SelStart + 1, SelLength);
          OldSelStart := OldSelStart - 1;
          if not ValidateCurrency then          {validate and format text}
            MessageBeep(0)
        end;
        AfterLen := Length(ValueJustEntered);

        if EditText <> ValueJustEntered then
        begin
          DataSource.Edit;
          EditText := ValueJustEntered;
        end;

        SelStart := OldSelStart;

        Key := #0;
        SetCursorPosition;
        exit;
     end;
    #27:                         {Escape}
      begin
        DataSource.Edit;
        SelectAll;
        ClearSelection;
        Key := #0;
        exit;
      end;
    '.', ',':
      begin
        if Key = ThousandSeparator then
          Key := #0;
        if Key = DecimalSeparator then
        begin
          if NumberOfDecimals > 0 then
            SelStart := Pos(DecimalSeparator, EditText);
          Key := #0;
          exit;
        end;
      end;
    ^C:  {copy}
      CopyToClipboard;
    ^X:  {cut}
      begin
      CutText;
      Key := #0;
      end;
    ^V:  {paste}
      begin
      PasteText;
      Key := #0;
      end;
    else
      begin
        MessageBeep(0);
        Key := #0;
      end;
    Key := #0;
  end;

  InvalidKey := False;
    if Key = ThousandSeparator then
      InvalidKey := True;
    if (Key = DecimalSeparator) and (NumberOfDecimals < 1) then
      InvalidKey := True;
    if (Key = DecimalSeparator) and (Pos(DecimalSeparator, EditText) > 0) then
      InvalidKey := True;

  if InvalidKey then
  begin
    MessageBeep(0);
    Key := #0;
  end;
end
else
  inherited KeyPress(Key);
InKeyPress := False;

SetCursorPosition;
end;

function TDBCurrencyEdit.ValidateCurrency: Boolean;
var
  TempFloat: extended;
  WithOutCommas, FormatString: string;
begin
  try
   WithOutCommas := StripCommas(ValueJustEntered);
   if WithOutCommas = '' then
     TempFloat := 0
   else
   TempFloat := StrToFloat(WithOutCommas);
  except
   on EConvertError do
   begin
     Result := False;
     exit;
   end;
  end;
  FormatString := GetFormatString(NumberOfDecimals);
  ValueJustEntered := FormatFloat(FormatString, TempFloat);
  if (Length(ValueJustEntered) > FMaxLength) then
    Result := False
  else
    Result := True;
end;

function TDBCurrencyEdit.GetFormatString(NumberOfDecimals: integer): string;
begin
  case NumberOfDecimals of
  0: Result := '#,##0';
  1: Result := '#,##0.0';
  2: Result := '#,##0.00';
  3: Result := '#,##0.000';
  4: Result := '#,##0.0000';
  else
     Result := '#,##0.00000';
  end;
end;

function TDBCurrencyEdit.StripCommas(TempString: string ): string;
begin
  DeletedCommas := 0;
  if TempString = '' then
     Result := FormatFloat(GetFormatString(NumberOfDecimals), 0);
  while (Pos(ThousandSeparator,TempString) > 0) do
    begin
      if Pos(ThousandSeparator,TempString) > (DistanceFromDecimal - SelStart) then
        DeletedCommas := DeletedCommas + 1;
      Delete(TempString, Pos(ThousandSeparator,TempString), 1);
    end;
  Result := TempString;

end;

procedure TDBCurrencyEdit.UpdateData(Sender: TObject);
begin
  if (Field <> nil) and (Field.DataSet.State in dsEditModes) then
     Field.Text := StripCommas(EditText);
  JustEnteredWithMouse := False;
end;

procedure TDBCurrencyEdit.FormatFieldText(Sender: TField; var Text: string; DisplayText: boolean);
begin
  if Sender.AsString = '' then
    Text := ''
  else
    Text := FormatFloat( GetFormatString(FNumberOfDecimals), StrToFloat(StripCommas(Sender.AsString)));
end;

procedure TDBCurrencyEdit.CMExit(var Message: TCMExit);
begin
  if (Field <> nil) and (Field.DataSet.State in dsEditModes) then
     Field.Text := StripCommas(EditText);
  inherited;
  JustEnteredWithMouse := False;
end;

procedure TDBCurrencyEdit.CMEnter(var Message: TCMGotFocus);
var
  DecimalPos: integer;
begin
  inherited;
  if EditText <> '' then  {only format and set cursor pos if EditText has a value}
  begin
    EditText := FormatFloat( GetFormatString(FNumberOfDecimals), StrToFloat(StripCommas(EditText)));
    DecimalPos := Pos(DecimalSeparator, EditText);
    SelStart := 0;
    SelLength := Length(EditText);
  end;
end;

procedure TDBCurrencyEdit.GetCursorPosition;
var
  DecimalPos: integer;
begin
  DecimalPos := Pos(DecimalSeparator, EditText);
  if DecimalPos <= 0 then
    DecimalPos := Length(EditText);

  if SelLength > 0 then
    DistanceFromDecimal := DecimalPos - (SelStart + SelLength)
  else
    DistanceFromDecimal := DecimalPos - SelStart;

end;

procedure TDBCurrencyEdit.SetCursorPosition;
var
  DecimalPos: integer;
begin
  DecimalPos := Pos(DecimalSeparator, EditText);

  if ForcedCursorPos <> -99 then
  begin
    SelStart := DecimalPos - ForcedCursorPos;
    ForcedCursorPos := -99;
  end
  else
  begin
   if DecimalPos <= 0 then
     DecimalPos := Length(EditText);
   if DistanceFromDecimal = 0 then
     SelStart := DecimalPos + 1
   else
       SelStart :=  DecimalPos - DistanceFromDecimal;
  end;
end;

procedure TDBCurrencyEdit.CutText;
begin
  Clipboard.AsText := SelText;;
  ValueJustEntered := EditText;
  Delete(ValueJustEntered, SelStart + 1, SelLength);
  if ValidateCurrency then
  begin
    DataSource.Edit;
    EditText := ValueJustEntered;
  end;
end;


procedure TDBCurrencyEdit.PasteText;
begin
  ValueJustEntered := EditText;
  if SelLength > 0 then
  begin
    Delete(ValueJustEntered, SelStart + 1, SelLength);
  end;
  if StrToFloat( StripCommas(ValueJustEntered)) = 0 then
  begin
    ValueJustEntered := '';
  end;
  if Clipboard.HasFormat(CF_TEXT) then
    Insert(Clipboard.AsText, ValueJustEntered, SelStart + 1);
  if ValidateCurrency then
  begin
    DataSource.Edit;
    EditText := ValueJustEntered;
  end
  else
    MessageBeep(0);
end;

function TDBCurrencyEdit.GetAsFloat: double;
begin
  Result := StrToFloat(StripCommas(EditText));
end;

procedure TDBCurrencyEdit.SetAsFloat(Value: double);
begin
  if Value <> StrToFloat(Text) then
  begin
    FAsFloat := Value;
    EditText := FormatFloat( GetFormatString(NumberOfDecimals), Value);
  end;
end;

procedure TDBCurrencyEdit.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  DecimalPos: integer;
begin
  inherited MouseUp(Button, Shift, X, Y);
  if (Button = mbLeft) then
  begin
    if not JustEnteredWithMouse and (EditText <> '') then  {only format and set cursor pos if EditText has a value}
      begin
        EditText := FormatFloat( GetFormatString(FNumberOfDecimals), StrToFloat(StripCommas(EditText)));
        DecimalPos := Pos(DecimalSeparator, EditText);
        SelStart := DecimalPos -1 ;
        JustEnteredWithMouse := True;
      end;
  end;
end;

procedure Register;
begin
  RegisterComponents('Data Controls', [TDBCurrencyEdit]);
end;

end.
