unit Demounit;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Printers, ExtCtrls, Menus, DemoComp;

type
  TDemoForm = class(TForm)
    Preview: TImage;
    ShowChart: TButton;
    ShowPreview: TButton;
    PrintChart: TButton;
    grpCommands: TGroupBox;
    grpOptions: TGroupBox;
    rbnPortrait: TRadioButton;
    rbnLandscape: TRadioButton;
    edtLeft: TEdit;
    lblLeft: TLabel;
    lblTop: TLabel;
    edtTop: TEdit;
    lblWidth: TLabel;
    edtWidth: TEdit;
    lblHeight: TLabel;
    edtHeight: TEdit;
    Info: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ShowChartClick(Sender: TObject);
    procedure ShowPreviewClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure rbnPortraitClick(Sender: TObject);
    procedure rbnLandscapeClick(Sender: TObject);
    procedure PrintChartClick(Sender: TObject);
    procedure InfoClick(Sender: TObject);
    procedure edtLeftEnter(Sender: TObject);
    procedure edtLeftExit(Sender: TObject);
    procedure edtLeftKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    Demo: TDemo;
    hOldFont, hFont: hFont;
    PrevHeight, PrevWidth: longint;
    Running: boolean;
    OH, OW: Longint;
    Xs: array[1..40] of Longint;
    Ys: array[1..40] of Longint;
    Ws: array[1..40] of Longint;
    Hs: array[1..40] of Longint;
    procedure DrawPreview;
  public
    { Public declarations }
  end;

var
  DemoForm: TDemoForm;

implementation

{$R *.DFM}

procedure TDemoForm.FormCreate(Sender: TObject);
var i: integer;
begin

  { First create line chart component. }
  Demo := TDemo.Create(Self);
  Demo.Visible := false;
  Demo.Top := 0;
  Demo.Left := 0;
  Demo.Height := ClientHeight;
  Demo.Width := grpOptions.Left - 6;
  Demo.Title := 'Simple line chart';

  { Save original window size and components size and positions for
    later calculations. Warning: here's fixed array and it's supposed
    that no component is created or deleted at run-time. There is
    more elegant solutions but this is just for simplicity. }
  for i := 0 to ComponentCount - 1 do
    if Components[i] is TControl then begin
      Xs[i] := TControl(Components[i]).Left;
      Ys[i] := TControl(Components[i]).Top;
      Ws[i] := TControl(Components[i]).Width;
      Hs[i] := TControl(Components[i]).Height;
    end;

  { Scale window so that it's size compared to screen size is same
    as designed. Note that divisors here are your desing-time
    screen dimensions. }
  OH := ClientHeight;
  OW := ClientWidth;
  Height := Screen.Height * longint(443) div 480;
  Width := Screen.Width  * longint(612) div 640;

  { Default values for preview. }
  edtLeft.Text   := '400';
  edtWidth.Text  := '1100';
  edtTop.Text    := '500';
  edtHeight.Text := '700';

end;

procedure TDemoForm.FormDestroy(Sender: TObject);
begin
  Demo.Free;
  Demo := nil;
end;

procedure TDemoForm.ShowChartClick(Sender: TObject);
begin
  Info.Visible := false;
  Preview.Visible := false;
  Demo.Visible := true;
end;

procedure TDemoForm.ShowPreviewClick(Sender: TObject);
var iLeft, iTop, iWidth, iHeight, x0, x1, y0, y1: longint;
begin
  Info.Visible    := false;
  Demo.Visible    := false;
  Preview.Visible := true;
  DrawPreview;
end;

procedure TDemoForm.FormResize(Sender: TObject);
var i: integer;
    c: TControl;
begin

  { Create font that size is relative to window size and delete
    old font. }
  hFont := CreateFont(ClientHeight div 25, ClientWidth div 90, 0, 0,
                      FW_BOLD, 0, 0, 0, 1, OUT_DEVICE_PRECIS, 0,
                      PROOF_QUALITY, FF_DONTCARE, 'Arial');
  hOldFont := DemoForm.Font.Handle;
  DemoForm.Font.Handle := hFont;
  DeleteObject(hOldFont);

  { Here's actual component scaling. }
  for i := 0 to ComponentCount - 1 do begin
    if Components[i] is TControl then begin
      c := TControl(Components[i]);
      c.Left   := Xs[i] * Longint(ClientWidth) div OW;
      c.Top    := Ys[i] * Longint(ClientHeight) div OH;
      c.Width  := Ws[i] * Longint(ClientWidth) div OW;
      c.Height := Hs[i] * Longint(ClientHeight) div OH;
    end;
  end;

  { Finally save window dimensions and redraw preview if needed. }

  PrevWidth  := ClientWidth;
  PrevHeight := ClientHeight;
  if Preview.Visible then
    DrawPreview;
end;

procedure TDemoForm.rbnPortraitClick(Sender: TObject);
begin
  if Preview.Visible then
    DrawPreview;
end;

procedure TDemoForm.rbnLandscapeClick(Sender: TObject);
begin
  if Preview.Visible then
    DrawPreview;
end;

procedure TDemoForm.PrintChartClick(Sender: TObject);
var x0, x1, y0, y1: integer;
begin
  { Get position and size user entered and print the chart. }
  x0 := StrToInt(edtLeft.Text);
  x1 := StrToInt(edtWidth.Text);
  y0 := StrToInt(edtTop.Text);
  y1 := StrToInt(edtHeight.Text);
  if rbnPortrait.Checked then
    Printer.Orientation := poPortrait
  else
    Printer.Orientation := poLandscape;
  Printer.BeginDoc;
  Demo.PaintTo(Printer.Canvas, x0, y0, x1, y1);
  Printer.EndDoc;
end;

procedure TDemoForm.DrawPreview;
var iLeft, iTop, iWidth, iHeight, x0, x1, y0, y1: longint;
    PHeight, PWidth: longint;
    Region: hRgn;
    RetCode: integer;
begin
  { Here's calculation of "paper in screen" size and position.
    I think that here's too many lines code but I this just demo...}
  if Printer.Orientation = poLandscape then begin
    PHeight := Printer.PageWidth;
    PWidth := Printer.PageHeight
  end else begin
    PHeight := Printer.PageHeight;
    PWidth := Printer.PageWidth
  end;
  if rbnPortrait.Checked then begin
    iHeight := Preview.Height * longint(95) div 100;
    iWidth := iHeight * PWidth div PHeight;
    if iWidth > Preview.Width then begin
      iWidth := Preview.Width * longint(95) div 100;
      iHeight := iWidth * PHeight div PWidth;
    end;
    iLeft := (Preview.Width - iWidth) div 2;
    iTop := (Preview.Height - iHeight) div 2;
    x0 := iLeft + StrToInt(edtLeft.Text) * iWidth div PWidth;
    x1 := StrToInt(edtWidth.Text) * iWidth div PWidth;
    y0 := iTop + StrToInt(edtTop.Text) * iHeight div PHeight;
    y1 := StrToInt(edtHeight.Text) * iHeight div PHeight;
  end else begin
    iWidth := Preview.Width * longint(95) div 100;
    iHeight := iWidth * PWidth div PHeight;
    if iHeight > Preview.Height then begin
      iHeight := Preview.Height * longint(95) div 100;
      iWidth := iHeight * PHeight div PWidth;
    end;
    iLeft := (Preview.Width - iWidth) div 2;
    iTop := (Preview.Height - iHeight) div 2;
    x0 := iLeft + StrToInt(edtLeft.Text) * iWidth div PHeight;
    x1 := StrToInt(edtWidth.Text) * iWidth div PHeight;
    y0 := iTop + StrToInt(edtTop.Text) * iHeight div PWidth;
    y1 := StrToInt(edtHeight.Text) * iHeight div PWidth;
  end;
  with Preview.Picture.Bitmap do begin
    Width := Preview.Width;
    Height := Preview.Height;
    { First fill the preview image with silver. }
    Canvas.Brush.Color := clSilver;
    Canvas.Brush.Style := bsSolid;
    Canvas.Pen.Color := clSilver;
    Canvas.Rectangle(0,0,Width,Height);
    { Draw paper to it. }
    Canvas.Brush.Color := clWhite;
    Canvas.Pen.Color := clBlack;
    Canvas.Rectangle(iLeft - 1, iTop - 1, iLeft + iWidth + 1, iTop + iHeight + 1);
    { Finally draw the chart to "paper". }
    Demo.PaintTo(Canvas, x0, y0, x1, y1);
  end;
end;

procedure TDemoForm.InfoClick(Sender: TObject);
begin
  { Don't allow user input in memo. }
  ShowChart.SetFocus;
end;

procedure TDemoForm.edtLeftEnter(Sender: TObject);
begin
  { Save original value tag property. }
  try
    TEdit(Sender).Tag := StrToInt(TEdit(Sender).Text);
  except
  end;
end;

procedure TDemoForm.edtLeftExit(Sender: TObject);
var value: integer;
begin
  { Check that user is entered valid integer value in editbox,
    if not don't allow user to exit from editbox. }
  try
    value := StrToInt(TEdit(Sender).Text);
    DrawPreview;
  except
    MessageDlg('Invalid integer value.' + #13 +
               'Press Esc to restore original value.',
               mtInformation, [mbOk], 0);
    TEdit(Sender).SetFocus;
  end;
end;

procedure TDemoForm.edtLeftKeyPress(Sender: TObject; var Key: Char);
begin
  { Esc key in editboxes restores the value which was when
    use entered to editbox. }
  if Key = #27 then
    TEdit(Sender).Text := IntToStr(TEdit(Sender).Tag);
end;

end.
