unit FmxPlus;
{ unit di interfaccia alle funzioni su file}
interface
uses
  SysUtils,Classes,Forms,Controls;

type
  TFileLogFun = function(const FileName: string; const SRec: TSearchRec): boolean of Object;

  function ScanDir(const Dir:  string;
                   const Mask: string;
                         RecurseDir: Boolean;
                         DirLogFun:  TFileLogFun;
                         FileLogFun: TFileLogFun): integer;

implementation

function ScanDir(const Dir:  string;
                 const Mask: string;
                       RecurseDir: Boolean;
                       DirLogFun:  TFileLogFun;
                       FileLogFun: TFileLogFun): integer;

var
  DirFound,FileFound : integer; {found elements}
  SpecList: TStringList;
  DirPath: string;
  {
   Scan directory for files and subdirectories
  }
  function SearchDir(const Dir: string): boolean;
  var
    NewDir: String;
    I: integer;
    {
     Scan specified directory for files specified in Mask
     Dir must end with '\'
    }
    function FindFiles(const Dir: string;
                         const Mask: string): boolean;
    var
      SearchRec: TSearchRec;
      FindResult: integer;
    begin
      Result := true;
      FindResult := FindFirst(Dir+Mask, faArchive, SearchRec);
      while (FindResult = 0) do
      begin
         if not FileLogFun(Dir+SearchRec.Name,SearchRec) then
        begin
          Result := false; {force higher level to exit}
          Break; {exit loop}
        end;
        Inc(FileFound);
        FindResult := FindNext(SearchRec);
      end;
      FindClose(SearchRec) {Win32 required}
    end; {FindFiles}

    {
     Search specified directory for subdirectories,
     Dir must end with '\'
    }
    function FindSubdirs(const Dir: string): boolean;
    var
      SearchRec: TSearchRec;
      FindResult: integer;
      TmpDir: string;
    begin
      Result := true;
      FindResult := FindFirst(Dir+'*.*', faDirectory, SearchRec);
      while (FindResult = 0) do
      begin
        if (SearchRec.Attr and faDirectory <> 0) and
           (SearchRec.Name <> '.') and
           (SearchRec.Name <> '..') then
        begin
          TmpDir := Dir+LowerCase(SearchRec.Name)+'\';
          if not DirLogFun(TmpDir,SearchRec) then
          begin
            Result := false; {force higher level to exit}
            Break; {exit loop}
          end;
          {recursive scan of subdirectory}
          Result := SearchDir(TmpDir);
          if not Result then Break; {force exit}
        end;
        Inc(DirFound);
        FindResult := FindNext(SearchRec); {get next subdirectory name}
      end;
      FindClose(SearchRec) {Win32 required}
    end; {FindSubdirs}

  begin {SearchDir}
    Result := true;
    I := 0;
    while I < SpecList.Count do
    begin
      Result := FindFiles(Dir,SpecList[I]);
      inc(I);
      if not Result then Break;
    end;

    if RecurseDir and Result then
    begin
      Result := FindSubdirs(Dir);
    end;
  end;   {SearchDir}
  {
   Splits File Specification Mask
  }
  procedure SplitSpecs;
  var
    P: integer;
    TmpSpec: string;
  begin
    TmpSpec := Mask;
   {empty mask forced to '*.*'}
    if length(TmpSpec) = 0 then TmpSpec := '*.*';
    P := Pos(';',TmpSpec);
    while P > 0 do
    begin
      SpecList.Add(Copy(TmpSpec,0,P-1));
      TmpSpec := Copy(TmpSpec,P+1,Length(TmpSpec)-P);
      P := Pos(';',TmpSpec);
    end;
    SpecList.Add(TmpSpec); {add last file spec}
    FileFound := 0;
  end;

begin {ScanDir}
  Screen.Cursor := crHourGlass;
  try
    DirPath := Dir;
    {Ensure DirPath has final '\'}
    if DirPath[Length(DirPath)] <> '\' then
      DirPath := DirPath + '\';
    SpecList := TStringList.Create;
    try
      SplitSpecs;
      SearchDir(DirPath);
    finally
      SpecList.Destroy;
    end;
  finally
   Screen.Cursor := crDefault;
  end;
  Result := FileFound;
end; {ScanDir}


end.
