unit DpGrdBck;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms;

type
  TGradColor = (clAqua, clBlue, clGreen, clPurple, clRed, clWhite, clYellow) ;
  TGradDirection = (gdDown, gdUp) ;

  TGradientBack = class(TGraphicControl)
  private
    FBackColor : TGradColor ;
    FDirection : TGradDirection ;
    FBC: array[0..255] of Longint;       { precalculated Brush.Colors }
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message wm_EraseBkgnd;
    procedure SetColor(v : TGradColor) ;
    procedure SetDirection(v : TGradDirection) ;
    procedure LoadColors ;
  protected
    procedure Paint; override;
  public
    constructor Create(AComponent: TComponent); override;
    procedure Loaded; override;
  published
    property BackColor : TGradColor read FBackColor write SetColor
                                    default clBlue ;
    property Direction : TGradDirection read FDirection write SetDirection
                                        default gdDown ;
  end;

procedure Register;

implementation

constructor TGradientBack.Create(AComponent: TComponent);
begin
  Align := alClient;
  FBackColor := clBlue ;
  FDirection := gdDown ;
  LoadColors ;
  inherited Create(AComponent);
end;

procedure TGradientBack.LoadColors ;
var
  X, Y: Integer;
begin
     if FDirection = gdDown then
        Y := 255
     else
        Y := 0 ;

     for X := 0 to 255 do begin
        case FBackColor of
           clAqua   : FBC[X] := RGB( 0, Y, Y );
           clBlue   : FBC[X] := RGB( 0, 0, Y );
           clGreen  : FBC[X] := RGB( 0, Y, 0 );
           clPurple : FBC[X] := RGB( Y, 0, Y );
           clRed    : FBC[X] := RGB( Y, 0, 0 );
           clWhite  : FBC[X] := RGB( Y, Y, Y );
           clYellow : FBC[X] := RGB( Y, Y, 0 );
        end ;
        if FDirection = gdDown then
           Dec(Y)
        else
           Inc(Y) ;
     end;
end ;

procedure TGradientBack.SetColor(v : TGradColor) ;
begin
     FBackColor := v ;
     LoadColors ;
     Paint ;
end ;

procedure TGradientBack.SetDirection(v : TGradDirection) ;
begin
     FDirection:= v ;
     LoadColors ;
     Paint ;
end ;

procedure TGradientBack.Loaded;
begin
  inherited Loaded;
end;

procedure TGradientBack.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;   { do not allow Windows to erase it since it }
end;                     { is going to completely redraw itself anyway.}

procedure TGradientBack.Paint;
var
  TempRect   : TRect   ;
  TempStepV  : Single  ;
  ColorCode  : Integer ;
  TempTop    : Integer ;
  TempHeight : Integer ;
begin
  TempStepV := Height / 255;                     { Height of each color band }
  TempHeight := Trunc(TempStepV + 1);            { adjust up to ensure overlap }
  with Canvas do begin
     TempTop := 0;
     ColorCode := 0;
     TempRect.Left := 0;
     TempRect.Right:= Width;
     for ColorCode := 0 to 255 do begin
        Brush.Color := FBC[ColorCode];
        TempRect.Top  := TempTop;
        TempRect.Bottom := TempTop + TempHeight;
        FillRect(TempRect);
        TempTop := Trunc(TempStepV * ColorCode);
     end;
  end;
end;

procedure Register;
begin
  RegisterComponents('Custom', [TGradientBack]);
end;

end.
