unit Dstamp;
{
 TDateStamp V1.1 is a simple component, which you may find useful
 for easy version stamping. It delivers the creation date, time
 and a VersionCount of your project.

 You have to set your version labels in the OnShow-Event to the
 DateStamp properties:

 procedure TForm1.FormShow(Sender: TObject);
 begin
    Label1.Caption:=DateStamp1.Date;
    Label2.Caption:=DateStamp1.Time;
    Label3.Caption:=IntToStr(DateStamp1.VersionCount);
    Label4.Caption:=DateStamp1.GetVersion;
 end;

 The Date and Time properties are updated every time you open
 the form (or the project). It's not the compile date but better
 than the date of the executable. The latter is user changeable!

 It's a slight overkill to use the PropertyEditor only for
 setting the properties readonly. You can remove it if you want,
 but you will not have the special VersionCount behaviour.

 This is a curious thing. The property VersionCount is only
 incremented when the form or unit was changed! and not when the
 project was opened or closed or saved without any changes.
 This is exactly what I need, but I don't know why it works in
 this way.
 (I think there are some optimisations in saving of properties
 within delphi (reason: ReadOnly-PropertyEditor) that working
 for me :-)

 history:
 V1.1:  added Create-initialization, GetVersion, Version: string

 Please drop a short eMail to matthias@penthouse.boerde.de
 if you use it. I'm interested in how many people will implement
 it.
 freely distributable
 Public Domain
 Matthias Weingart, 12.12.95, matthias@penthouse.boerde.de
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DsgnIntf;

type
  TDateStamp = class(TComponent)
  private
    { Private-Deklarationen }
   FDate : string;
   FTime : string;
   FVersionCount: longint;
   FVersion : string;
  protected
    { Protected-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    procedure Loaded; override;
  public
    { Public-Deklarationen }
    function GetVersion: string;
  published
    { Published-Deklarationen }
       Property Version : string Read FVersion Write FVersion;
       Property Date : string  Read FDate  Write FDate;
       Property Time : string  Read FTime  Write FTime;
       Property VersionCount: longint Read FVersionCount Write FVersionCount;
  end;

procedure Register;

implementation

constructor TDateStamp.Create(AOwner: TComponent);
begin
     inherited Create( AOwner );
      if csDesigning in ComponentState then
      begin
          FDate := DateToStr( Now );
          FTime := TimeToStr( Now );
          FVersion := 'V1.0';
      end;
end;

procedure TDateStamp.Loaded;
begin
     inherited Loaded;
      if csDesigning in ComponentState then
      begin
          FVersionCount:= FVersionCount + 1;
          FDate := DateToStr( Now );
          FTime := TimeToStr( Now );
      end;
end;

function TDateStamp.GetVersion: string;
begin
     Result := Version +'.'+ IntToStr( VersionCount );
end;

{ Property Editors }
type
  TCountProperty = class(TIntegerProperty)
  public
        function GetAttributes: TPropertyAttributes; override;
  end;

function TCountProperty.GetAttributes: TPropertyAttributes;
begin
  	Result := [ paMultiSelect, paReadOnly ];
end;

type
  TDTProperty = class(TStringProperty)
  public
        function GetAttributes: TPropertyAttributes; override;
  end;

function TDTProperty.GetAttributes: TPropertyAttributes;
begin
  	Result := [ paMultiSelect, paReadOnly ];
end;

procedure Register;
begin
  RegisterPropertyEditor( TypeInfo( longint ), TDateStamp, 'VersionCount', TCountProperty );
  RegisterPropertyEditor( TypeInfo( string  ), TDateStamp, 'Date' , TDTProperty );
  RegisterPropertyEditor( TypeInfo( string  ), TDateStamp, 'Time' , TDTProperty );
  RegisterComponents('Samples', [TDateStamp]);
end;

end.
