{---------------------------------------------}
{ TEmail component for Borland Delphi         }
{                                             }
{ Written by Keith Darragh, (c) SWIFTsoft 1995}
{                                             }
{ Version : 1.00                              }
{                                             }
{ History :                                   }
{                                             }
{ 1.00 : Initial version                      }
{---------------------------------------------}

{$F+}

unit Email;

interface

uses Classes, WinTypes, WinProcs, SysUtils;

{------------------}
{ MAPI data types. }
{------------------}

type
  UINT      =  Integer;
  ULONG     =  LongInt;
  lpULONG   = ^LongInt;
  LHANDLE   =  LongInt;
  lpLHANDLE = ^LHANDLE;
  DWORD     =  Longint;

{-----------------------------------}
{ MAPI File attachment information. }
{-----------------------------------}

type
  lpMapiFileDesc = ^TMapiFileDesc;
  TMapiFileDesc = Record
    ulReserved : ULONG;                  { Reserved for future use. Must be 0 }
    flFlags : ULONG;  		         { Flags }
    nPosition : ULONG;                   { Attachment position in message body }
    lpszPathName : PChar;                { Full path name of attachment file }
    lpszFileName : PChar;                { Original filename (optional) }
    lpFileType : Pointer;                { Attachment file type (reserved) }
  end;

{-----------------------------}
{ MAPI recipient information. }
{-----------------------------}

type
  lpMapiRecipDesc = ^TMapiRecipDesc;
  TMapiRecipDesc = Record
    ulReserved : ULONG;                  { Reserved for future use. Must be 0 }
    ulRecipClass : ULONG;                { Recipient class - }
                                         { MAPI_TO, MAPI_CC, }
                                         { MAPI_BCC, MAPI_ORIG }
    lpszName : PChar;                    { Recipient name }
    lpszAddress : PChar;                 { Recipient address (optional) }
    ulEIDSize : ULONG;                   { Size (in bytes) of lpEntryID }
    lpEntryID : Pointer;                 { System-specific recipient reference }
  end;

{---------------------------}
{ MAPI message information. }
{---------------------------}

type
  lpMapiMessage = ^TMapiMessage;
  TMapiMessage = Record
    ulReserved : ULONG;                  { Reserved for future use. Must be 0 }
    lpszSubject : PChar;                 { Message subject }
    lpszNoteText : PChar;                { Message text }
    lpszMessageType : PChar;             { Message class }
    lpszDateReceived : PChar;            { In YYYY/MM/DD HH:MM format }
    lpszConversationID : PChar;		 { Conversation thread ID }
    flFlags : ULONG;                     { Unread, return receipt }
    lpOriginator : lpMapiRecipDesc;      { Originator descriptor }
    nRecipCount : ULONG;                 { Number of recipients }
    lpRecips : lpMapiRecipDesc;          { Recipient descriptors }
    nFileCount : ULONG;                  { Number of file attachments }
    lpFiles : lpMapiFileDesc;            { Attachment descriptors }
  end;

{-----------------------------}
{ MAPI constants information. }
{-----------------------------}

Const
  MAPI_ORIG                     =  0;    { Recipient is message originator. }
  MAPI_TO                       =  1;    { Recipient is primary recipient. }
  MAPI_CC                       =  2;    { Recipient is copy recipient. }
  MAPI_BCC                      =  3;    { Recipient is blind-copy recipient. }

  MAPI_UNREAD            :ULONG = $0001;
  MAPI_RECEIPT_REQUESTED :ULONG = $0002;
  MAPI_SENT              :ULONG = $0004;

  MAPI_LOGON_UI          :ULONG = $0001; { Display logon UI }
  MAPI_NEW_SESSION       :ULONG = $0002; { Don't get default if available. }
  MAPI_DIALOG            :ULONG = $0008; { Prompt to resolve ambig. names }
  MAPI_UNREAD_ONLY       :ULONG = $0020; { Only unread messages }
  MAPI_ENVELOPE_ONLY     :ULONG = $0040; { Only header information }
  MAPI_PEEK              :ULONG = $0080; { Don't mark message as read }
  MAPI_GUARANTEE_FIFO    :ULONG = $0100; { Guarantee FIFO MAPIFindNext }
  MAPI_BODY_AS_FILE      :ULONG = $0200; { Save body as first attachment }
  MAPI_AB_NOMODIFY       :ULONG = $0400; { Don't modify PAB entries }
  MAPI_SUPPRESS_ATTACH   :ULONG = $0800; { Header and body, no files }
  MAPI_FORCE_DOWNLOAD    :ULONG = $1000; { Force message download from server }

  MAPI_OLE               :ULONG = $0001; { Set if attachment is an OLE Object }
  MAPI_OLE_STATIC        :ULONG = $0002; { Set if attachment is Static OLE    }

{ MAPI Error codes. }

const
  SUCCESS_SUCCESS                 =  0;
  MAPI_USER_ABORT                 =  1;
  MAPI_E_FAILURE                  =  2;
  MAPI_E_LOGIN_FAILURE            =  3;
  MAPI_E_DISK_FULL                =  4;
  MAPI_E_INSUFFICIENT_MEMORY      =  5;
  MAPI_E_ACCESS_DENIED		  =  6;
  MAPI_E_TOO_MANY_SESSIONS        =  8;
  MAPI_E_TOO_MANY_FILES           =  9;
  MAPI_E_TOO_MANY_RECIPIENTS      = 10;
  MAPI_E_ATTACHMENT_NOT_FOUND     = 11;
  MAPI_E_ATTACHMENT_OPEN_FAILURE  = 12;
  MAPI_E_ATTACHMENT_WRITE_FAILURE = 13;
  MAPI_E_UNKNOWN_RECIPIENT        = 14;
  MAPI_E_BAD_RECIPTYPE            = 15;
  MAPI_E_NO_MESSAGES              = 16;
  MAPI_E_INVALID_MESSAGE          = 17;
  MAPI_E_TEXT_TOO_LARGE           = 18;
  MAPI_E_INVALID_SESSION	  = 19;
  MAPI_E_TYPE_NOT_SUPPORTED	  = 20;
  MAPI_E_AMBIGUOUS_RECIPIENT	  = 21;
  MAPI_E_MESSAGE_IN_USE		  = 22;
  MAPI_E_NETWORK_FAILURE	  = 23;
  MAPI_E_INVALID_EDITFIELDS	  = 24;
  MAPI_E_INVALID_RECIPS		  = 25;
  MAPI_E_NOT_SUPPORTED		  = 26;

{--------------------------}
{ Define TEmail component. }
{--------------------------}

type
  TEmail = class(TComponent)

  private
    hSession : ULONG;                            { MAPI Session number. }

    FAttachment : String;
    FRecip : String;
    FSubject : String;
    FText : String;
    FUserName : String;
    FUserPassword : String;

    procedure MyMessage(Text : String);                   { DEBUG only. }

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Logon: Integer;
    function Logoff: Integer;
    function SendMail: Integer;
    function CountUnread: ULONG;

  published
    property Attachment: string read FAttachment write FAttachment;
    property Recipient: string read FRecip write FRecip;
    property Subject: string read FSubject write FSubject;
    property Text: string read FText write FText;
    property UserName: String read FUserName write FUserName;
    property UserPassword: String read FUserPassword write FUserPassword;

  end;

procedure Register;

Implementation

{ Declare calls to MAPI.DLL }

Function MAPIFindNext(lhSession : LHANDLE;
                      ulUIParam : ULONG;
                      lpszMessageType : PChar;
                      lpszSeedMessageID : PChar;
                      flFlags : ULONG;
                      ulReserved : ULONG;
                      lpszMessageID : PChar) : ULONG; external 'MAPI';

Function MAPILogoff(lhSession : LHANDLE;
                    ulUIParam : ULONG;
                    flFlags : ULONG;
                    ulReserved : ULONG) : ULONG; external 'MAPI';

Function MAPILogon(ulUIParam : ULONG;
                   lpszName : PChar;
                   lpszPassword : PChar;
                   flFlags : ULONG;
                   ulReserved : ULONG;
                   lplhSession : LPHANDLE) : ULONG; external 'MAPI';

Function MAPISendMail(lhSession : LHANDLE;
                      ulUIParam : ULONG;
                      lpMessage : lpMapiMessage;
                      flFlags : ULONG;
                      ulReserved : ULONG) : ULONG; external 'MAPI';

{ Register the E-Mail component in the toolbar. }

procedure Register;
begin
  RegisterComponents('SWIFTsoft',[TEmail]);
end;

{ Create object. }

constructor TEmail.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  hSession := 0;
end;

{ Destroy object. }

destructor TEmail.Destroy;
begin
  If hSession <> 0 then
    Logoff;

  inherited Destroy;
end;

{ Logon to E-Mail system. }

function TEmail.Logon: Integer;
var
  szName : Array[0..256] of char;
  szPassword : Array[0..256] of char;
  flFlag : ULONG;

begin
  Result := SUCCESS_SUCCESS;

  { Check if already logged in. }

  If hSession = 0 then
  begin
    StrPCopy(szName,UserName);
    StrPCopy(szPassword,UserPassword);

    flFlag := MAPI_New_Session + MAPI_Logon_UI;
    Result := MAPILogon(0, @szName, @szPassword, flFlag, 0, @hSession);
  end;
end;

{ Logoff E-Mail system. }

function TEmail.Logoff: Integer;
begin
  Result := SUCCESS_SUCCESS;

  If hSession <> 0 then
  begin
    Result := MAPILogoff(hSession, 0, 0, 0);
    hSession := 0;
  end;
end;

{ Send Mail message. }

function TEmail.SendMail: Integer;
var
  MapiMessage   : TMapiMessage;
  MapiRecipDesc : TMapiRecipDesc;
  MapiFileDesc  : TMapiFileDesc;
  szRecipName   : Array[0..256] of char;
  szSubject     : Array[0..256] of char;
  szText        : Array[0..256] of char;
  szPathName    : Array[0..256] of char;
  flFlags       : ULONG;
  flLogoff      : Boolean;
begin
  { Auto Logoff, if no session active. }

  If hSession = 0 then flLogoff := True
  else                 flLogoff := False;

  { Logon to mail server. }

  If Logon <> SUCCESS_SUCCESS then exit;

  { Initialise MAPI structures. }

  FillChar(MapiMessage,   Sizeof(TMapiMessage),   0);
  FillChar(MapiRecipDesc, Sizeof(TMapiRecipDesc), 0);
  FillChar(MapiFileDesc,  Sizeof(TMapiFileDesc),  0);

  { Fill in subject & message text. }

  MapiMessage.lpszSubject  := StrPCopy(szSubject, FSubject);
  MapiMessage.lpszNoteText := StrPCopy(szText,    FText);

  flFlags := MAPI_DIALOG;

  If Length(FRecip) > 0 then
  begin
    flFlags := 0; { Don't display MAPI Dialog if recipient specified. }

    MapiRecipDesc.ulRecipClass := MAPI_TO;
    MapiRecipDesc.lpszName     := StrPCopy(szRecipName,FRecip);
    MapiMessage.nRecipCount    := 1;
    MapiMessage.lpRecips       := @MapiRecipDesc;
  end;

  { Process file attachments. }

  If Length(FAttachment) > 0 then
  begin
    MapiFileDesc.nPosition    := $ffffffff;  {Top of message. }
    MapiFileDesc.lpszPathName := StrPCopy(szPathName,FAttachment);
    MapiMessage.nFileCount    := 1;
    MapiMessage.lpFiles       := @MapiFileDesc;
  end;

  Result := MAPISendMail(hSession, 0, @MapiMessage, flFlags, 0);

  { Auto logoff, if no session was active. }

  if flLogoff = True then Logoff;
end;

{ Find Next message. }

function TEmail.CountUnread: ULONG;
var
  szMessageID : Array[0..256] of char;
  Count : ULONG;
  flLogoff : Boolean;
begin
  Count := 0;

  { Auto Logoff, if no session active. }

  If hSession = 0 then flLogoff := True
  else                 flLogoff := False;

  { Logon to mail server. }

  If Logon <> SUCCESS_SUCCESS then exit;

  { Start at first message. }

  FillChar(szMessageID,Sizeof(szMessageID),0);

  while MAPIFindNext(hSession,0,nil,@szMessageID,MAPI_UNREAD_ONLY,0,@szMessageID) = 0 do
     Count := Count + 1;

  { Auto logoff, if no session was active. }

  if flLogoff = True then Logoff;

  Result := Count;
end;

{ ---- MyMessage routine ---- }

procedure TEmail.MyMessage(Text : String);
var
  txt : array[0..79] of char;
  cap : array[0..79] of char;
begin
  MessageBox(0, StrPCopy(Txt, Text), StrPCopy(Cap, 'SWIFT E-Mail'), 0);
end;

end.
