program ExDelphi;
  {-executes Delphi, minimizing all apps currently running}
  {-then restores the desktop after you exit Delphi}
  {(c) Copyright 1995-1996, Eminent Domain Software}

{$R *.RES}
uses
  WinProcs, WinTypes, Messages;

type
  PMinAppInfo = ^TMinAppInfo;
  TMinAppInfo = record        {info for minimized app}
    Handle:     HWnd;         {window handle to be restored}
    RestoreMsg: Longint;      {message to restore window}
    Prev:       PMinAppInfo;  {pointer to prior window}
  end;  { TAppInfo }

var
  LastMinApp: PMinAppInfo;    {last app minimized}

    procedure MinimizeWindow (var WinHandle: HWnd);
      {-minimizes the specified window}
    var
      NewApp: PMinAppInfo;
    begin
      {minimize the window}
      SendMessage (WinHandle, WM_SYSCOMMAND, SC_MINIMIZE, 0);
      {create a new entry in the minimized app list}
      New (NewApp);
      NewApp^.Handle := WinHandle;
      if IsZoomed (WinHandle)
        then NewApp^.RestoreMsg := SC_MAXIMIZE  {to restore, maximize window}
        else NewApp^.RestoreMsg := SC_RESTORE;  {just restore window}
      NewApp^.Prev := LastMinApp;
      LastMinApp := NewApp;
    end;  { MinimizeWindow }

  function EnumProc (WinHandle: HWnd; Param: LongInt): Boolean; far;
    {-call back function for EnumWindows}
    {see EnumWindowsProc() in WINAPI.HLP for more information}
  var
    WindowTitle: array[0..255] of char;
  begin
    {check to see if the current window needs to be minimized}
    GetWindowText (WinHandle, @WindowTitle, 255);
    if (WindowTitle[0]<>#0) and            {if  window has a title,}
       (GetParent (WinHandle) = 0) and     {    is not a child window,}
       (not IsIconic (WinHandle)) and      {    is not already minimized,}
       (IsWindowVisible (WinHandle)) then  {and is visible}
    begin
      MinimizeWindow (WinHandle);
      EnumProc := FALSE;
    end {:} else
      EnumProc := TRUE;
  end; { EnumProc }

procedure CleanDesktop;
  {-minimizes all windows on the desktop}
var
  EnumAddress: TFarProc;
begin
  EnumAddress := MakeProcInstance (@EnumProc, 0);
  repeat
    {until all windows have been minimized}
  until EnumWindows (EnumAddress, 0);
  FreeProcInstance (EnumAddress);
end;  { CleanDesktop }

procedure ExecuteDelphi;
  {-executes Delphi and then waits until Delphi exits}
var
  DelphiID:    THandle;
  ExResultSt:  string;
  Msg:         TMsg;
begin
  DelphiID := WinExec ('Delphi.EXE', SW_SHOW);   {Execute Delphi}
  if DelphiID < 32 then
  begin
    Str (DelphiID, ExResultSt);
    ExResultSt := 'Error Loading Delphi : ' + ExResultSt + #0;
    MessageBox (0, @EXResultSt[1], 'EDS ExDelphi Loader',
                mb_OK or mb_IconInformation);
  end;  { if... }
  {Wait until Delphi's finished}
  repeat
    while PeekMessage (Msg, 0, 0, 0, PM_REMOVE) do  {get a message from the message queue}
    begin
      if Msg.Message <> WM_QUIT then  {message not for me}
      begin
        TranslateMessage (Msg);       {send message for other apps to process}
        DispatchMessage (Msg);
      end {:} else
        Halt (Msg.wParam);            {quit processing}
    end;  { while }
  until GetModuleUsage (DelphiID) = 0;
end;  { ExecuteDelphi }

procedure RestoreDesktop;
  {-restores all windows in our minimized window list}
var
  PrevApp: PMinAppInfo;
begin
  while LastMinApp <> nil do
  begin
    PrevApp := LastMinApp^.Prev;           {point to previous window in list}
    if IsWindow (LastMinApp^.Handle) and   {if window is still valid}
       IsIconic (LastMinApp^.Handle) then  {and is still minimized}
    begin
      {restore window as before}
      SendMessage (LastMinApp^.Handle, WM_SYSCOMMAND,
                   LastMinApp^.RestoreMsg, 0);
    end;  { if... }
    Dispose (LastMinApp);                  {dispose entry in list}
    LastMinApp := PrevApp;                 {now do previous window}
  end;  { while }
end;  { RestoreDesktop }

begin
  LastMinApp := nil;
  CleanDesktop;
  ExecuteDelphi;
  RestoreDesktop;
end.  { ExDelphi }
