{-------------------------------------------------------------------}
{ EXTFILE - Extended FileListbox: shows size, date & time of files  }
{ v. 1.01 November, 5 1995                                          }
{-------------------------------------------------------------------}
{ Copyright Enrico Lodolo                                           }
{ via F.Bolognese 27/3 - 440129 Bologna - Italy                     }
{ CIS 100275,1255 - Internet ldlc18k1@bo.nettuno.it                 }
{ modified by Joseph Glosz - September, 22 1995 :                   }
{  - more control over positioning                                  }
{  - more generic width calcs for date & time                       }
{ bug fix by Enrico Lodolo - October, 30 1995                       }
{  - no more errors when ftDirectory is set in FileType property    }
{-------------------------------------------------------------------}

unit ExtFile;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, FileCtrl;

type
  OffsetType = (ScreenEdge,LeftEdge,Relative);
  TExtFileListBox = class(TFileListBox)
  private
    FShowSize:boolean;
    FShowDate:boolean;
    FShowTime:boolean;
    FSizePos:integer;
    FDatePos:integer;
    FTimePos:integer;
    FNameWd:integer;
    FSizeWd:integer;
    FDateWd:integer;
    FTimeWd:integer;
    FSeparator: boolean;
    FSizeOffset: OffsetType;
    FDateOffset: OffsetType;
    FTimeOffset: OffsetType;
    procedure SetShowSize(Value: boolean);
    procedure SetShowDate(Value: boolean);
    procedure SetShowTime(Value: boolean);
    procedure SetSizePos(Value: integer);
    procedure SetDatePos(Value: integer);
    procedure SetTimePos(Value: integer);
    procedure SetSeparator(Value: boolean);
    procedure SetSizeOffset(Value: OffsetType);
    procedure SetDateOffset(Value: OffsetType);
    procedure SetTimeOffset(Value: OffsetType);
  protected
    procedure SetWidths;
    procedure DrawItem(Index:integer;Rect:TRect;State:TOwnerDrawState); override;
  public
    constructor Create(AOwner:TComponent); override;
  published
    property ShowSize:boolean read FShowSize write SetShowSize default True;
    property ShowDate:boolean read FShowDate write SetShowDate default True;
    property ShowTime:boolean read FShowTime write SetShowTime default True;
    property SizePos:integer read FSizePos write SetSizePos default 0;
    property DatePos:integer read FDatePos write SetDatePos default 5;
    property TimePos:integer read FTimePos write SetTimePos default 5;
    property Separator: boolean
      read FSeparator
      write SetSeparator
      default true;
    property SizeOffset: OffsetType read FSizeOffset write SetSizeOffset default Relative;
    property DateOffset: OffsetType read FDateOffset write SetDateOffset default Relative;
    property TimeOffset: OffsetType read FTimeOffset write SetTimeOffset default Relative;
  end;

procedure Register;

implementation

procedure TExtFileListBox.SetShowSize(Value:boolean);
begin
  if Value<>FShowSize then
    begin
    FShowSize:=Value;
    Update;
    end;
end;

procedure TExtFileListBox.SetShowDate(Value:boolean);
begin
  if Value<>FShowDate then
    begin
    FShowDate:=Value;
    Update;
    end;
end;

procedure TExtFileListBox.SetSeparator(Value: boolean);
begin
  if Value<>FSeparator then
    begin
    FSeparator := Value;
    Update;
    end;
end;

procedure TExtFileListBox.SetSizeOffset(Value: OffsetType);
begin
  if Value<>FSizeOffset then
    begin
    FSizeOffset := Value;
    Update;
    end;
end;

procedure TExtFileListBox.SetDateOffset(Value: OffsetType);
begin
  if Value<>FDateOffset then
    begin
    FDateOffset := Value;
    Update;
    end;
end;
procedure TExtFileListBox.SetTimeOffset(Value: OffsetType);
begin
  if Value<>FTimeOffset then
    begin
    FTimeOffset := Value;
    Update;
    end;
end;

procedure TExtFileListBox.SetShowTime(Value: boolean);
begin
  if Value<>FShowTime then
    begin
    FShowTime:=Value;
    Update;
    end;
end;

procedure TExtFileListBox.SetSizePos(Value: integer);
begin
  if Value<>FSizePos then
    begin
    FSizePos:=Value;
    Update;
    end;
end;

procedure TExtFileListBox.SetDatePos(Value: integer);
begin
  if Value<>FDatePos then
    begin
    FDatePos:=Value;
    Update;
    end;
end;

procedure TExtFileListBox.SetTimePos(Value: integer);
begin
  if Value<>FTimePos then
    begin
    FTimePos:=Value;
    Update;
    end;
end;

constructor TExtFileListBox.Create(AOwner: TComponent);
{-- Creates the component and sets the defaults }
begin
  inherited Create(AOwner);
  FShowSize := true;
  FShowDate := true;
  FShowTime := true;
  FSeparator := true;
  FSizeOffset := Relative;
  FTimeOffset := Relative;
  FDateOffset := Relative;
  FSizePos := 0;
  FDatePos := 5;
  FTimePos := 5;
end;

procedure TExtFileListBox.SetWidths;
{-- Calculates the max. widths of name, size, date and time }
begin
  with Canvas do
    begin
    FNameWd:=TextWidth('XXXXXXXX.XXX');
    if FSeparator
      then FSizeWd:=TextWidth('888888888')
      else FSizeWd:=TextWidth('888,888,888');

    {width is calculated accordingly to user's own system's date & time formats}
    FDateWd:=TextWidth(DateToStr(Now));
    FTimeWd:=TextWidth(TimeToStr(Now));
    end;
end;

procedure TExtFileListBox.DrawItem(Index: integer;
                                   Rect: TRect;
                                   State: TOwnerDrawState);
{-- Draws a line of the ListBox }
var Bitmap: TBitmap;
    Offset: integer;
    OldAlign: Word;
    SR: TSearchRec;
    DT: TDateTime;
    St: string;
    IsDir: Boolean;

function SetOffset(xOffset: OffsetType;
                   SizePos,LastOffset,ItemWidth: integer): integer;
begin
  case xOffset of
    ScreenEdge:  result := SizePos;
    LeftEdge:    result := Rect.Left + SizePos;
    Relative:    result := LastOffset+ItemWidth+SizePos;  {can be neg or pos}
  end;
end;



begin
  inherited DrawItem(Index,Rect,State);
  with Canvas do
    begin
    SetWidths;
    Offset:=Rect.Left+FNameWd;
    if (FShowGlyphs=True) and (Bitmap<>nil) then
      begin
      Bitmap:=TBitmap(Items.Objects[Index]);
      Offset:=Offset+Bitmap.Width+6;
      end;
    {-- Retrieves Size, Date and Time of the current file }
    if Directory[Length(Directory)]='\'
      then St:=Directory+Items[Index]
      else St:=Directory+'\'+Items[Index];
    {-- Handles directories ------------------}
    IsDir:=Items[Index][1]='[';
    if not IsDir then
      begin
        FindFirst(St,faAnyFile,SR);
        try
          DT:=FileDateToDateTime(SR.Time);
        except
          DT:=0;
        end;
      end;
    OldAlign:=SetTextAlign(Handle,ta_right);  {right alignment of text}
    if FShowSize then
      begin
      Offset := SetOffset(SizeOffset,FSizePos,Offset,FSizeWd);
      if not IsDir then
        if Separator
          then TextOut(Offset,Rect.Top,FormatFloat('###,###,###',SR.Size))
          else TextOut(Offset,Rect.Top,IntToStr(SR.Size));
      end;
    if FShowDate then
      begin
      Offset := SetOffset(DateOffset,FDatePos,Offset,FDateWd);
      try
        if not IsDir then
          TextOut(Offset,Rect.Top,DateToStr(DT));
      finally
      end;
      end;
    if FShowTime then
      begin
      Offset := SetOffset(TimeOffset,FTimePos,Offset,FTimeWd);
      try
        if not IsDir then
          TextOut(Offset,Rect.Top,TimeToStr(DT));
      finally
      end;
      end;
    {-- Sets the text alignment to the original value }
    SetTextAlign(Handle,OldAlign);
    end;
end;

procedure Register;

begin
  RegisterComponents('Samples', [TExtFileListBox]);
end;

end.
