{
 Designer: Craig Ward, 100554.2072@compuserve.com
 Date:     18/1/96
 Version:  2.0

 Function: Generic Filter Dialog. Sets\Remove filters on tables.

 Notes:    This dialog is set to work on TQuerys (from Woll2Woll software).

           The dialog works in several steps, manipulating an array. This array is
           the key to the dialog, since it stores the tables that the calling form
           is a host to. Manipulation of filters on tables, must manipulate the information
           stored in the Filter property of the tables stored in the array.

            [1] read the calling form, populating the table array
            [2] populate the combos (using the first array)
            [3] populate the list box (using the table array)
            [4] allow the user to set or remove filters
            [5] saving filter info into the array

            (the next step would be for the calling form to re-activate the
             filters on its tables)

           The dialog will work on up to iTabArray tables on the calling form.
           Increase or decrease this value accordingly.

********************************************************************************}
unit Filter;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, SysUtils, Dialogs, Menus, Messages, ExtCtrls, DB, DBTables,
  Cwlist, Print, Query;

type
  TFilterDlg = class(TForm)
    Bevel1: TBevel;
    Label1: TLabel;
    Label2: TLabel;
    comboFields: TComboBox;
    editFilter: TEdit;
    Label3: TLabel;
    comboConditional: TComboBox;
    btnFilter: TButton;
    btnCancel: TButton;
    btnHelp: TButton;
    SpeedButton1: TSpeedButton;
    Label4: TLabel;
    comboTab: TComboBox;
    SpeedButton2: TSpeedButton;
    Label5: TLabel;
    Bevel2: TBevel;
    SpeedButton3: TSpeedButton;
    listFilters: TcwListBox;
    procedure HelpBtnClick(Sender: TObject);
    procedure btnFilterClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure comboTabChange(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure editFilterKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    procedure SetComboField;
  public
    { Public declarations }
    FSQL: TStringList;
    procedure SetComboTab;
    procedure SetListBox;
    function CustShowModal: TModalResult;  {returns modal result}
  end;

var
  FilterDlg: TFilterDlg;

const
 iTabArray = 20;                                          {stores size of array}


{*******************************************************************************}

implementation

{$R *.DFM}


{***form's preferences*********************************************************}

{on key down}
procedure TFilterDlg.editFilterKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 {add filter if return key}
 if key = VK_RETURN then SpeedButton2Click(sender);
end;

{on create}
procedure TFilterDlg.FormCreate(Sender: TObject);
begin
 FSQL := TStringList.create;
end;

{on destroy}
procedure TFilterDlg.FormDestroy(Sender: TObject);
begin
 FSQL.free;
end;


{***buttons*********************************************************************}

{on change, set field combo}
procedure TFilterDlg.comboTabChange(Sender: TObject);
begin
 setComboField;
end;

{add filter}
procedure TFilterDlg.SpeedButton2Click(Sender: TObject);
begin
 if editFilter.text = '' then exit;
 {add text to list box - adding quotes to filter expression in case of string fields}
 if QueryDlg.queryPrint.FieldByName(comboFields.text).dataType = ftString then
  listFilters.Items.Add(PrintDlg.FTable+'."'+ comboFields.text +'" '+comboConditional.text +' "'+ editFilter.text +'"')
 else
  listFilters.Items.Add(PrintDlg.FTable+'."'+ comboFields.text +'" '+comboConditional.text +' '+ editFilter.text);

end;

{delete all filters}
procedure TFilterDlg.SpeedButton3Click(Sender: TObject);
begin
 listFilters.clear;
end;

{delete filter}
procedure TFilterDlg.SpeedButton1Click(Sender: TObject);
begin
 listFilters.DeleteSelected;
end;


{help}
procedure TFilterDlg.HelpBtnClick(Sender: TObject);
begin
 Application.HelpCommand(HELP_CONTEXT,FilterDlg.helpContext);
end;


{ok}
procedure TFilterDlg.btnFilterClick(Sender: TObject);
var
 i: integer;
begin

 FSQL.clear;

 {add to string-list the list-box items}
 if listFilters.items.count > 0 then
 for i := 0 to (listFilters.Items.count -1) do
  begin
   if i <> (listFilters.Items.Count -1) then
    FSQL.Add(listFilters.Items.Strings[i]+' AND ')
   else
    FSQL.Add(listFilters.Items.Strings[i]);
  end;

 listFilters.clear;
 editFilter.clear;

 modalResult := mrOK;

end;

{cancel dialog}
procedure TFilterDlg.btnCancelClick(Sender: TObject);
begin
 modalResult := mrCancel;
end;


{***custom routines*************************************************************}

{set table combo}
procedure TFilterDlg.SetComboTab;
var
 iInc: integer;
begin

 comboTab.clear;

 comboTab.Items.Add(PrintDlg.FTable);

 comboTab.itemIndex := 0;
 setComboField;

end;

{set field combo}
procedure TFilterDlg.SetComboField;
var
 iTab, iFld: integer;
begin

 comboFields.clear;

 for iFld := 0 to (QueryDlg.QueryPrint.FieldCount -1) do
  begin
   comboFields.items.Add(QueryDlg.QueryPrint.Fields[iFld].FieldName);
  end;

 comboFields.itemIndex := 0;

end;

{show modal}
function TFilterDlg.CustShowModal: TModalResult;
begin
 result := showModal;
end;

{set list box}
procedure TFilterDlg.SetListBox;
var
 i: integer;
begin

 listFilters.clear;

 for i := 0 to (FSQL.count -1) do
  begin
   listFilters.items.Add(FSQL[i]);
  end;

end;




{}
end.
