unit BtnWin;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, AtAbout, ExtCtrls, StdCtrls, FileCtrl, Spin,
  IniFiles,BPOption;

type
  TButtonsForm = class(TForm)
    SpeedPanel: TPanel;
    SBtnInfo: TSpeedButton;
    SBtnExit: TSpeedButton;
    ButtonsScrollBox: TScrollBox;
    StatusPanel: TPanel;
    StatusDirPanel: TPanel;
    Label1: TLabel;
    StatusDirLabel: TLabel;
    StatusFilePanel: TPanel;
    StatusFileLabel: TLabel;
    SBtnOptions: TSpeedButton;
    procedure SBtnInfoClick(Sender: TObject);
    procedure SBtnExitClick(Sender: TObject);
    procedure ButtonsPerRowChange(Sender: TObject);
    procedure ButtonSizeChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SBtnOptionsClick(Sender: TObject);
  private
    BtnSize :integer;
    BtnPerRow :integer;
    { Private declarations }
  public
    procedure ButtonsProc(Sender: TObject);
    procedure CreateButtons (FileListBox :TFileListBox);
    procedure ClearButtons;
    procedure ResizeButtons;
    { Public declarations }
  end;

var
  ButtonsForm: TButtonsForm;

implementation

{$R *.DFM}

procedure TButtonsForm.SBtnInfoClick(Sender: TObject);
begin
  AmlTabbedAbout := TAmlTabbedAbout.Create(Application);
  try
    AmlTabbedAbout.ShowModal;;
  finally
    AmlTabbedAbout.Free;
  end;
end;

procedure TButtonsForm.SBtnExitClick(Sender: TObject);
begin
  Close;
end;

procedure TButtonsForm.ClearButtons;
var i :integer;
begin
  ButtonsScrollBox.Free;
  ButtonsScrollBox := TScrollBox.Create(ButtonsForm);
  with ButtonsScrollBox do
  begin
    Parent := ButtonsForm;
    Align := alClient;
    AutoScroll := true;
  end;
end;

procedure TButtonsForm.ResizeButtons;
var
  i : integer;
begin
  with ButtonsScrollBox do
  begin
    HorzScrollBar.Position := 0;
    VertScrollBar.Position := 0;
    for i := 0 to ComponentCount - 1 do
      with (Components[i] as TSpeedButton) do
      begin
        Width :=  BtnSize;
        Height :=  BtnSize;
        Top  := Height * (i div BtnPerRow);
        Left := Width  * (i mod BtnPerRow);
      end;
  end;
end;

procedure TButtonsForm.ButtonsProc(Sender: TObject);
begin
  StatusFileLabel.Caption := (Sender as TSpeedButton).Hint;
end;

procedure TButtonsForm.CreateButtons(FileListBox :TFileListBox);
var
  i,x,y :integer;
  GlyphBtn :TSpeedButton;
  TmpBmp :TBitmap;
  OldCursor :TCursor;
begin
  StatusDirLabel.Caption := FileListBox.Directory;
  StatusFileLabel.Caption := '';
  OldCursor := Screen.Cursor;
  Screen.Cursor := crHourglass;
  try
    TmpBmp := TBitmap.Create;
    for i := 0 to FileListBox.Items.Count - 1 do
    begin
      GlyphBtn := TSpeedButton.Create(ButtonsScrollBox);
      with GlyphBtn do
      begin
        Parent := ButtonsScrollBox;
        Width :=  BtnSize;
        Height :=  BtnSize;
        Top  := Height * (i div BtnPerRow);
        Left := Width  * (i mod BtnPerRow);
        TmpBmp.LoadFromFile(FileListBox.Items[i]);
        Glyph := TmpBmp;
        Hint := ExtractFileName(FileListBox.Items[i]);
        ShowHint := true;
        OnClick := ButtonsProc;
      end;
    end;
  finally
    TmpBmp.Free;
    Screen.Cursor := OldCursor;
  end;
end;

procedure TButtonsForm.ButtonsPerRowChange(Sender: TObject);
begin
  ResizeButtons;
end;

procedure TButtonsForm.ButtonSizeChange(Sender: TObject);
begin
  ResizeButtons;
end;

procedure TButtonsForm.FormCreate(Sender: TObject);
var
  DelphiIni : TIniFile;
begin
  DelphiIni  := TIniFile.Create('delphi.ini');
  BtnSize := DelphiIni.ReadInteger('AmlBmpSelector.BtnPalette','BtnSize',25);
  BtnPerRow := DelphiIni.ReadInteger('AmlBmpSelector.BtnPalette','BtnPerRow',20);
  DelphiIni.Free;
end;

procedure TButtonsForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
var
  DelphiIni : TIniFile;
begin
  DelphiIni  := TIniFile.Create('delphi.ini');
  DelphiIni.WriteInteger('AmlBmpSelector.BtnPalette','BtnSize',BtnSize);
  DelphiIni.WriteInteger('AmlBmpSelector.BtnPalette','BtnPerRow',BtnPerRow);
  DelphiIni.Free;
end;
procedure TButtonsForm.SBtnOptionsClick(Sender: TObject);
begin
  BtnPalOptions := TBtnPalOptions.Create(Application);
  try
    with BtnPalOptions do
    begin
      ButtonSize.Value := BtnSize;
      ButtonsPerRow.Value := BtnPerRow;
      if ShowModal = mrOK then
      begin
        BtnSize := ButtonSize.Value;
        BtnPerRow := ButtonsPerRow.Value;
        ResizeButtons;
      end;
    end;
  finally
    BtnPalOptions.Free;
  end;
end;

end.
