{*
HelpContext Property Editor
Version 1.0
Developed by Roger.Karas@fnb.sprint.com
*}

unit Helplst;
interface
uses Classes, dsgnIntf, SysUtils, Dialogs, Controls;

type
  THelpContextEditor = Class(TPropertyEditor)
  public
  	function  GetAttributes : TPropertyAttributes; override;
    function  GetValue : String; override;
		procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(Const Value : string); override;
  end;

	procedure Register;
	function RemoveWhiteSpace(str : string) : string;

implementation

{******************************************************************************}
procedure Register;
begin
  RegisterPropertyEditor( TypeInfo( THelpContext), nil, '', THelpContextEditor);
end;

{******************************************************************************}
function THelpContextEditor.GetAttributes : TPropertyAttributes;
begin
	Result := [paValueList, paMultiSelect];
end;

{******************************************************************************}
function THelpContextEditor.GetValue : string;
begin
	result := IntToStr(GetOrdValue);
end;

{******************************************************************************}
procedure THelpContextEditor.GetValues(Proc: TGetStrProc);
var
	i: integer;
  f : TextFile;
  blnInHelpSection : Boolean;
  strLine : String;
  retVal : word;
begin
	{Open a file, helpid.txt.  If it can not be opened, ask the user if he/she
  wants to create a sample helpid.txt file.}
	try
	AssignFile(f, 'helpid.txt');
 	Reset(f);
  except
  	on e:exception do
    begin
    	retVal := MessageDlg('File helpid.txt could not be used.' + #13 + #13 +
      'Reason: ' + e.Message + #13 + #13 +
      'Do you want to create a sample helpid.txt file?',
      mtError, [mbYes, mbNo], 0);

    	if retVal = mrYes then
      begin
      	AssignFile(f, 'helpId.txt');
			 	Rewrite(f);
			  Writeln(f, '{HELP CONSTANTS BEGIN}');
			  Writeln(f, 'HELP_Contents=1');
			  Writeln(f, 'HELP_Glossary=2');
			  Writeln(f, 'HELP_TechSupport=3');
			  Writeln(f, '{HELP CONSTANTS END}');
        CloseFile(f);
        Reset(f);
      end
      else
      	exit;
		end;
  end;

  {Read through the file one line at a time, looking for help identifers between
  the two comment lines: HELP CONSTANTS BEGIN and HELP CONSTANTS END}
 	blnInHelpSection := False;
  while (not Eof(f)) and (not blnInHelpSection) do
  begin
  	ReadLn(f, strLine);
    if pos('{HELP CONSTANTS BEGIN}', Uppercase(strLine)) > 0 then
    	blnInHelpSection:=True;
	end;

  while (blnInHelpSection) and (not Eof(f)) do
  begin
	  Readln(f, strLine);
    if pos('{HELP CONSTANTS END}', Uppercase(strLine)) > 0 then
    	blnInHelpSection := False
    else
    begin
	    strLine := RemoveWhiteSpace(strLine);
  		if strLine<>'' then
	    	Proc(strLine);
    end;
  end;
 	CloseFile(F);
end;

{******************************************************************************}
procedure THelpContextEditor.SetValue(const Value : string);
var
	intPos : integer;
begin
  intPos := Pos('=', Value);
	SetOrdValue(StrToInt(Copy(Value, intPos+1, length(Value))));
end;

{******************************************************************************}
function RemoveWhiteSpace(str : string) : string;
var
	s : string;
	i, intOffset : integer;
begin
	s := str;

  {remove the ; and everthing that follows it}
  intOffset := Pos(';', s);
  if intOffset > 0 then
	 	s := Copy(s, 1, intOffset-1);

  {remove any remain whitespace}
  Result := '';
  for i:=1 to length(s) do
		if not ((s[i]=' ') or (s[i]=#9)) then
    	Result := Result + s[i];
end;

end.
