(*
From: psteinsund@AWI-Bremerhaven.DE (Per Ivar Steinsund)

Could you please add this unit to the Delphi Super Page Freeware library?
It seems to a lot of people who need huge arrays, judgeing from the amount
of questions about this in the newsgroups. This unit can easily be change
to support any array or more than 2-dimensional arrays. This method using
the global memory for arrays is nearly as fast as normal arrays.

Per Ivar Steinsund

*)

Unit Huge;

(***********************************************)
(* Unit for making dynamic arrays and matrix.  *)
(* The elementsize has a limit of 64 kB, but   *)
(* the total size of each array/matrix is only *)
(* limited by available memory.                *)
(*                                             *)
(* Per Ivar Steinsund (1995)                   *)
(*                                             *)
(* Example and use:                            *)
(*                                             *)
(* var J:longint;                              *)
(*     Arr:THugeRealArray;                     *)
(*                                             *)
(* begin                                       *)
(*    Arr:=THugeRealArray.Init(0,200000,       *)
(*             SizeOf(Real),true));            *)
(*    if Arr.Data<>nil then                    *)
(*       for J:=0 to 200000 do Arr.I(J)^:=J/5; *)
(*    Arr.Destroy;                             *)
(* end;                                        *)
(*                                             *)
(***********************************************)

interface

uses  Classes,WinTypes,WinProcs;

Const
    SegSize=$FFFF;

Type
    Adr=record
       Off,Seg:Word;
    end;

    THugeArray=Class
    private
       ElementSize,ElementsPerSeg:LongInt;
       procedure GrabMem(var NewData:Pointer; Range:LongInt;
Change,Message:boolean);
       function GetPtr(Pos:LongInt):Pointer;
    public
       Data:Pointer;
       MinX,MaxX:LongInt;
       constructor Init(Min,Max:LongInt; Size:Word; Message:boolean);
       function ChangeSize(Min,Max:LongInt; Message:boolean):boolean;
       function I(X:LongInt):Pointer;
       destructor Destroy; override;
    end;

    PReal=^Real;

    THugeRealArray=class(THugeArray)
    public
       function I(X:LongInt):PReal;
    end;

    THugeMatrix=class(THugeArray)
    public
       MinY,MaxY,RangeY:LongInt;
       constructor Init(Min1,Max1,Min2,Max2:LongInt; Size:Word;
Message:boolean);
       function ChangeSize(Min1,Max1,Min2,Max2:LongInt;
Message:boolean):boolean;
       function I(X,Y:LongInt):Pointer;
    end;

    THugeWordMatrix=class(THugeMatrix)
       function I(X,Y:LongInt):PWord;
    end;

    PSingle=^Single;

    THugeSingleMatrix=class(THugeMatrix)
    public
       function I(X,Y:LongInt):PSingle;
    end;

implementation

    var EMessage:array [0..80] of char;

    constructor THugeArray.Init(Min,Max:LongInt; Size:Word; Message:boolean);
    begin
       MinX:=Min;
       MaxX:=Max;
       ElementSize:=Size;
       GrabMem(Data,MaxX-MinX+1,false,Message);
    end;

    destructor THugeArray.Destroy;
    begin
       GlobalFreePtr(Data);
    end;

    function THugeArray.I(X:LongInt):Pointer;
    const
       Error:PChar='Error in huge array, continue? X=%lu';
    begin
{$IFOPT R+}
       if (X<MinX) or (X>MaxX) then
       begin
          WVSPrintF(EMessage,Error,X);
          if id_No=MessageBox(0,EMessage,'Index out of range!',mb_YesNo or
mb_IconQuestion) then
          begin
             Destroy;
             Halt;
          end;
       end;
{$ENDIF}
       I:=GetPtr(X-MinX);
    end;

    function THugeArray.ChangeSize(Min,Max:LongInt; Message:boolean):boolean;
    var NewData:Pointer;
    begin
       MinX:=Min;
       MaxX:=Max;
       GrabMem(NewData,MaxX-MinX+1,true,Message);
       if NewData<>nil then
       begin
          ChangeSize:=true;
          Data:=NewData;
       end
       else ChangeSize:=false;
    end;

    procedure THugeArray.GrabMem(var NewData:Pointer; Range:LongInt;
Change,Message:boolean);
    var MemSize:LongInt;
    begin
       MemSize:=Range*ElementSize;
       if not Change then ElementsPerSeg:=SegSize div ElementSize;
       if (MemSize>SegSize) and (SegSize mod ElementSize<>0) then
          MemSize:=(Range div ElementsPerSeg)*SegSize+ElementSize*(Range
mod ElementsPerSeg);
       if Change then NewData:=GlobalReAllocPtr(Data,MemSize,Gmem_Moveable
or Gmem_NoDiscard)
                 else NewData:=GlobalAllocPtr(Gmem_Moveable or
Gmem_NoDiscard,MemSize);
       if (NewData=nil) and Message then
          MessageBox(0,'It is not possible to allocate enough memory.',
                     'Error',mb_OK or mb_IconInformation);
    end;

    function THugeArray.GetPtr(Pos:LongInt):Pointer;
    begin
       GetPtr:=Ptr(Adr(Data).Seg+(Pos div ElementsPerSeg)*SelectorInc,
                   Adr(Data).Off+(Pos mod ElementsPerSeg)*ElementSize);
    end;

    function THugeRealArray.I(X:LongInt):PReal;
    begin
       I:=inherited I(X);
    end;

    constructor THugeMatrix.Init(Min1,Max1,Min2,Max2:LongInt; Size:Word;
Message:boolean);
    begin
       MinX:=Min1;
       MaxX:=Max1;
       MinY:=Min2;
       MaxY:=Max2;
       RangeY:=1+MaxY-MinY;
       ElementSize:=Size;
       GrabMem(Data,(MaxX-MinX+1)*(MaxY-MinY+1),false,Message);
    end;

    function THugeMatrix.ChangeSize(Min1,Max1,Min2,Max2:LongInt;
Message:boolean):boolean;
    var NewData:Pointer;
    begin
       MinX:=Min1;
       MaxX:=Max1;
       MinY:=Min2;
       MaxY:=Max2;
       RangeY:=1+MaxY-MinY;
       GrabMem(NewData,(MaxX-MinX+1)*(MaxY-MinY+1),true,Message);
       if NewData<>nil then
       begin
          ChangeSize:=true;
          Data:=NewData;
       end
       else ChangeSize:=false;
    end;

    function THugeMatrix.I(X,Y:LongInt):Pointer;
    const
       Error:PChar='Error in huge matrix, continue? X=%lu and Y=%lu';
    begin
{$IFOPT R+}
       if (X<MinX) or (X>MaxX) or (Y<MinY) or (Y>MaxY) then
       begin
          WVSPrintF(EMessage,Error,X);
          WVSPrintF(EMessage,EMessage,Y);
          if id_No=MessageBox(0,EMessage,'Index out of range!',mb_YesNo or
mb_IconQuestion) then
          begin
             Destroy;
             Halt;
          end;
       end;
{$ENDIF}
       I:=GetPtr(Y-MinY+RangeY*(X-MinX));
    end;

    function THugeWordMatrix.I(X,Y:LongInt):PWord;
    begin
       I:=inherited I(X,Y);
    end;

    function THugeSingleMatrix.I(X,Y:LongInt):PSingle;
    begin
       I:=inherited I(X,Y);
    end;

end.


