{ TJPicture v1.00 by James Hargrave (hargrave@mep.com)
                     Vancouver, BC
                     Canada
  ---------------------------------------------------------------------------
  Copyright 1995 by Minds Edge Productions Inc. (info@mep.com)
  ---------------------------------------------------------------------------
  This Delphi control is freeware (ie. maybe used 100% free of charge) if
  you find it useful or have any: comments,source/controls of your own,bugs,
  suggestions, or anything please let me know.  The author accepts no
  responsibility for its use or misuse.
  ---------------------------------------------------------------------------
  Thanks to:  Paul Thiessen, Eddie Karsai, Green Day (for keeping the dust
  out of my CD-ROM (and play'n some excellent tunes)), Elvis, the writers for
  the Simpsons and the Tick, and every and anyone who uses Delphi.
  ---------------------------------------------------------------------------
  This is my very first delphi control so please send me your comments.  I
  am releasing this in hopes that it will help someone learn more about
  Delphi which in turn will help spread the use of Borlands greatest
  creation to date.
  ---------------------------------------------------------------------------
  PERPOSE:  This control was designed because I wanted a way to display two
            bitmaps depending on a certain state.  Origionally I had two bitmaps
            on top of each other and would hide the one I didn't want showen.
            This control makes it a little cleaner.
  ---------------------------------------------------------------------------
  PROPERTIES: This class is inherited from TGraphicControl so has all of it's
              properties plus a few redeclarations.  The following are new
              key properties:

             PictureOne     : TBitMap  - Picture Number one
             PictureTwo     : TBitMap  - Picture Number two
             CurrentPicture : BYTE     - Which picture is currently being displayed
                                         ( 1 = PictureOne
                                           2 = PictureTwo
                                           anything else clears the picture (ie. displays nothing)
             ToggleOnClick  : BOOLEAN  - Does clicking on the picture toggle between
                                         PictureOne and PictureTwo
             Stretch        : BOOLEAN  - Do the pictures stretch to fit the control
             AutoSize       : BOOLEAN  - Should the height and width of the control match
                                         the bitmap.
  ---------------------------------------------------------------------------
}


unit JPicture;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Menus, Dialogs, JAboutE, DsgnIntf;

type
  TJPicture = class(TGraphicControl)
  private
    { Private declarations }
    FAbout           : TAboutBoxObject;
    FPictureOne      : TBitMap;
    FPictureTwo      : TBitMap;
    FCurrentPicture  : BYTE;
    FToggleOnClick   : BOOLEAN;
    FStretch         : BOOLEAN;
    FAutoSize        : BOOLEAN;
    procedure SetPictureOne (PicOne : TBitMap);
    procedure SetPictureTwo (PicTwo : TBitMap);
    procedure SetCurrentPicture (PicToSet : BYTE);
    procedure SetToggleOnClick (ToggleToSet : BOOLEAN);
    procedure SetStretch (NewStretch : BOOLEAN);
    procedure SetAutoSize (NewAutoSize : BOOLEAN);
  protected
    { Protected declarations }
    procedure Paint; override; {Called on WM_PAINT message}
    procedure Click; override;
  public
    { Public declarations }
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;
    function CurrentPicHeight : INTEGER;
    function CurrentPicWidth : INTEGER;
  published
    { Published declarations }
    {New Properties}
    property About          : TAboutBoxObject read FAbout write FAbout; {Thanks to Martyn Dowsett}
    property PictureOne     : TBitMap read FPictureOne write SetPictureOne;
    property PictureTwo     : TBitMap read FPictureTwo write SetPictureTwo;
    property CurrentPicture : BYTE read FCurrentPicture write SetCurrentPicture default 1;
    property ToggleOnClick  : BOOLEAN read FToggleOnClick write FToggleOnClick default TRUE;
    property Stretch        : BOOLEAN read FStretch write SetStretch default FALSE;
    property AutoSize       : BOOLEAN read FAutoSize write SetAutoSize default FALSE;
    {Inherited Properties}
    property Height default 65;
    property Width default 65;
    property ShowHint default FALSE;
    property Enabled default TRUE;
    property Visible default TRUE;
    property Align default alNone;
    property DragCursor default crDrag;
    property DragMode default dmManual;
    property ParentShowHint default TRUE;
    property PopUpMenu;
    {Events}
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDblClick;
    property OnDragOver;
    property OnEndDrag;
    property OnClick;  {Modified (see the Click Procedure)}
  end;

procedure Register;

implementation

(* -------------------------------------------------------------------------- *)

function TJPicture.CurrentPicHeight : INTEGER;
BEGIN
  IF (CurrentPicture = 1) THEN
    Result := PictureOne.Height
  ELSE
    IF (CurrentPicture = 2) THEN
      Result := PictureTwo.Height
    ELSE
      Result := 0;
END;

(* -------------------------------------------------------------------------- *)

function TJPicture.CurrentPicWidth : INTEGER;
BEGIN
  IF (CurrentPicture = 1) THEN
    Result := PictureOne.Width
  ELSE
    IF (CurrentPicture = 2) THEN
      Result := PictureTwo.Width
    ELSE
      Result := 0;
END;

(* -------------------------------------------------------------------------- *)

constructor TJPicture.Create (AOwner : TComponent);
BEGIN
  inherited Create (AOwner);
  Height := 65;
  Width := 65;
  FPictureOne := TBitMap.Create;
  FPictureTwo := TBitMap.Create;
  FCurrentPicture := 1;
  ToggleOnClick := TRUE;
  Stretch := FALSE;
END;

(* -------------------------------------------------------------------------- *)

destructor TJPicture.Destroy;
BEGIN
  FPictureOne.Free;
  FPictureTwo.Free;
  inherited Destroy;
END;

(* -------------------------------------------------------------------------- *)

procedure TJPicture.Paint;

{DrawPicture is a nested function that determines what type of draw to do
 if Stretch is set the pictures must be stretched, if not then draw them
 normally.}
PROCEDURE DrawPicture (Pic : TBitMap);
BEGIN
  IF (Stretch) THEN
    Canvas.StretchDraw (ClientRect, Pic)
  ELSE
      Canvas.Draw (0,0,Pic)
END;

BEGIN
  IF (CurrentPicture = 1) THEN
  BEGIN
   IF (NOT PictureOne.Empty) THEN
       DrawPicture (PictureOne);
  END
  ELSE
  BEGIN
    IF (CurrentPicture = 2) THEN
    BEGIN
      IF (NOT PictureTwo.Empty) THEN
        DrawPicture (PictureTwo)
      ELSE
        DrawPicture (NIL);  {If the current picture isn't 1 or 2 then clear the curren picture}
    END; {If Pic = 2}
  END; {Else (Pic <> 1)}

END;

(* -------------------------------------------------------------------------- *)

procedure TJPicture.Click;
BEGIN
  inherited Click;
  IF (ToggleOnClick) THEN
  BEGIN
    IF (CurrentPicture = 1) THEN
      CurrentPicture := 2
    ELSE
      CurrentPicture := 1;
  END;
END;

(* -------------------------------------------------------------------------- *)

procedure TJPicture.SetPictureOne (PicOne : TBitMap);
BEGIN
  IF (PicOne <> FPictureOne) THEN         {If it's not already Set }
  BEGIN
    FPictureOne.Assign (PicOne);          {Set picture one}
    IF (AutoSize) THEN
    BEGIN
      Height := PicOne.Height;
      Width := PicOne.Width;
    END;
    Invalidate;                           {and redraw the screen}
  END;
END;

(* -------------------------------------------------------------------------- *)

procedure TJPicture.SetPictureTwo (PicTwo : TBitMap);
BEGIN
  IF (PicTwo <> FPictureTwo) THEN
  BEGIN
    FPictureTwo.Assign (PicTwo);
    IF (AutoSize) THEN
    BEGIN
      Height := PicTwo.Height;
      Width := PicTwo.Width;
    END;
    Invalidate;
  END;
END;

(* -------------------------------------------------------------------------- *)

procedure TJPicture.SetCurrentPicture (PicToSet : BYTE);
BEGIN
  IF (FCurrentPicture <> PicToSet) THEN
  BEGIN
    FCurrentPicture := PicToSet;
    IF (AutoSize) THEN
    BEGIN
      Height := CurrentPicHeight;
      Width := CurrentPicWidth;
    END;
    Invalidate;
  END;
END;

(* -------------------------------------------------------------------------- *)

procedure TJPicture.SetToggleOnClick (ToggleToSet : BOOLEAN);
BEGIN
  IF (FToggleOnClick <> ToggleToSet) THEN
  BEGIN
    FToggleOnClick := ToggleToSet;
  END;
END;

(* -------------------------------------------------------------------------- *)

procedure TJPicture.SetStretch (NewStretch : BOOLEAN);
BEGIN
  IF (FStretch <> NewStretch) THEN
  BEGIN
    FStretch:= NewStretch;
    Invalidate; {Redraw the picture Stretched}
  END;
END;

(* -------------------------------------------------------------------------- *)

procedure TJPicture.SetAutoSize (NewAutoSize : BOOLEAN);
BEGIN
  IF (FAutoSize <> NewAutoSize) THEN
  BEGIN
    FAutoSize:= NewAutoSize;
    IF (FAutoSize) THEN
    BEGIN
      Height := CurrentPicHeight;
      Width := CurrentPicWidth;
    END;
    Invalidate; {Redraw the picture AutoSized}
  END;
END;

(* -------------------------------------------------------------------------- *)

procedure Register;
begin
  RegisterComponents('Samples', [TJPicture]); {Change this if you want the control}
                                              {on a page other then Samples}
  {Now Register the about box property Editor (See JAboutE.Pas)}
  RegisterPropertyEditor(TypeInfo(TAboutBoxObject),nil,'', TAboutBoxEditor);
end;

(* -------------------------------------------------------------------------- *)

end. {End of File}
