{ Check box List box 																													}
{ by Pablo Pissanetzky																													}
unit LBCheck;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls , Menus;

type
		TCheckEvent = procedure ( Sender : TObject ; Index : Integer ) of object;
  	TCheckListBox = class(TCustomListBox)
		private
   	Bitmap 			: TBitmap;			{ The standard Windows checkbox bitmaps }
     FAuthor			: string;				{ Me 																		}
     FOnCheck 		: TCheckEvent;	{ OnCheck event property 								}
     FOnUnCheck  : TCheckEvent;	{ OnUnCheck event property 							}

     { Read function for Checked property }
     function  IsChecked( Index : Integer ) : Boolean;
     { Write fucntion for Checked property , sets Checked[ Index ] := AChecked }
     procedure Check( Index : Integer ; AChecked : Boolean );
     { Draws the checkbox for the given item in the appropriate state }
     procedure DrawCheckBox( Index : Integer ; State: TOwnerDrawState );
     procedure SetAuthor( Author : string );
  	published
     {	Properties not published 	}
     {   Style 			: set to lbOwnerDrawFixed by the constructor	}
     {   OnDrawItem 	: used to draw check boxes										}

   	{ Properties Added 					}
     property Author 		: string read FAuthor write SetAuthor;
   	property OnCheck 		: TCheckEvent read FOnCheck write FOnCheck;
     property OnUncheck 	: TCheckEvent read FOnUncheck write FOnUnCheck;
     { Original properties 			}
     property Align; property BorderStyle; property Color; property Columns;
     property Ctl3D; property Cursor; property DragCursor; property DragMode;
     property Enabled; property ExtendedSelect; property Font; property Height;
     property HelpContext; property Hint; property IntegralHeight; property ItemHeight;
     property Items; property Left; property MultiSelect; property Name;
     property OnClick; property OnDblClick; property OnDragDrop; property OnDragOver;
     property OnEndDrag; property OnEnter; property OnExit; property OnKeyDown;
     property OnKeyPress; property OnKeyUp; property OnMeasureItem; property OnMouseDown;
     property OnMouseMove; property OnMouseUp; property ParentColor; property ParentCtl3D;
     property ParentFont; property ParentShowHint; property PopupMenu; property ShowHint;
     property Sorted; property TabOrder; property TabStop; property Tag; property Top;
     property Visible; property Width;
		public
   	{ Overriden constructor and destructor }
			constructor Create( AParent : TComponent ); override;
     destructor Destroy; override;
     { Checked property. When True , the item is checked }
   	property Checked[ Index : Integer ] : Boolean read 	IsChecked write Check;
     { Sets the Checked property to AChecked for all selected items }
     procedure CheckSelection( AChecked : Boolean );
     { Toggles the check of the given item }
     procedure Toggle( Index : Integer );
     { Sets the Checked property to AChecked for all items in the list box }
     procedure CheckAll( AChecked : Boolean );
  	protected
   	{ Overriden owner draw method }
   	procedure DrawItem( Index: Integer; ARect: TRect; State: TOwnerDrawState);override;
     { Overriden mouse down event }
    	procedure MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
     { Protected handlers for OnCheck and OnUncheck events }
     procedure CheckEvent( Index : Integer ); virtual;
     procedure UnCheckEvent( Index : Integer ); virtual;
  end;

const
	CheckBoxWidth = 13;
 CheckBoxHeight = 13;
 Margin = 2;

procedure Register;

implementation

	{ Class for internal use only 																		}
 { When an item is first checked,																	}
 { Items.Objects[ item ] is assigned a pointer to a TCheck object 	}
	type
 	TCheck = class( TObject )
  		Checked : Boolean;
   	constructor Create;
  	end;

{------------------------------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('Samples', [TCheckListBox]);
end;
{------------------------------------------------------------------------------}
{ PUBLIC																																				}
{ Constructor for TCheck																												}
constructor TCheck.Create;
begin
	inherited Create;
	Checked := False;
end;
{------------------------------------------------------------------------------}
{ PUBLIC , OVERRIDE																														}
{ Constructor for TCheckListBox																								}
constructor TCheckListBox.Create( AParent : TComponent );
begin
	inherited Create( AParent );
 Style := lbOwnerDrawFixed;
 Bitmap :=TBitmap.Create;
	Bitmap.Handle := LoadBitmap( 0 , PChar( OBM_CHECKBOXES ) );
 FAuthor := 'Pablo Pissanetzky';
end;
{------------------------------------------------------------------------------}
{ PUBLIC	, OVERRIDE																														}
{ Constructor for TCheckListBox																								}
destructor TCheckListBox.Destroy;
begin
	Bitmap.Free;
 inherited Destroy;
end;
{------------------------------------------------------------------------------}
{ PRIVATE																																			}
{ Draws the check box for the item at Index in the given state 								}
procedure TCheckListBox.DrawCheckBox( Index : Integer ; State: TOwnerDrawState );
const
 CheckedSource 	: TRect = ( Left:CheckBoxWidth; Top:0; Right:CheckBoxWidth * 2 ; Bottom:CheckBoxHeight);
 UncheckedSource : TRect = ( Left:0;  Top:0; Right:CheckBoxWidth; Bottom:CheckBoxHeight);
var
	ARect : TRect;
begin
	ARect := ItemRect( Index );
 with ARect do
 	begin
   	Inc( Left , Margin );
     Inc( Top , Margin );
     Right := Left + CheckBoxWidth + Margin;
     Bottom := Top + CheckBoxHeight + Margin;
   end;

	with Canvas do
 	begin
     if ( odSelected in State ) then { if this item is selected, draw in reverse }
       CopyMode := cmNotSrcCopy
     else
       CopyMode := cmSrcCopy;

     if ( Checked[ Index ] ) then { copy the appropriate bitmap }
				CopyRect( ARect , Bitmap.Canvas , CheckedSource )
			else
				CopyRect( ARect , Bitmap.Canvas , UnCheckedSource );
   end;
end;
{------------------------------------------------------------------------------}
{ PROTECTED , OVERRIDE																													}
{ Clears the rectangle, draws the check box and draws the item text 						}
procedure TCheckListBox.DrawItem( Index: Integer; ARect: TRect; State: TOwnerDrawState);
begin
	with Canvas do
 	begin
   	FillRect( ARect ); 							{ Clear rectangle }
     DrawCheckBox( Index , State ); 	{ Draw the check box }
     TextOut( ARect.Left + CheckBoxWidth + ( Margin * 2 ) , ARect.Top , Items[ Index ] ); { Draw the text }
   end;
end;
{------------------------------------------------------------------------------}
{ PROTECTED , OVERRIDE 																												}
{ Calls inherited MouseDown ( which calls user's mouse down )									}
{ If the user has clicked inside a check box , toggle the checkbox and draw 		}
{ the focus rectangle.																													}
procedure TCheckListBox.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
	Index 		: Integer;
 AChecked 	: Boolean;
 Rect      : TRect;
begin
 inherited MouseDown( Button , Shift , X , Y );	{ call inherited method }
	if ( Button = mbLeft ) then
 	begin
 		Index := ItemAtPos( Point( X , Y ) , True );{ Find the index of the item clicked }
 		if ( Index > -1 ) then											{ If it is a valid item }
   		begin
     		Rect := ItemRect( Index );							{ Get the rectangle for the item }
         with Rect do                            { Build the rectangle for the checkbox }
         	begin
             Inc( Left , Margin );
             Inc( Top , Margin );
             Right := Left + CheckBoxWidth ;
             Bottom := Top + CheckBoxHeight ;
           end;
         if PtInRect( Rect , Point( X , Y ) ) then { if the user clicked inside the check box }
         	begin
     				Toggle( Index );												{ Toggle the item's checked state }
     				Canvas.DrawFocusRect( ItemRect( Index ) );	{ draw focus rectangle }
           end;
   		end;
   end;
end;
{------------------------------------------------------------------------------}
{ PRIVATE																																			}
{ Returns True if the item at Index is Checked, otherwise returns False 				}
function TCheckListBox.IsChecked( Index : Integer ) : Boolean;
begin
	if ( Items.Objects[ Index ] = nil ) then	{ if objects is nil , this item has never been checked }
 	Result := False
 else
 	with Items.Objects[ Index ] as TCheck do { otherwise , objects points to a TCheck object with the current state }
   	Result := Checked;
end;
{------------------------------------------------------------------------------}
{ PRIVATE																																			}
{ Sets the item at index to be checked if AChecked is True 										}
{ or unchecked if AChecked is False																						}
{ If the item was not already in the given state, it redraws the check box			}
{ and generates either an OnCheck or OnUnCheck event 													}
procedure TCheckListBox.Check( Index : Integer ; AChecked : Boolean );
var
	State : TOwnerDrawState;
begin
	if ( Items.Objects[ Index ] = nil ) then		{ if no object exists, create one }
		Items.Objects[ Index ] := TCheck.Create;

 with Items.Objects[ Index ] as TCheck do
 	if ( Checked <> AChecked ) then        		{ if the new state is different from the existing state }
   	begin
   		Checked := AChecked;
 			if ( Selected[ Index ] ) then   			{ if the item is selected, it must be drawn accordingly }
 				State := [ odSelected ]
       else
       	State := [];
       DrawCheckBox( Index , State );				{ draw the check box }
       if ( AChecked ) then                  { generate appropriate event }
       	CheckEvent( Index )
       else
       	UnCheckEvent( Index );
     end;
end;
{------------------------------------------------------------------------------}
{ PUBLIC																																				}
{ Sets all selected items to checked if AChecked is True, unchecked otherwise	}
procedure TCheckListBox.CheckSelection( AChecked : Boolean );
var
	Index : Integer;
begin
	if ( SelCount > 0 ) then
 	for Index := 0 to Items.Count - 1 do
     if ( Selected[ Index ] ) then
     	Checked[ Index ] := AChecked;
end;
{------------------------------------------------------------------------------}
{ PUBLIC																																				}
{ Toggles the state of the item at the given index 														}
procedure TCheckListBox.Toggle( Index : Integer );
begin
	Checked[ Index ] := not Checked[ Index ];
end;
{------------------------------------------------------------------------------}
{ PROTECTED , VIRTUAL  																												}
{ Calls user's OnCheck event handler																						}
procedure TCheckListBox.CheckEvent( Index : Integer );
begin
	if ( Assigned( FOnCheck ) ) then
 	OnCheck( Self , Index );
end;
{------------------------------------------------------------------------------}
{ PROTECTED , VIRTUAL 																													}
{ Calls user's OnUnChecked event handler																				}
procedure TCheckListBox.UnCheckEvent( Index : Integer );
begin
	if ( Assigned( FOnUncheck ) ) then
 	OnUnCheck( Self , Index );
end;
{------------------------------------------------------------------------------}
{ PUBLIC																																				}
{ Checks all items in the list box if AChecked is True, unchecks them otherwise}
procedure TCheckListBox.CheckAll( AChecked : Boolean );
var
	Index : Integer;
begin
	for Index := 0 to Items.Count - 1 do
 	Checked[ Index ] := AChecked;
end;
{------------------------------------------------------------------------------}
procedure TCheckListBox.SetAuthor( Author : string );
begin
end;
{------------------------------------------------------------------------------}
end.
