unit Main;

{ Demo program for the TLZHCompressor component.
  Copyright Douglas P. Webb, 1995, All rights Reserved. }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, LZH_OP;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileExitItem: TMenuItem;
    SpeedBar: TPanel;
    SpeedButton1: TSpeedButton;
    Compress1: TMenuItem;
    Decompress1: TMenuItem;
    OpenDialog: TOpenDialog;
    LZHC: TLZHCompressor;
    Label1: TLabel;
    CompresswithVerify1: TMenuItem;
    About1: TMenuItem;  { E&xit }
    procedure FileExit(Sender: TObject);
    procedure Compress1Click(Sender: TObject);
    procedure LZHCPressFinished(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Decompress1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure About1Click(Sender: TObject);
    procedure CompresswithVerify1Click(Sender: TObject);
  private
    infile, outfile : File;

    procedure GetBytes(Sender:TObject; VAR DTA; NBytes:WORD; VAR Bytes_Got : WORD);
    procedure PutBytes(Sender:TObject; VAR DTA; NBytes:WORD; VAR Bytes_Put : WORD);

    { Dirk Henckels  -  8.12.95  -  email dirk@ifib.uni-karlsruhe.de }
    function DateiVerifizieren(const inputfile, outputfile: TFileName): boolean;
    function DateiKomprimieren(const inputfile, outputfile: TFileName): boolean;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

const
  ChunkSize = 4096;
var
  CopyBuffer   : PChar;
  VerifyBuffer : PChar;

{ File Manipulation Routines for use with Douglas P. Webb's Compression Component }
{ Dirk Henckels  -  8.12.95  -  email dirk@ifib.uni-karlsruhe.de }

{ verify file "inputfile" with file "outputfile" }
{ result = true: files are equal, false: files are unequal }
function TMainForm.DateiVerifizieren(const inputfile, outputfile: TFileName): boolean;
{ inputfile = source file path and name }
{ outputfile = distination file path and name }
  var
    cv, cn: PChar;
    BytesIn, BytesOut: Longint;
    Source, Dest: Integer; { handles }
    i: integer;
  begin
  { start verification }
  Result := false;
  Source := FileOpen(inputfile, fmShareDenyWrite); { open source file }

  if Source < 0 then
    begin
     { source unreadable }
    end
  else
    begin
    try
      Dest := FileOpen(outputfile, fmShareDenyWrite); { open destination file }
      if Dest < 0 then
        begin
          { destination unwritable }
        end
      else
        begin
        try
          Result := true;   { Assume success }
          repeat
            BytesIn := FileRead(Source, CopyBuffer^, ChunkSize); { read chunk }
            BytesOut := FileRead(Dest, VerifyBuffer^, ChunkSize); { read chunk }
            if BytesIn = BytesOut then
              begin
              cv := CopyBuffer;
              cn := VerifyBuffer;
              for i := 0 to BytesIn-1 do
                begin
                if cv^ <> cn^ then
                  begin
                  Result := false;
                  { data comparison error occured }
                  end;
                Inc(cv);
                Inc(cn);
                end;
              end;
          until (BytesIn < ChunkSize) or (BytesOut < ChunkSize);
          { until we run out of chunks }

          if (BytesIn <> BytesOut) then
            begin
            Result := false;
            { A length error occurred in the comparison }
            end
          else
            begin
            { No length comparison error occurred  }
            end;
        finally
          FileClose(Dest); { close the destination file }
        end;
        end;
    finally
      FileClose(Source); { close the source file }
    end;
    end;
  end;

{ compress file "inputfile" to file "outputfile" }
{ result = true: everything worked out fine, false: some problem occured }
function TMainForm.DateiKomprimieren(const inputfile, outputfile: TFileName): boolean;
{ inputfile = source file path and name }
{ outputfile = distination file path and name }
  begin
  { start compression }
  AssignFile(InFile, inputfile);
  Reset(Infile,1);
  AssignFile(OutFile, outputfile);
  Rewrite(Outfile,1);
  try
    Label1.Caption := 'Compressing...';
    LZHC.Compress;
  finally
    CloseFile(infile);
    CloseFile(outfile);
  end;

  { Verify that the temporary file LZVER$$$.$$$ matches the
      original file that was compressed. }
  AssignFile(InFile, outputfile);
  Reset(Infile,1);
  AssignFile(OutFile, 'LZVER$$.$$$');
  Rewrite(Outfile,1);
  try
    Label1.Caption := 'Decompressing...';
    LZHC.DeCompress;
  finally
    CloseFile(infile);
    CloseFile(outfile);
  end;

  Result := DateiVerifizieren(inputfile, 'LZVER$$.$$$');   { do verification with patch }
  Screen.Cursor := crDefault;
  Application.ProcessMessages;

  { Delete temporary file }
  Deletefile('LZVER$$.$$$');
  end;




procedure TMainForm.GetBytes(Sender:TObject; VAR DTA; NBytes:WORD; VAR Bytes_Got : WORD);
  begin
  Screen.Cursor := crSizeWE + (SUCC(Screen.Cursor) - crSizeWE) MOD 4;
  Application.ProcessMessages;
  BlockRead(infile, DTA, NBytes, Bytes_Got);
  end;

procedure TmainForm.PutBytes(Sender:TObject; VAR DTA; NBytes:WORD; VAR Bytes_Put : WORD);
  begin
  BlockWrite(outfile, DTA, NBytes, Bytes_Put);
  end;


procedure TMainForm.FileExit(Sender: TObject);
  begin
  Close;
  end;

procedure TMainForm.Compress1Click(Sender: TObject);
  begin
  OpenDialog.Filter := 'All Files|*.*';
  if OpenDialog.Execute then
    begin
    AssignFile(InFile, OpenDialog.FileName);
    Reset(Infile,1);
    AssignFile(OutFile, ChangeFileExt(OpenDialog.FileName,'.LZD'));
    Rewrite(Outfile,1);
    Screen.Cursor := crSizeWE;
    Application.ProcessMessages;
    try
      Label1.Caption := 'Compressing...';
      LZHC.Compress;
    finally
      Screen.Cursor := crDefault;
      Application.ProcessMessages;
      CloseFile(infile);
      CloseFile(outfile);
    end;
    end;
  end;


procedure TMainForm.CompresswithVerify1Click(Sender: TObject);
  begin
  OpenDialog.Filter := 'All Files|*.*';
  if OpenDialog.Execute then
    begin
    if not DateiKomprimieren(OpenDialog.FileName, ChangeFileExt(OpenDialog.FileName,'.LZD')) then
      ShowMessage('Compression verify failed.')
    else
      ShowMessage('Compression verified.');
    end;
  end;



procedure TMainForm.Decompress1Click(Sender: TObject);
  begin
  OpenDialog.Filter := 'LHA 1.x Style|*.LZD';
  if OpenDialog.Execute then
    begin
    AssignFile(InFile, OpenDialog.FileName);
    Reset(Infile,1);
    AssignFile(OutFile, ChangeFileExt(OpenDialog.FileName,'.$$$'));
    Rewrite(Outfile,1);
    Screen.Cursor := crSizeWE;
    Application.ProcessMessages;
    try
      Label1.Caption := 'Decompressing...';
      LZHC.DeCompress;
    finally
      Screen.Cursor := crDefault;
      Application.ProcessMessages;
      CloseFile(infile);
      CloseFile(outfile);
    end;
    end;

  end;


procedure TMainForm.LZHCPressFinished(Sender: TObject);
  begin
  Label1.Caption := 'Done!';
  end;

procedure TMainForm.FormCreate(Sender: TObject);
  begin
  { Assigning these at design time doesn't seem to work. }
  LZHC.OnGetBytes := GetBytes;
  LZHC.OnPutBytes := PutBytes;

  CopyBuffer   := StrAlloc(ChunkSize);
  VerifyBuffer := StrAlloc(ChunkSize);
  end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
  begin
  StrDispose(CopyBuffer);
  StrDispose(VerifyBuffer);

end;

procedure TMainForm.About1Click(Sender: TObject);
  begin
  ShowMessage('TLZHCompressor Test/Verification Program.'+#13+
              'Copyright Douglas P. Webb  1995.'+#13+
              'Original verification code by Dirk Henckels.');
  end;

end.
