unit Main;
interface
{$A-} { Don't Word Align the file records; keep them compact }
{$X+} { Extended syntax is required for zero-terminated strings }

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, Grids, ExtCtrls;

const
  wm_UserRestore = wm_User + 1;  { user message for multi-instance handling }

const
  SmallWd = 12;     { dimension of grids on the form }
  SmallHt = 8;
  LargeWd = 26;
  LargeHt = 16;
  MinChar = 32;     { only chars 32..126 are supported }
  MaxChar = 126;
  OpenCh  = ord('A')-MinChar;    { open fonts at this character }
  MaxHistoryLength = 4;
  HistoryIniTitle = 'Font History';
  HistoryIni = 'win.ini';        { or FontEdit.ini if you preferred }

type
  TBinaryFontHeader = record
     Signature     : array [0..5] of Char;   { allow room for 0-terminator }
     Proportional  : Boolean;
     NoDefaultWid  : Boolean;
     SmallWidth    : ShortInt;
     LargeWidth    : ShortInt;
  end;

  TSmallArray = record
    MarginL,MarginR: ShortInt;
    Bits: array [0..SmallWd-1,0..SmallHt-1] of Boolean;
  end;

  TLargeArray = record
    MarginL,MarginR: ShortInt;
    Bits: array [0..LargeWd-1,0..LargeHt-1] of Boolean;
  end;

  TCharDefn = class
    Code: Byte;
    Small: TSmallArray;
    Large: TLargeArray;
    public
      constructor Create(Ascii: Char);
      procedure   AssignTo(Dest: TCharDefn);
      procedure   ClearBits;
      procedure   NoMargins;
      procedure   CentredMargins(ReqdSWid:ShortInt; ReqdLWid:ShortInt);
      function    IsEmpty: boolean;
      procedure   SaveToStream(Stream: TStream);
      procedure   LoadFromStream(Stream: TStream);
  end;


  TFontEditorForm = class(TForm)
    SmallGrid: TDrawGrid;
    LargeGrid: TDrawGrid;
    ScrollBar1: TScrollBar;
    CodeCombo: TComboBox;
    HexCode: TLabel;
    MarginPanel: TPanel;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    MainMenu: TMainMenu;
    MenuFile: TMenuItem;
    MenuNew: TMenuItem;
    N1: TMenuItem;
    MenuOpen: TMenuItem;
    MenuSave: TMenuItem;
    MenuSaveAs: TMenuItem;
    N2: TMenuItem;
    MenuExit: TMenuItem;
    MenuEdit: TMenuItem;
    MenuDoubleSize: TMenuItem;
    MenuBold: TMenuItem;
    N3: TMenuItem;
    MenuBloom: TMenuItem;
    MenuBloomAll: TMenuItem;
    N4: TMenuItem;
    MenuErase: TMenuItem;
    MenuEraseAll: TMenuItem;
    N5: TMenuItem;
    MenuClear: TMenuItem;
    MenuCode: TMenuItem;
    MenuMake: TMenuItem;
    MenuOptions: TMenuItem;
    MenuHelp: TMenuItem;
    MenuAbout: TMenuItem;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuAboutClick(Sender: TObject);
    procedure CodeComboChange(Sender: TObject);
    procedure CodeComboKeyPress(Sender: TObject; var Key: Char);
    procedure ScrollBar1Change(Sender: TObject);
    procedure MenuNewClick(Sender: TObject);
    procedure MenuOpenClick(Sender: TObject);
    procedure MenuSaveClick(Sender: TObject);
    procedure MenuSaveAsClick(Sender: TObject);
    procedure MenuDoubleSizeClick(Sender: TObject);
    procedure MenuBoldClick(Sender: TObject);
    procedure MenuBloomClick(Sender: TObject);
    procedure MenuBloomAllClick(Sender: TObject);
    procedure MenuEraseClick(Sender: TObject);
    procedure MenuEraseAllClick(Sender: TObject);
    procedure MenuClearClick(Sender: TObject);
    procedure MenuMakeClick(Sender: TObject);
    procedure MenuOptionsClick(Sender: TObject);
    procedure GridDrawCell(Sender: TObject; Col, Row: Longint; Rect: TRect; State: TGridDrawState);
    procedure GridMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure GridMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure GridMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure GridDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure GridDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure NullDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure HistoryClick(Sender:TObject);
  private
    { Private declarations }
    EditChar: TCharDefn;
    CurrentFile: TFileName;
    Drawing: Boolean;
    Setting: Boolean;
    FontHeader: TBinaryFontHeader;
    IsDirty: Boolean;
    DragMarginL,DragMarginR: ShortInt;
    DragLeft: Boolean;
    FileHistory: TStringList;
    HistoryBase: integer;
    GenSmall,GenLarge,GenLBloom,GenRBloom,
      GenIndex,GenZeroPad,GenHexComment: Boolean;

    procedure HistoryCreate;
    procedure HistoryDestroy;
    procedure HistoryUpdate;
    procedure HistoryNotifyWorkfile(FileName:string);
    function  CountChars: Integer;
    procedure MakeNewFont(const NewHeader: TBinaryFontHeader);
    procedure ClearChars;
    function  SaveChangesQuery : boolean;
    procedure AssignGenerationDefaults;
    procedure OpenFile(FileName:string);
    procedure ShowNewFile(Name:string);
    procedure ShowNewChar;
    procedure ReadFontBinary(const Filename:TFileName);
    procedure WriteFontBinary(const Filename:TFileName);
    function  SmallCol(const Ar:TSmallArray; c:integer):integer;
    function  LargeCol(const Ar:TLargeArray; c:integer):Word;
    procedure SetCol(var Ar:TSmallArray; c:integer; W:Word);
    procedure SetCols(var Ar:TLargeArray; c:integer; W:Word);
    procedure BloomChar(var ch:TCharDefn);
    procedure MarginEraseSmall(var ch:TCharDefn);
    procedure MarginEraseLarge(var ch:TCharDefn);
    procedure MakeAsm(DefaultName:string);
  public
    { Public declarations }
    procedure wmUserRestore(var msg:TMessage); message wm_UserRestore;
    procedure Generate;
  end;

var
  FontEditorForm: TFontEditorForm;



implementation

uses NewFont,Preview,Options,IniFiles;

{$R *.DFM}

const
  MarginColor = clOlive;

  DefaultBinaryFontHeader : TBinaryFontHeader = (
     Signature     : 'BFONT';
     Proportional  : False;
     NoDefaultWid  : False;
     SmallWidth    : 5;
     LargeWidth    : 10
   );

type
  EBadFontFile = class(Exception)
  end;



{ ------------- TCharDefn CLASS ---------------------- }

constructor TCharDefn.Create(Ascii: Char);
begin
  inherited Create;
  Code := Byte(Ascii);
  ClearBits;
  NoMargins;
end;

procedure TCharDefn.AssignTo(Dest:TCharDefn);
{ copy entire object, including Code }
begin
  Dest.Code := Code;
  Dest.Small := Small;
  Dest.Large := Large;
end;

procedure TCharDefn.ClearBits;
begin
  FillChar(Small.Bits,sizeof(Small.Bits),False);
  FillChar(Large.Bits,sizeof(Large.Bits),False);
end;

procedure TCharDefn.NoMargins;
begin
  Small.MarginL := -1;
  Small.MarginR := SmallWd;
  Large.MarginL := -1;
  Large.MarginR := LargeWd;
end;

procedure TCharDefn.CentredMargins(ReqdSWid:ShortInt; ReqdLWid:ShortInt);
begin
  Small.MarginL := (SmallWd - ReqdSWid) div 2 - 1;
  Small.MarginR := Small.MarginL + ReqdSWid + 1;
  Large.MarginL := (LargeWd - ReqdLWid) div 2 - 1;
  Large.MarginR := Large.MarginL + ReqdLWid + 1;
end;

function TCharDefn.IsEmpty : boolean;
var r,c:integer;
begin
  Result := False;
  if Code = 32 then Exit;   { space char doesn't need pixels to exist }
  with Small do
    for r := 0 to pred(SmallHt) do
     for c := 0 to pred(SmallWd) do
       if Bits[c,r] then Exit;
  with Large do
    for r := 0 to pred(LargeHt) do
     for c := 0 to pred(LargeWd) do
       if Bits[c,r] then Exit;
  Result := True;
end;

procedure TCharDefn.SaveToStream(Stream: TStream);
{ Because all its fields are simple types or records, streaming is easy }
{ Beware: record streaming seems to screw up if Word Alignment is used. }
begin
  Stream.Write(Code, SizeOf(Code));
  Stream.Write(Small,SizeOf(Small));
  Stream.Write(Large,SizeOf(Large));
end;

procedure TCharDefn.LoadFromStream(Stream: TStream);
begin
  Stream.Read(Code,SizeOf(Code));
  if (Code < MinChar) or (Code > MaxChar) then
    raise EBadFontFile.CreateFmt('Invalid char %d found',[Code]);
  Stream.Read(Small,SizeOf(Small));
  Stream.Read(Large,SizeOf(Large));
end;

{----------------- TFontEditorForm ----------------------------------}

procedure TFontEditorForm.FormCreate(Sender: TObject);
const
  tabs:array[0..1] of integer = (60,120);
var ch:Char;
    NewChar:TCharDefn;
begin
  with CodeCombo.Items do
    for ch := Char(MinChar) to Char(MaxChar) do
      begin
        NewChar := TCharDefn.Create(ch);
        AddObject(ch,TObject(NewChar));
      end;
  ScrollBar1.SetParams(0, 0, pred(CodeCombo.Items.Count));
  MakeNewFont(DefaultBinaryFontHeader);
  HistoryCreate;
end;

procedure TFontEditorForm.FormDestroy(Sender: TObject);
var i:integer;
begin
  with CodeCombo.Items do        { release the combobox TCharDefn arrays }
    for i := 0 to pred(Count) do
      TCharDefn(Objects[i]).Free;
  HistoryDestroy;
end;

procedure TFontEditorForm.wmUserRestore(var msg:TMessage);
begin
  Application.Restore;
end;

procedure TFontEditorForm.MenuExitClick(Sender: TObject);
begin
  Close;
end;

function TFontEditorForm.CountChars : integer;
var i:integer;
begin
  Result := 0;
  with CodeCombo.Items do
    for i := 0 to pred(Count) do
      with Objects[i] as TCharDefn do
        if not IsEmpty then inc(Result);
end;

procedure TFontEditorForm.MakeNewFont(const NewHeader: TBinaryFontHeader);
begin
  FontHeader := NewHeader;
  ClearChars;                { set margins to suit FontHeader }
  EditChar := CodeCombo.Items.Objects[OpenCh] as TCharDefn;
  CurrentFile := '';
  ShowNewFile('Untitled.FB');
  IsDirty := False;
  ShowNewChar;
end;

procedure TFontEditorForm.ClearChars;
{ Clear all bits on each character, and reset margins to suit font }
var
  i:integer;
begin
  with CodeCombo.Items do        { release the combobox TCharDefn arrays }
    for i := 0 to pred(Count) do
      with Objects[i] as TCharDefn do
        begin
          ClearBits;
          if (FontHeader.Proportional and FontHeader.NoDefaultWid)
            then NoMargins
            else CentredMargins(FontHeader.SmallWidth,FontHeader.LargeWidth);
        end;
end;


{ ---------------- FILE HISTORY ----------------------------------- }
{ I'd like to eventually do this as a TFileHistory component, eg.
    property   Menu         FileMenu1:TMenuItem
    property   IniFileName  'win.ini'
    property   MaxHistory   3
    method     NotifyOpen
    event      OnClick
  An alternative approach, by Douglas Horn, appears in December 95
  issue of Delphi Informant (refer DI9512DH.ZIP on Compuserve).
}

procedure TFontEditorForm.HistoryCreate;
var i:integer;
    Ini:TIniFile;
    s:string;
begin
  Ini := TIniFile.Create(HistoryIni);
  try
    FileHistory := TStringList.Create;
    for i := 1 to MaxHistoryLength do with Ini do
      begin
        s := ReadString(HistoryIniTitle,IntToStr(i),'');
        if length(s) > 0 then FileHistory.Add(s);
      end;
  finally
    Ini.Free;
  end;
  HistoryBase := MenuFile.Count;
  HistoryUpdate;
end;

procedure TFontEditorForm.HistoryUpdate;
var i:integer;
    NewItem:TMenuItem;
    Ini:TIniFile;
    CurrDir,s:string;
begin
  GetDir(0,CurrDir);                   { fetch the current drive }
  { Under Win3.11 GetDir includes a trailing slash only when
    in a drive's root directory.  }
  if length(CurrDir) > 3 then AppendStr(CurrDir,'\');
  with MenuFile do                     { the TMenuItem to be appended }
    begin
      while Count > HistoryBase do     { delete history items }
        Delete(HistoryBase);
      if FileHistory.Count > 0 then    { append to menu }
        begin
          NewItem := TMenuItem.Create(Self);
          NewItem.Caption := '-';
          Add(NewItem);
          for i := 1 to FileHistory.Count do
            begin
              s := FileHistory.Strings[i-1];   { includes path }
              if ExtractFilePath(s) = CurrDir then
                s := ExtractFileName(s);   { filename portion only }
              NewItem := TMenuItem.Create(Self);
              NewItem.Caption := Format('&%d %s',[i,s]);
              NewItem.Tag := i-1;
              NewItem.OnClick := HistoryClick;
              Add(NewItem);
            end;
        end;
    end;
  Ini := TIniFile.Create(HistoryIni);
  try
    with FileHistory do
      for i := 1 to Count do
        Ini.WriteString(HistoryIniTitle,IntToStr(i),Strings[i-1]);
  finally
    Ini.Free;
  end;
end;

procedure TFontEditorForm.HistoryNotifyWorkfile(FileName:string);
{ Notify history list that a file has been opened }
var Posn:integer;
begin
  FileName := ExpandFileName(FileName);
  with FileHistory do
    begin
      Posn := IndexOf(FileName);
      if Posn >= 0 then
        Delete(Posn)    { move from existing position to top of list }
      else
        if Count >= MaxHistoryLength then
          Delete(MaxHistoryLength-1);      { discard bottom of list }
      Insert(0,FileName);
    end;
  HistoryUpdate;
end;

procedure TFontEditorForm.HistoryClick(Sender:TObject);
{ All history menu items invoke this handler when clicked }
begin
  with Sender as TMenuItem do
    OpenFile(FileHistory.Strings[Tag]);
end;

procedure TFontEditorForm.HistoryDestroy;
begin
  FileHistory.Free;
end;



{ ------------- DISPLAY CONTROL COORDINATION ---------------------- }

procedure TFontEditorForm.ShowNewFile(Name:string);
begin
  Name := ExtractFileName(Name);
  Delete(Name,Length(Name)-2,3);  { remove .FB extension }
  with FontHeader do
    if Proportional then
      Caption := Format('%s - proportional',[Name])
    else
      Caption := Format('%s - fixed width (%d,%d)',
                        [Name,SmallWidth,LargeWidth]);
  { only allow margin adjustments for proportional fonts }
  MarginPanel.Enabled := FontHeader.Proportional;
  with MarginPanel do
    if Enabled then Color := MarginColor
               else Color := clBtnFace;
end;


procedure TFontEditorForm.MenuClearClick(Sender: TObject);
begin
  if MessageDlg('Clear this character?',mtConfirmation, mbOkCancel, 0)
                  <> mrOk then Exit;
  EditChar.ClearBits;
  SmallGrid.Invalidate;
  LargeGrid.Invalidate;
  IsDirty := True;
end;

procedure TFontEditorForm.ShowNewChar;
{ make other controls consistent with Char selection }
begin
  ScrollBar1.Position := EditChar.Code - MinChar;
  CodeCombo.ItemIndex := EditChar.Code - MinChar;
  HexCode.Caption := Format('$%.2x',[EditChar.Code]);
  SmallGrid.Invalidate;
  LargeGrid.Invalidate;
end;

procedure TFontEditorForm.CodeComboChange(Sender: TObject);
begin
  with Sender as TComboBox do
    if ItemIndex >= 0 then
      begin
        EditChar := TCharDefn(Items.Objects[ItemIndex]);
        ShowNewChar;
      end;
end;

procedure TFontEditorForm.CodeComboKeyPress(Sender: TObject; var Key: Char);
{ There's a bug here: when list is dropped down, lowercase alpha keys scroll
  it to the uppercase equivalents.  I haven't tried to work out why. }
var Ascii:integer;
begin
  Ascii := ord(Key);
  if (Ascii >= MinChar) and (Ascii <= MaxChar) then
    with Sender as TComboBox do
      ItemIndex := Ascii - MinChar;
end;

procedure TFontEditorForm.ScrollBar1Change(Sender: TObject);
begin
  EditChar := TCharDefn(CodeCombo.Items.Objects[ScrollBar1.Position]);
  ShowNewChar;
end;


{ ---------------- GRID DRAWING EVENTS ------------------------ }
{ Note that grids have properties DefaultDrawing = TabStop = false }

procedure TFontEditorForm.GridDrawCell(Sender: TObject; Col, Row: Longint;
                                       Rect: TRect; State: TGridDrawState);
const
  CellColors : array [False..True,False..True] of TColor =
               ((MarginColor,clDkGray),(clWindow,clHighlight));
var
  BetweenMgns,BitSet:boolean;
begin
  if Sender = SmallGrid then with EditChar.Small do
    BetweenMgns := (MarginL < Col) and (Col < MarginR)
  else with EditChar.Large do
    BetweenMgns := (MarginL < Col) and (Col < MarginR);
  with (Sender as TDrawGrid).Canvas do
    with EditChar do
      begin
        if Sender = SmallGrid then BitSet := Small.Bits[Col,Row]
                              else BitSet := Large.Bits[Col,Row];
        Brush.Color := CellColors[BetweenMgns,BitSet];
        FillRect(Rect);
      end;
end;

procedure TFontEditorForm.GridMouseDown(Sender: TObject; Button: TMouseButton;
                                        Shift: TShiftState; X,Y: Integer);
var c,r: Longint;
begin
  (Sender as TDrawGrid).MouseToCell(X, Y, c, r);
  { Grid does not seem to capture control for right mouse clicks.
    Line below gives us notification of right MouseUp events. }
  SetCaptureControl(Sender as TControl);
  Setting := Button = mbLeft;
  if Sender = SmallGrid
    then EditChar.Small.Bits[c,r] := Setting
    else EditChar.Large.Bits[c,r] := Setting;
  { Line below is more efficient than TDrawGrid(Sender).Invalidate: }
  with Sender as TDrawGrid do
      GridDrawCell(Sender,c,r,CellRect(c,r),[]);
  Drawing := True;
  IsDirty := True;
end;

procedure TFontEditorForm.GridMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
{ Beware! when mouse moves left/above the grid, MouseToCell returns negative
  row & column values: this must be caught or array range-error/GPF occurs. }
var c,r: Longint;
begin
  if not Drawing then Exit;
  (Sender as TDrawGrid).MouseToCell(X, Y, c, r);
  if (c < 0) or (r < 0) then Exit;
  if Sender = SmallGrid
       then EditChar.Small.Bits[c,r] := Setting
       else EditChar.Large.Bits[c,r] := Setting;
  with Sender as TDrawGrid do
      GridDrawCell(Sender,c,r,CellRect(c,r),[]);
end;

procedure TFontEditorForm.GridMouseUp(Sender: TObject; Button: TMouseButton;
                                      Shift: TShiftState; X, Y: Integer);
begin
  Drawing := False;
  SetCaptureControl(nil);  { release the mouse }
end;


{ ------------- BINARY STREAM I/O ---------------------- }

procedure TFontEditorForm.ReadFontBinary(const Filename:TFileName);
var
  Stream: TFileStream;
  Header: TBinaryFontHeader;
  NoChars,i:integer;
  NewChar: TCharDefn;
begin
  Screen.Cursor := crHourGlass;
  try
    Stream := TFileStream.Create(Filename, fmOpenRead or fmShareDenyWrite);
    Stream.Read(Header,SizeOf(Header));
    { char arrays are compared as if they're zero-terminated strings }
    if Header.Signature <> DefaultBinaryFontHeader.Signature then
      raise EBadFontFile.Create('Bad header');
    FontHeader := Header;
    ClearChars;
    NewChar := TCharDefn.Create(#0);
    Stream.Read(NoChars,SizeOf(NoChars));
    for i := 1 to NoChars do
      begin
        NewChar.LoadFromStream(Stream);
        NewChar.AssignTo(CodeCombo.Items.Objects[NewChar.Code-MinChar] as TCharDefn);
      end;
  finally
    Screen.Cursor := crDefault;
    NewChar.Free;
    Stream.Free;
  end;
end;

procedure TFontEditorForm.WriteFontBinary(const Filename:TFileName);
var
  Stream: TFileStream;
  NoChars,i: integer;
begin
  Screen.Cursor := crHourGlass;
  try
    Stream := TFileStream.Create(Filename, fmCreate);
    { streams can write records but not classes (unless persistent) }
    Stream.Write(FontHeader,SizeOf(FontHeader));
    NoChars := CountChars;
    Stream.Write(NoChars,SizeOf(NoChars));
    with CodeCombo.Items do
      for i := 0 to pred(Count) do
        with Objects[i] as TCharDefn do
          if not IsEmpty then SaveToStream(Stream);
  finally
    Screen.Cursor := crDefault;
    Stream.Free;
  end;
end;

{ ------------- NEW/OPEN/SAVE DIALOGS ---------------------- }

function TFontEditorForm.SaveChangesQuery : boolean;
{ Returns false if user Cancels, otherwise true. }
var Prompt: string;
begin
  Result := True;
  if IsDirty then
    begin
      if Length(CurrentFile) = 0
         then Prompt := 'Do you wish to save current font?'
         else Prompt := Format('Save changes to %s?',[ExtractFileName(CurrentFile)]);
      case MessageDlg(Prompt,mtConfirmation,mbYesNoCancel,0) of
        mrYes:    MenuSaveClick(Self);
        mrCancel: Result := False;
      end;
    end;
end;

procedure TFontEditorForm.AssignGenerationDefaults;
var Ini: TIniFile;
begin
  GenLBloom := False;
  GenRBloom := FontHeader.Proportional;
  GenIndex := True;
  Ini := TIniFile.Create(GenerateIniName);
  try
    GenSmall := Ini.ReadBool(Section, SmallItem, True);
    GenLarge := Ini.ReadBool(Section, LargeItem, True);
    Ini.WriteBool(Section, LBloomItem, GenLBloom);
    Ini.WriteBool(Section, RBloomItem, GenRBloom);
    Ini.WriteBool(Section, IndexItem, GenIndex);
    GenZeroPad := Ini.ReadBool(Section, ZeroPadItem, True);
    GenHexComment := Ini.ReadBool(Section, HexCommentItem, True);
  finally
    Ini.Free;
  end;
end;

procedure TFontEditorForm.MenuNewClick(Sender: TObject);
var NewHeader: TBinaryFontHeader; { record }
begin
  if SaveChangesQuery then   { allow existing work to be saved first }
    with NewFontForm do
      begin
        { assign "New Font" dialog defaults to those currently in use }
        cbProportional.Checked := FontHeader.Proportional;
        cbNoDefault.Checked := FontHeader.NoDefaultWid;
        SmallWidth.Value := FontHeader.SmallWidth;
        LargeWidth.Value := FontHeader.LargeWidth;
        { execute "New Font" dialog }
        if ShowModal <> idCancel then
          begin
            { produce a header for the font being created }
            NewHeader := DefaultBinaryFontHeader;
            NewHeader.Proportional := cbProportional.Checked;
            NewHeader.NoDefaultWid := cbNoDefault.Checked;
            NewHeader.SmallWidth := SmallWidth.Value;
            NewHeader.LargeWidth := LargeWidth.Value;
            MakeNewFont(NewHeader);  { updates caption }
            AssignGenerationDefaults;
          end;
      end;
end;

procedure TFontEditorForm.MenuOpenClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    OpenFile(OpenDialog.FileName);
end;

procedure TFontEditorForm.OpenFile(FileName:string);
var NewDir:string;
    DLen:integer;
begin
  if not SaveChangesQuery then Exit;
  CurrentFile := FileName;
  try
    { Under Win3.11 ChDir doesn't like what ExtractFilePath produces,
      although it seems ok under Win95!  I had to strip the trailing '\'
      except when changing to a drive's root directory, when \ is reqd. }
    NewDir := ExtractFilePath(CurrentFile);
    DLen := length(NewDir);
    if (NewDir[DLen] = '\') and (DLen > 3) then
      Delete(NewDir,DLen,1);
    ChDir(NewDir);
  finally
  end;
  ReadFontBinary(CurrentFile);
  HistoryNotifyWorkfile(CurrentFile);
  AssignGenerationDefaults;
  IsDirty := False;
  ShowNewFile(CurrentFile);
  EditChar := CodeCombo.Items.Objects[OpenCh] as TCharDefn;
  ShowNewChar;
end;

procedure TFontEditorForm.MenuSaveClick(Sender: TObject);
begin
  if CurrentFile <> '' then
    begin
      WriteFontBinary(CurrentFile);
      IsDirty := False;
      ShowNewFile(CurrentFile);
      HistoryNotifyWorkfile(CurrentFile);
    end
  else MenuSaveAsClick(Sender);
end;

procedure TFontEditorForm.MenuSaveAsClick(Sender: TObject);
begin
  if SaveDialog.Execute then
   begin
     CurrentFile := SaveDialog.FileName;
     MenuSaveClick(Sender);
   end;
end;

procedure TFontEditorForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if IsDirty then
    case MessageDlg('Save font before exit?',mtConfirmation,mbYesNoCancel,0) of
      mrCancel: CanClose := False;
      mrYes:    MenuSaveClick(Sender);
    end;
end;


{ ------------- PIXEL EFFECTS ----------------------------- }

function TFontEditorForm.SmallCol(const Ar:TSmallArray; c:integer):integer;
var r:integer;
begin
  Result := 0;
  for r := pred(SmallHt) downto 0 do   { MSB at bottom }
    Result := Result shl 1 + ord(Ar.Bits[c,r]);
end;

function TFontEditorForm.LargeCol(const Ar:TLargeArray; c:integer):Word;
var r:integer;
begin
  Result := 0;
  for r := pred(LargeHt) downto 0 do   { MSB at bottom }
    Result := Result shl 1 + ord(Ar.Bits[c,r]);
end;

procedure TFontEditorForm.SetCol(var Ar:TSmallArray; c:integer; W:Word);
var r:integer;
begin
  if (c >= 0) and (c < SmallWd) then
    for r := 0 to pred(SmallHt) do
      Ar.Bits[c,r] := (W and (word(1) shl r)) <> 0;
  IsDirty := True;
end;

procedure TFontEditorForm.SetCols(var Ar:TLargeArray; c:integer; W:Word);
var r:integer;
begin
  if (c >= 0) and (c < LargeWd) then
    for r := 0 to pred(LargeHt) do
      Ar.Bits[c,r] := (W and (word(1) shl r)) <> 0;
  IsDirty := True;
end;

procedure TFontEditorForm.MenuDoubleSizeClick(Sender: TObject);
{ Enlarge x2 the pixels between margins in small grid, aligned with left margin
  in large grid.  Ensure that the enlargement doesn't overflow large array. }
var Rs,Cs,r,c:integer;
    Pixel:boolean;
begin
  with EditChar do
    begin
      FillChar(Large.Bits,sizeof(Large.Bits),False);
      for Cs := succ(Small.MarginL) to pred(Small.MarginR) do
       for Rs := 0 to pred(SmallHt) do
         begin
           Pixel := Small.Bits[Cs,Rs];
           r := Rs * 2;
           c := (Cs - succ(Small.MarginL)) * 2 + succ(Large.MarginL);
           if succ(c) >= LargeWd then Continue; { clip if overflow }
           Large.Bits[c,r] := Pixel;
           Large.Bits[c+1,r] := Pixel;
           Large.Bits[c,r+1] := Pixel;
           Large.Bits[c+1,r+1] := Pixel;
         end;
    end;
  LargeGrid.Invalidate;
  IsDirty := True;
end;


procedure TFontEditorForm.MenuBoldClick(Sender: TObject);
{ Smear the large grid horizontally by 1 pixel }
var r,c:integer;
begin
  with EditChar.Large do
    for c := pred(LargeWd) downto 1 do
      for r := 0 to pred(LargeHt) do
        Bits[c,r] := Bits[c,r] or Bits[pred(c),r];
  LargeGrid.Invalidate;
  IsDirty := True;
end;


procedure TFontEditorForm.BloomChar(var ch:TCharDefn);
var W:Word;
                  function Bloom(v:Word):Word;
                  begin
                    Result := v or (v shl 1) or (v shr 1);
                  end;
begin
  with ch do
    begin
      W := Bloom(SmallCol(Small,succ(Small.MarginL)));
      SetCol(Small,Small.MarginL,W);
      W := Bloom(SmallCol(Small,pred(Small.MarginR)));
      SetCol(Small,Small.MarginR,W);
      W := Bloom(LargeCol(Large,succ(Large.MarginL)));
      SetCols(Large,Large.MarginL,W);
      W := Bloom(LargeCol(Large,pred(Large.MarginR)));
      SetCols(Large,Large.MarginR,W);
    end;
end;

procedure TFontEditorForm.MenuBloomClick(Sender: TObject);
begin
  BloomChar(EditChar);
  SmallGrid.Invalidate;
  LargeGrid.Invalidate;
end;

procedure TFontEditorForm.MenuBloomAllClick(Sender: TObject);
var i:integer;
    ch:TCharDefn;
begin
  with CodeCombo.Items do
    for i := 0 to pred(Count) do
      begin
        ch := Objects[i] as TCharDefn;
        BloomChar(ch);
      end;
  SmallGrid.Invalidate;
  LargeGrid.Invalidate;
end;

procedure TFontEditorForm.MarginEraseSmall(var ch:TCharDefn);
var r,c:integer;
begin
  with ch.Small do
    for r := 0 to pred(SmallHt) do
      begin
        for c := 0 to MarginL do Bits[c,r] := False;
        for c := MarginR to pred(SmallWd) do Bits[c,r] := False;
      end;
  IsDirty := True;
end;

procedure TFontEditorForm.MarginEraseLarge(var ch:TCharDefn);
var r,c:integer;
begin
  with ch.Large do
    for r := 0 to pred(LargeHt) do
      begin
        for c := 0 to MarginL do Bits[c,r] := False;
        for c := MarginR to pred(LargeWd) do Bits[c,r] := False;
      end;
  IsDirty := True;
end;

procedure TFontEditorForm.MenuEraseClick(Sender: TObject);
begin
  MarginEraseSmall(EditChar);
  MarginEraseLarge(EditChar);
  SmallGrid.Invalidate;
  LargeGrid.Invalidate;
end;

procedure TFontEditorForm.MenuEraseAllClick(Sender: TObject);
var i:integer;
    ch:TCharDefn;
begin
  with CodeCombo.Items do
    for i := 0 to pred(Count) do
      begin
        ch := Objects[i] as TCharDefn;
        MarginEraseSmall(ch);
        MarginEraseLarge(ch);
      end;
  SmallGrid.Invalidate;
  LargeGrid.Invalidate;
end;


{ ------------- DRAG/DROP MARGIN ADJUSTMENT ---------------------- }

procedure TFontEditorForm.GridDragOver(Sender,Source: TObject; X,Y:Integer;
                                       State:TDragState; var Accept:Boolean);
var c,r: Longint;
begin
  Accept := (Source = MarginPanel) and FontHeader.Proportional;
  if Accept then
    begin
      (Sender as TDrawGrid).MouseToCell(X, Y, c, r);
      case State of
        dsDragEnter: if Sender = SmallGrid then with EditChar.Small do
                         begin
                           DragMarginL := MarginL;  { save original margins }
                           DragMarginR := MarginR;
                           DragLeft := c <= (MarginL+MarginR) div 2;
                           MarginEraseSmall(EditChar);
                           Invalidate;
                         end
                       else with EditChar.Large do
                         begin
                           DragMarginL := MarginL;
                           DragMarginR := MarginR;
                           DragLeft := c <= (MarginL+MarginR) div 2;
                           MarginEraseLarge(EditChar);
                           Invalidate;
                         end;
        dsDragLeave: if Sender = SmallGrid then with EditChar.Small do
                         begin
                           MarginL := DragMarginL;   { restore orig margins }
                           MarginR := DragMarginR;
                         end
                       else with EditChar.Large do
                         begin
                           MarginL := DragMarginL;
                           MarginR := DragMarginR;
                         end;
        dsDragMove: if Sender = SmallGrid then with EditChar.Small do
                       begin
                         MarginL := DragMarginL;
                         MarginR := DragMarginR;
                         if DragLeft and (c >= MarginR) then DragLeft := False;
                         if (c <= MarginL) and not DragLeft then DragLeft := True;
                         if DragLeft then MarginL := pred(c)
                                     else MarginR := succ(c);
                       end
                    else with EditChar.Large do
                       begin
                         MarginL := DragMarginL;
                         MarginR := DragMarginR;
                         if DragLeft and (c >= MarginR) then DragLeft := False;
                         if (c <= MarginL) and not DragLeft then DragLeft := True;
                         if DragLeft then MarginL := pred(c)
                                     else MarginR := succ(c);
                       end;
      end;{case}
      (Sender as TDrawGrid).Invalidate;
    end;
end;

procedure TFontEditorForm.GridDragDrop(Sender, Source: TObject; X, Y: Integer);
{ Margin has been dropped on a grid, so set the margin at that point. }
var c,r: Longint;
begin
  (Sender as TDrawGrid).MouseToCell(X, Y, c, r);
  with EditChar do
    if Sender = SmallGrid then
        if DragLeft then Small.MarginL := pred(c)
                    else Small.MarginR := succ(c)
    else
        if DragLeft then Large.MarginL := pred(c)
                    else Large.MarginR := succ(c);
end;

procedure TFontEditorForm.NullDragOver(Sender,Source:TObject; X,Y:Integer;
                                       State:TDragState; var Accept:Boolean);
begin
  Accept := Sender = Source;
end;


{ -------------------- TEXT GENERATION ------------------------- }
{ This is fairly specific to my application, and probably won't be
  of much interest to most readers. }

procedure TFontEditorForm.Generate;
var DefaultName:TFileName;
    Ini:TIniFile;
begin
  DefaultName := ExtractFileName(CurrentFile);
  Delete(DefaultName,Length(DefaultName)-2,3);  { remove .FB extension }
  PreviewForm.SaveDialog.FileName := DefaultName;
  Screen.Cursor := crHourGlass;
  Ini := TIniFile.Create(GenerateIniName);
  try
    GenSmall := Ini.ReadBool(Section, SmallItem, True);
    GenLarge := Ini.ReadBool(Section, LargeItem, True);
    GenLBloom := Ini.ReadBool(Section, LBloomItem, False);
    GenRBloom := Ini.ReadBool(Section, RBloomItem, False);
    GenIndex := Ini.ReadBool(Section, IndexItem, True);
    GenZeroPad := Ini.ReadBool(Section, ZeroPadItem, True);
    GenHexComment := Ini.ReadBool(Section, HexCommentItem, True);
  finally
    Ini.Free;
  end;
  MakeAsm(DefaultName);
  Screen.Cursor := crDefault;
end;


procedure TFontEditorForm.MakeAsm(DefaultName:string);
const
  LHSIndent = 12;
  HdrTab = 11;
var
  StdIndent,Line:string;
  i,c,ChWid,NoChars,IxSize,Ix,ChNo:integer;

             procedure Emit(s:string);
             begin
               PreviewForm.Memo.Lines.Add(s);
             end;

             procedure StartEmit;
             begin
               PreviewForm.Memo.Clear;
             end;

             procedure EndEmit;
             begin
               PreviewForm.Memo.Modified := False;
             end;

             function PadStr(src:string; wid:integer):string;
             begin
               Result := src;
               while Length(Result) < wid do
                 Result := Result + ' ';
             end;

             procedure AppendLine(value:integer);
             var digits:integer;
             begin
               if GenZeroPad or (value > $9F)
                 then digits := 3
                 else digits := 1+ord(value > $9);
               AppendStr(Line,','+IntToHex(value,digits)+'h');
             end;

             procedure AppendCol(const Ar:TSmallArray; c:integer);
             begin AppendLine(SmallCol(Ar,c));
             end;

             procedure AppendCols(const Ar:TLargeArray; c:integer);
             var W:word;
             begin
               W := LargeCol(Ar,c);
               AppendLine(W shr 8);
               AppendLine(W and $FF);
             end;

             function DbCh(Code:byte):string;
             begin
               if Code = $27 then Result := 'db 27h'
                             else Result := Format('db ''%s''',[chr(Code)]);
             end;

             procedure IndexEtc(prefix:TFileExt);
             var i:integer; abbrev:string[8];
             begin
               if not FontHeader.Proportional then
                 Emit(StdIndent+PadStr('db 1',HdrTab)+'; inter-char gap');
               if GenIndex then
                 begin
                   abbrev := prefix+Copy(DefaultName,1,4);
                   i:=0; Ix:=0; ChNo:=0;
                   while (ChNo < NoChars) and (Ix < 10) do
                     with CodeCombo.Items do
                       begin
                         while (Objects[i] as TCharDefn).IsEmpty do
                           inc(i);
                         if ChNo mod IxSize = 0 then
                           begin
                             if ChNo > 0 then Emit(StdIndent+DbCh((Objects[i] as TCharDefn).Code));
                             Line := 'dw '+abbrev+IntToStr(Ix);
                             if ChNo = 0 then Line := PadStr(Line,HdrTab)+'; table base';
                             Emit(StdIndent+Line);
                             inc(Ix);
                           end;
                         inc(ChNo); inc(i);
                       end;
                   Emit(StdIndent+'db 0');
                 end;
               Emit(StdIndent+';');
               if FontHeader.Proportional then
                 begin
                   Line := StdIndent+'; [code],[width],';
                   if GenLBloom then AppendStr(Line,'[LBloom],');
                   if GenRBloom then AppendStr(Line,'[RBloom],');
                   AppendStr(Line,'[data..]');
                   Emit(Line);
                 end;
             end;
begin
  StartEmit;
  NoChars := CountChars;
  IxSize := NoChars div 10;  if IxSize < 7 then IxSize := 7;
  if DefaultName = '' then DefaultName := 'NoName';
  Emit(Format('; Font %s, generated %s',[DefaultName,DateTimeToStr(Now)]));
  Line := Format('; %d characters: ',[NoChars]);
  if FontHeader.Proportional then
    Emit(Line+'proportional')
  else with FontHeader do
    Emit(Line+Format('size %dx%d, %dx%d',[SmallWidth,SmallHt,LargeWidth,LargeHt]));
  Emit('');
  StdIndent := PadStr('',LHSIndent);
  Emit(StdIndent+'cseg');
  DefaultName := LowerCase(DefaultName);
  DefaultName[1] := UpCase(DefaultName[1]);
  {----------------------------------------}
  if GenSmall then
    begin
      Emit('');
      Emit('; Small version');
      if FontHeader.Proportional
        then Line := 'db 0'
        else Line := Format('db %d',[FontHeader.SmallWidth]);
      Emit(PadStr('F'+DefaultName+':',LHSIndent)+PadStr(Line,HdrTab)+'; width');
      IndexEtc('F');
      Ix:=0; ChNo:=0;
      with CodeCombo.Items do
       for i := 0 to pred(Count) do with Objects[i] as TCharDefn do
         if not IsEmpty then
           begin
             Line := DbCh(Code);
             with Small do
               if FontHeader.Proportional then
                 begin
                   ChWid := MarginR-MarginL-1;
                   AppendStr(Line,Format(',%.2xh',[ChWid]));
                   if GenLBloom then
                     if MarginL < 0 then AppendLine(0)
                                    else AppendCol(Small,MarginL);
                   if GenRBloom then
                     if MarginR >= SmallWd then AppendLine(0)
                                           else AppendCol(Small,MarginR);
                 end;
             for c := succ(Small.MarginL) to pred(Small.MarginR) do
               AppendCol(Small,c);
             if GenHexComment then
               AppendStr(Line,'  ; $'+IntToHex(Code,2));
             if GenIndex and (ChNo mod IxSize = 0) and (Ix < 10) then
               begin
                 Emit(PadStr('F'+Copy(DefaultName,1,4)+IntToStr(Ix)+':',LHSIndent)+Line);
                 inc(Ix);
               end
             else Emit(StdIndent+Line);
             inc(ChNo);
           end;
      Emit(StdIndent+'db 0');
    end;
  {----------------------------------------}
  if GenLarge then
    begin
      Emit('');
      Emit('; Large version');
      if FontHeader.Proportional
        then Line := 'db 0'
        else Line := Format('db %d',[FontHeader.LargeWidth]);
      Emit(PadStr('F2'+DefaultName+':',LHSIndent)+PadStr(Line,HdrTab)+'; width (bytes)');
      IndexEtc('F2');
      Ix:=0; ChNo:=0;
      with CodeCombo.Items do
       for i := 0 to pred(Count) do with Objects[i] as TCharDefn do
         if not IsEmpty then
           begin
             Line := DbCh(Code);
             with Large do
               if FontHeader.Proportional then
                 begin
                   ChWid := MarginR-MarginL-1;
                   AppendStr(Line,Format(',%.2xh',[ChWid]));
                   if GenLBloom then
                     if MarginL < 0 then
                       begin
                         AppendLine(0); AppendLine(0);
                       end
                     else AppendCols(Large,MarginL);
                   if GenRBloom then
                     if MarginR >= LargeWd then
                       begin
                         AppendLine(0); AppendLine(0);
                       end
                     else AppendCols(Large,MarginR);
                 end;
             for c := succ(Large.MarginL) to pred(Large.MarginR) do
               AppendCols(Large,c);
             if GenHexComment then
               AppendStr(Line,'  ; $'+IntToHex(Code,2));
             if GenIndex and (ChNo mod IxSize = 0) and (Ix < 10) then
               begin
                 Emit(PadStr('F2'+Copy(DefaultName,1,4)+IntToStr(Ix)+':',LHSIndent)+Line);
                 inc(Ix);
               end
             else Emit(StdIndent+Line);
             inc(ChNo);
           end;
      Emit(StdIndent+'db 0');
    end;
  Emit('; end');
  Emit('');
  EndEmit;
end;

{---------------------------------------------------------------------}

procedure TFontEditorForm.MenuMakeClick(Sender: TObject);
begin
  Generate;
  PreviewForm.ShowModal;
end;

procedure TFontEditorForm.MenuOptionsClick(Sender: TObject);
begin
  OptionsDlg.ShowModal;
end;

procedure TFontEditorForm.MenuAboutClick(Sender: TObject);
var s: array [0..255] of Char;
begin
  try
   StrPCopy(s,'write.exe '+ExtractFilePath(Application.ExeName)+'readme.wri');
   WinExec(s, SW_SHOWMAXIMIZED);
  except
    ShowMessage('Freeware release 1.69a'#13+'Enjoy at own risk!'#13#13
               +'See README.WRI for details.'#13+'gw@anfi.pacit.tas.gov.au');
  end;
end;


end.
