unit NewCal;

{Derived from the Calender Unit and donated to the public
 domain by John Newlin.  Enjoy!}

interface

uses
  Classes, Controls, Messages, WinTypes, Forms,
  Graphics, WinProcs, StdCtrls, Grids, SysUtils,Calendar,
  Dialogs;

Type
  Str5 = String[5];
  (*
  This function prototype is used to provide a means of placing
  text or a small bit map (20 x 20 is a nice size) into a cell
  at runtime.  Example:

       TForm1 = class(TForm)
         ---
         ---
       protectd
         ---
         Function IsSpecial(Col,Row : Longint;
                            NumDay : integer;
                            Var CellText : Str5;
                            Var Fg,Bg : Tcolor;
                            Var Glyph : TBitmap;
                            State : TGridDrawState) : boolean;
         ---
       End;

  Then implement it in the following fashion:

         Function TForm1.IsSpecial;
         begin
           result := true;
           case NumDay of
              1 : if col = 1 then Glyph := A_Bmp;
             14 : Glyph := AbitMap;
             22 : begin
                    fg := clRed;
                    CellText := 'Doctr';
                  end;
             else result := false;
           end;
         end;
  *)
  TSpecial =
    Function(Col,Row : Longint;
             NumDay : integer;     {Day of the month}
             Var CellText : str5;  {text for cell}
             var Fg,Bg : TColor;   {text colors}
             var Glyph : TBitMap;  {if used then text ignored}
             State : TGridDrawState) : boolean of Object;

  TCCal = class(TCalendar)
  private
    FSpecial : TSpecial;
    FHdrFg : TColor; {row 0 header foreground color}
    FHdrBg : TColor; {row 0 header background color}
    FSelBg : TColor; {color of the selected cell}
  protected
    FOnSpecial : TSpecial;
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect;
                       AState: TGridDrawState); Override;
    Procedure SetHdrFgColor(Color : TColor);
    Procedure SetHdrBgColor(Color : TColor);
    Procedure SetSelBgColor(Color : TColor);
  public
    Property OnSpecial : TSpecial read FOnSpecial write FOnSpecial;
    Constructor Create(Aowner : TComponent); Override;
  published
    Property HdrFg : TColor read FHdrFg write SetHdrFgColor Default clBlack;
    Property HdrBg : TColor read FHdrBg write SetHdrBgColor Default clWhite;
    Property SelBg : TColor read FSelBg write SetSelBgColor Default clNavy;
  end;

Var
  Cal : TCCal;

Procedure Register;

implementation

Procedure Register;
Begin
  RegisterComponents('Samples',[TCCal]);
End;

Procedure TCCal.SetHdrFgColor(Color:TColor);
Begin
  if Color <> FHdrFg then
    begin
      FHdrFg := Color;
      Invalidate;
    end;
End;

Procedure TCCal.SetHdrBgColor(Color:TColor);
Begin
  if Color <> FHdrBg then
    begin
      FHdrBg := Color;
      Invalidate;
    end;
End;

Procedure TCCal.SetSelBgColor(Color:TColor);
Begin
  if Color <> FSelBg then
    Begin
      FSelBg := Color;
      Invalidate;
    End;
End;

function getNum(Num: string): Integer;
var
  v: integer;
  code: integer;
begin
   val(Num, v, Code);
   if code = 0 then Result := v else Result := 0;
end;

procedure TCCal.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);
var
  Glyph : TBitMap;
  TempRect: TRect;
  DayNum : integer;
  TheText,DayText : string[5];
  Special : boolean;
  WeekDay : integer;
  Fg,Bg,FgSave,BgSave : Tcolor;
  Ctext : Str5;

Label TheEnd;

begin
  FgSave := Canvas.Font.Color;
  BgSave := Canvas.Brush.Color;
  TheText := CellText[ACol, ARow];
  with ARect, Canvas do
    begin
      if arow = 0 then
        begin
          Font.Color := FHdrFg;
          Brush.Color :=  FHdrBg;
          TheText := CellText[ACol,ARow];
          Font.Color := FHdrFg;
          TextRect(ARect, Left + (Right - Left - TextWidth(TheText)) div 2,
           Top + (Bottom - Top - TextHeight(TheText)) div 2, TheText);
          goto TheEnd;
        end;
      DayText := CellText[ACol, ARow];
      WeekDay := DayofWeek(CalendarDate);
      DayNum := GetNum(DayText);
      if gdSelected in Astate then Brush.Color := FSelBg;
      TextRect(ARect, Left, Top, DayText);  { Draw the day at the upper left corner }
      Special := false;
      (* You could put your special criteria right here as
      special := (DayNum = 15) and (Acol = 0);
       or as
      special := (CellText[Acol,0] = 'Sun') and (DayNum = 15);
      *)
      if Assigned(FOnSpecial) then
        begin
          Glyph := nil;
          bg := Brush.Color;
          fg := Font.Color;
          Special := OnSpecial(Acol,Arow,DayNum,Ctext,Fg,Bg,
                               Glyph,Astate);
        end;
      if special then
        begin
          Font.Color := Fg;
          Brush.color := Bg;
          {make a small rectangle within the larger one}
          TempRect := Rect(Left, Bottom - TextHeight('W')+1, Right, Bottom);
          if Glyph = nil then
            TextRect(TempRect, TempRect.Right - TextWidth(Ctext),
                     TempRect.Top, CText) else
                BrushCopy(Bounds({tempRect.Left + 14}
                 tempRect.right - Glyph.Width,
                 TempRect.Bottom - Glyph.Height,
                 Glyph.Width, Glyph.Height),
                 Glyph, Bounds(0, 0, Glyph.Width, Glyph.Height),
                 Glyph.Canvas.Pixels[0, Glyph.Height - 1]);
        end;
    end;
  TheEnd:
  Canvas.Font.Color := FgSave;
  Canvas.Brush.Color := BgSave;
end;


Constructor TCCal.Create;
Begin
  Inherited Create(AOwner);
  FhdrFg := clBlack;
  FhdrBg := clSilver;
  FSelBg := clBlue;
End;

end.
