unit Options;    { Code generation options dialog }

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
     StdCtrls, ExtCtrls, IniFiles;

const
  GenerateIniName = 'win.ini';
  Section = 'FontEditor';        { strings for the INI file }
  SmallItem = 'Small';
  LargeItem = 'Large';
  LBloomItem = 'LeftBloom';
  RBloomItem = 'RightBloom';
  IndexItem = 'Index';
  ZeroPadItem = 'ZeroPadding';
  HexCommentItem = 'HexComments';


type
  TOptionsDlg = class(TForm)
    GroupBox1: TGroupBox;
    cbSmall: TCheckBox;
    cbLarge: TCheckBox;
    cbLBloom: TCheckBox;
    cbRBloom: TCheckBox;
    cbIndex: TCheckBox;
    cbZeroPad: TCheckBox;
    cbHexComment: TCheckBox;
    Ok: TBitBtn;
    Cancel: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OkClick(Sender: TObject);
    procedure CancelClick(Sender: TObject);
  private
    { Private declarations }
    Ini:TIniFile;
    procedure ReadSettings;
  public
    { Public declarations }
  end;

var
  OptionsDlg: TOptionsDlg;

implementation

{$R *.DFM}


procedure TOptionsDlg.FormCreate(Sender: TObject);
begin
  Ini := TIniFile.Create(GenerateIniName);
  ReadSettings;
end;

procedure TOptionsDlg.FormDestroy(Sender: TObject);
begin
  Ini.Free;
end;

procedure TOptionsDlg.ReadSettings;
begin
  cbSmall.Checked := Ini.ReadBool(Section, SmallItem, True);
  cbLarge.Checked := Ini.ReadBool(Section, LargeItem, True);
  cbLBloom.Checked := Ini.ReadBool(Section, LBloomItem, False);
  cbRBloom.Checked := Ini.ReadBool(Section, RBloomItem, True);
  cbIndex.Checked := Ini.ReadBool(Section, IndexItem, True);
  cbZeroPad.Checked := Ini.ReadBool(Section, ZeroPadItem, True);
  cbHexComment.Checked := Ini.ReadBool(Section, HexCommentItem, True);
end;

procedure TOptionsDlg.FormShow(Sender: TObject);
begin
  ReadSettings;
end;

procedure TOptionsDlg.OkClick(Sender: TObject);
begin
  Ini.WriteBool(Section, SmallItem, cbSmall.Checked);
  Ini.WriteBool(Section, LargeItem, cbLarge.Checked);
  Ini.WriteBool(Section, LBloomItem, cbLBloom.Checked);
  Ini.WriteBool(Section, RBloomItem, cbRBloom.Checked);
  Ini.WriteBool(Section, IndexItem, cbIndex.Checked);
  Ini.WriteBool(Section, ZeroPadItem, cbZeroPad.Checked);
  Ini.WriteBool(Section, HexCommentItem, cbHexComment.Checked);
end;

procedure TOptionsDlg.CancelClick(Sender: TObject);
{ Restore the checkboxes to settings prior to Cancel, so that
  they're always valid for inspection by the Main form }
begin
  ReadSettings;
end;

end.
