unit Print;
interface
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Printers, StdCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
{ copy the following Type from the help file - API}
Type
   TTextMetric = record
      tmHeight: Integer;
      tmAscent: Integer;
      tmDescent: Integer;
      tmInternalLeading: Integer;
      tmExternalLeading: Integer;
      tmAveCharWidth: Integer;
      tmMaxCharWidth: Integer;
      tmWeight: Integer;
      tmItalic: Byte;
      tmUnderlined: Byte;
      tmStruckOut: Byte;
      tmLastChar: Byte;
      tmDefaultChar: Byte;
      tmBreakChar: Byte;
      tmPitchAndFamily: Byte;
      tmCharSet: Byte;
      tmOverhang: Integer;
      tmDigitizedAspectX: Integer;
      tmDigitizedAspectY: Integer;
      end;

{ Define the external, far API function }
Function GetTextMetrics(Hand:Integer; var t:TTextMetric):Bool;
              far;external 'GDI';

Function GetActiveWindow:Integer;far;external 'User';
Function GetDeviceCaps(Hand, Index :Integer): integer;
              far;external 'GDI';

procedure TForm1.Button2Click(Sender: TObject);
begin
   Halt;
end;

procedure ChangeFont(const FontName : string;
                     const FontSize,
                           BottomPrevLine,
                           Offset2BaseLineBiggestFont : integer;
                     var TopOfFont : integer);
var RetVal : Bool;
    Metrics : TTextMetric;
begin
{ This routine is called whenever a font changes in size or name.
  It assumes that the baseline has been determined for the
  largest font to printed on the current line.  Its sole purpose
  is to determine the vertical position of the characters to be
  printed.
}
   Printer.Canvas.Font.Name := FontName;
   Printer.Canvas.Font.Size := FontSize;

{ It appears necessary to output 'something' (even a null '')
  to the canvas in order that GetMetrics recognizes the new font.
  Is there a better way to do it?
}
   Printer.Canvas.TextOut(0,0,'');
   RetVal := GetTextMetrics(Printer.Handle, Metrics);
   TopOfFont := BottomPrevLine
              + Offset2BaseLineBiggestFont
              - Metrics.tmAscent;
end;

function GetPrinterDotsPerInch : integer;
const GET_HOR_BITS = 88; { 88 is the 'magic # to get horiz DotsPerInch }
begin
    Result := GetDeviceCaps(Printer.Handle, GET_HOR_BITS);
end;

function Tab(const DotsPerInch : integer; const TabVal : real) : integer;
begin
   Tab := trunc(DotsPerInch * TabVal);	{ TabVal is in inches }
end;

procedure OutputString(const HorPos : integer;
                       const AlignmentType : char;
                       const Offset2BottomBiggestFont :integer;
                       const Str : string;
                       const NewLine : boolean;
                       var   TopOfFont : integer;
                       var   BottomPrevLine : integer);
const PrevHorPos : integer = 0;   { create a Static variable }
var   LenOfStr, TmpHorPos : Integer;
begin
   LenOfStr := Printer.Canvas.TextWidth(Str);
   if UpCase(AlignmentType) = 'R' then        { right justify }
      TmpHorPos := HorPos - LenOfStr
   else if UpCase(AlignmentType) = 'C' then   { centre about HorPos }
      TmpHorPos := HorPos - (LenOfStr div 2)
   else
      TmpHorPos := HorPos;                    { left justify }

{ Check printing position does not overlap previously printed text }
   if TmpHorPos > PrevHorPos then
      PrevHorPos := TmpHorPos;
   Printer.Canvas.TextOut(PrevHorPos, TopOfFont, Str);
   If NOT NewLine then
      PrevHorPos := PrevHorPos + LenOfStr
   else
      begin
      PrevHorPos := 0;
{ Adjust variables for next line }
      BottomPrevLine := BottomPrevLine + Offset2BottomBiggestFont;
      TopOfFont := TopOfFont + Offset2BottomBiggestFont;
      end;
end;

procedure GetSpecsMaxFont (const    FontName : string;
                           const MaxFontSize : Integer;
                            var Offset2BaseLineBiggestFont,
                                Offset2BottomBiggestFont : Integer);
var RetVal : Bool;
    Metrics : TTextMetric;
begin
   { This routine is called whenever the largest font for a line
     is changed. It effectively sets variables that are used in
     line spacing to accomodate the largest font.
   }
   Printer.Canvas.Font.Name := FontName;
{ Change the font size to maximum size to be used on line }
   Printer.Canvas.Font.Size := MaxFontSize;
   Printer.Canvas.TextOut(0,0,'');
   RetVal := GetTextMetrics(Printer.Handle, Metrics);
   Offset2BaseLineBiggestFont := Metrics.tmExternalLeading
                   + Metrics.tmAscent;
   Offset2BottomBiggestFont := Offset2BaseLineBiggestFont
                             + Metrics.tmDescent;
end;

procedure TForm1.Button1Click(Sender: TObject);
const  NEW_LINE = True; NOT_NEW_LINE = FALSE;
Var
    BottomPrevLine,
    Offset2BottomBiggestFont,
    Offset2BaseLineBiggestFont,
    DotsPerInch,
    FontSize,
    PageWidth,
    PageHeight,
    TopOfFont : Integer;
    FontName, Msg1, Msg2, Msg3, Msg4, Msg5 : STRING;
begin
   PageWidth := Printer.PageWidth;     { not used at the moment }
   PageHeight := Printer.PageHeight;
   DotsPerInch := GetPrinterDotsPerInch;  { used by Tab function }
   Msg1 := 'This is 1 gy';    Msg2 := 'Here''s 2 gy';
   Msg3 := 'and THREE gy';
   Msg4 := 'finally FOUR gy'; Msg5 := 'and Five gy';

   BottomPrevLine := 0;             { There was no previous line  }

   Printer.BeginDoc;
{ Determine the baseline & bottom of the font as offset from the space
  occupied by the font and the external leading.  The values used are:
           - external leading ( the inter line spacing added above
             the characters as recommended by the font designer
           - the ascent ( the space from the top of the largest character
             to the line on which the characters like "H" are placed)
           - the descent (the space used below the baseline for
             letters like "y" and "g")
           - the total space used by the font is the sum of the above
             3 values
           - Offset2BaseLineBiggestFont
            (the distance from the previous line to the base line used
             by the characters like "H")
           - TopOfFont (the absolute position on the canvas used
             as the Y coordinate in TextOut.  It would be the Y-coord
             for the upper left of the letter "H".)
  Note: This module only works if you determine the specs for largest
        font on a line before printing the first string.}

   FontName := 'Lucida Calligraphy';
   FontSize := 30;
   GetSpecsMaxFont(FontName, FontSize, Offset2BaseLineBiggestFont,
                                       Offset2BottomBiggestFont);

{ now print out something with Font Size = 12 pt }
   FontSize := 12;
   ChangeFont(FontName, FontSize, BottomPrevLine,
                        Offset2BaseLineBiggestFont, TopOfFont);
{ using 0 as the value for HorPos prints at the leftmost position
  or immediately after the previous string on the line }
   OutputString(0, 'L', Offset2BottomBiggestFont, Msg1, NOT_NEW_LINE,
                                            TopOfFont, BottomPrevLine);

   FontSize := 30;
   ChangeFont(FontName, FontSize, BottomPrevLine,
                        Offset2BaseLineBiggestFont, TopOfFont);
   OutputString(0, 'L', Offset2BottomBiggestFont, Msg2, NOT_NEW_LINE,
                                            TopOfFont, BottomPrevLine);

   FontSize := 12;
   ChangeFont(FontName, FontSize, BottomPrevLine,
                        Offset2BaseLineBiggestFont, TopOfFont);
   OutputString(0, 'L', Offset2BottomBiggestFont, Msg3, NEW_LINE,
                                            TopOfFont, BottomPrevLine);

{ Test font change and different font size mix }
   FontName := 'Times New Roman';
   FontSize := 20;
   GetSpecsMaxFont(FontName, FontSize, Offset2BaseLineBiggestFont,
                                       Offset2BottomBiggestFont);

   FontSize := 20;
   ChangeFont(FontName, FontSize, BottomPrevLine,
                        Offset2BaseLineBiggestFont, TopOfFont);
   OutputString(0, 'L', Offset2BottomBiggestFont, Msg5, NOT_NEW_LINE,
                                            TopOfFont, BottomPrevLine);

   FontSize := 8;
   ChangeFont(FontName, FontSize, BottomPrevLine,
                        Offset2BaseLineBiggestFont, TopOfFont);
   OutputString(0, 'L', Offset2BottomBiggestFont, Msg4, NEW_LINE,
                                            TopOfFont, BottomPrevLine);

{ Test left justify and tab, since the rest of test involves
  the same font and size, only ChangeFont & GetSpecsMaxFont once}
   FontSize := 12;
   GetSpecsMaxFont(FontName, FontSize, Offset2BaseLineBiggestFont,
                                       Offset2BottomBiggestFont);
   ChangeFont(FontName, FontSize, BottomPrevLine,
                        Offset2BaseLineBiggestFont, TopOfFont);
   OutputString(Tab(DotsPerInch,0.5), 'L', Offset2BottomBiggestFont,
                'AbCdEfGh', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,1.5), 'L', Offset2BottomBiggestFont,
                'AbCdEfGh', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,2.5), 'L', Offset2BottomBiggestFont,
                'AbCdEfGh', NEW_LINE, TopOfFont, BottomPrevLine);

   OutputString(Tab(DotsPerInch,0.5), 'L', Offset2BottomBiggestFont,
                'AbCdEfGh', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,1.5), 'L', Offset2BottomBiggestFont,
                'AbCdEfGh', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,2.5), 'L', Offset2BottomBiggestFont,
                'AbCdEfGh', NEW_LINE, TopOfFont, BottomPrevLine);

{ Test right justify }
   OutputString(Tab(DotsPerInch,1.5), 'R', Offset2BottomBiggestFont,
                '12.00', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,2.5), 'R', Offset2BottomBiggestFont,
                '123.00', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,3.5), 'R', Offset2BottomBiggestFont,
                '1456.00', NEW_LINE, TopOfFont, BottomPrevLine);

   OutputString(Tab(DotsPerInch,1.5), 'R', Offset2BottomBiggestFont,
                '1456.00', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,2.5), 'R', Offset2BottomBiggestFont,
                '12.00', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,3.5), 'R', Offset2BottomBiggestFont,
                '123.00', NEW_LINE, TopOfFont, BottomPrevLine);

{ Test centring }
   OutputString(Tab(DotsPerInch,1.5), 'C', Offset2BottomBiggestFont,
                '12.00', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,2.5), 'C', Offset2BottomBiggestFont,
                '123.00', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,3.5), 'C', Offset2BottomBiggestFont,
                '1456.00', NEW_LINE, TopOfFont, BottomPrevLine);

   OutputString(Tab(DotsPerInch,1.5), 'C', Offset2BottomBiggestFont,
                '1456.00', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,2.5), 'C', Offset2BottomBiggestFont,
                '12.00', NOT_NEW_LINE, TopOfFont, BottomPrevLine);
   OutputString(Tab(DotsPerInch,3.5), 'C', Offset2BottomBiggestFont,
                '123.00', NEW_LINE, TopOfFont, BottomPrevLine);

   Printer.EndDoc
end;

end.
