{-------------------------------------------------------}
{                                                       }
{Delphi VCL to add a program group and add program items}
{ by Andrew McLean                                      }
{                                                       }
{ Version 1.0a - 7th Jan 1996                           }
{ Initial release.                                      }
{                                                       }
{ Version 1.1a - 18th Jan 1996                          }
{ Added two properties: programGroup                    }
{                       programItems                    }
{ There is now just one method to make the group:       }
{   makeGroup                                           }
{                                                       }
{               THIS IS A FREE COMPONENT!!              }
{                                                       }
{  If you modify this please let me know.               }
{                                                       }
{  My web: http://www.aerosoft.com.au                   }
{          http://www.aerosoft.com.au/delphi            }
{          http://www.aerosoft.com.au/delphi/progman    }
{                                                       }
{  You can contact me via the 'contact aerosoft' link   }
{                                                       }
{  If anyone knows how to hide the other DDE properties }
{  I'd be most interested in hearing from you.          }
{                                                       }
{-------------------------------------------------------}

{-------------------------------------------------------}
{                                                       }
{ Reference Material:                                   }
{ PC Magazine - Turbo Pascal for Windows                }
{ Neil J. Rubenking                                     }
{ Ziff-Davis Press                                      }
{                                                       }
{ Page: 743-753                                         }
{                                                       }
{-------------------------------------------------------}

unit Progman;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes,
  DdeMan;

type
  TActivateType = (activateNormalSize,activateAndIconize,activateMaximize,iconizeNoActivate);
  EItemError= class(Exception);

  TProgMan = class(TDdeClientConv)
  private
    { Private declarations }
    FOnGroupError,
    FOnItemError: TNotifyEvent;

    FprogramGroup: string;
    FprogramItems: TstringList;

    FshowGroup: TActivateType;
    procedure setItems(TS: TStringList);

    procedure createGroup(name: string);
    procedure addItem(cmdLine,itemName,iconPath: string);
    procedure openProgramManager;
    procedure closeProgramManager;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure makeGroup;

  published
    {properties and events}

    property programItems: TStringList read FprogramItems write setItems;
    property programGroup: string read FprogramGroup write FprogramGroup;

    property ShowGroup: TActivateType read FshowGroup write FshowGroup;
    property OnGroupError: TNotifyEvent read FOnGroupError write FOnGroupError;
    property OnItemError: TNotifyEvent read FOnItemError write FOnItemError;
  end;

procedure Register;

implementation


procedure Register;
begin
  RegisterComponents('Andrew', [TprogMan]); {yes, change it to whatever}
end;


constructor TProgMan.Create(AOwner: TComponent);
begin
  inherited create(aOwner);
  FshowGroup:=activateNormalSize;

  FprogramItems:=TStringList.Create;
end;


destructor TProgMan.Destroy;
begin
  closeLink;
  FprogramItems.free;
  inherited Destroy;
end;

procedure TProgMan.setItems(TS: TStringList);
begin
  {We must assign a copy of the string because the 'write' method
  in the property declaration only assigns points to the strings and
  not a copy of them.}
  FprogramItems.assign(TS);
end;

procedure TProgMan.openProgramManager;
begin
  connectMode:=ddeAutomatic;
  DDEservice:='progman';   {Delphi BUG: DDEservice is never set (or is it my fault!)}
  SetLink('progman', DDETopic);
  openLink;
end;


procedure TProgMan.closeProgramManager;
begin
  closeLink;
end;


procedure TProgMan.createGroup(name: string);
var
  macro: string;
  cmd: array[0..255] of Char;
  i: integer;
begin
  {create a program group using DDE}
  macro:=format('[CreateGroup(%s)]',[name])+#13#10;
  strPCopy(cmd,macro);

  if not executeMacro(cmd,false) then
  begin
    if assigned(FOnGroupError) then
      FOnGroupError(Self);
  end;

  {Setup display state}
  case FshowGroup of
    activateNormalSize: i:=1;
    activateAndIconize: i:=2;
    activateMaximize: i:=3;
    iconizeNoActivate: i:=7;
  end;

  macro:=format('[ShowGroup(%s,%d)]',[name,i])+#13#10;
  strPCopy(cmd,macro);
  executeMacro(cmd,false);
end;


procedure TProgMan.addItem(cmdLine,itemName,iconPath: string);
var
  macro: string;
  cmd: array[0..255] of Char;
begin
  {add program items using DDE}

  macro:='[AddItem('+cmdLine;

  if itemName<>'' then
    macro:=macro+','+itemName;

  if iconPath<>'' then
    macro:=macro+','+iconPath;

  macro:=macro+')]'+#13#10;

  strPCopy(cmd, macro);

  if not executeMacro(cmd,false) then
    if assigned(FOnItemError) then
      FOnItemError(Self);
end;


procedure TProgMan.makeGroup;
var
  i: integer;
  s1,s2,s3: string;

  procedure parseLine(sx: string; var s1,s2,s3: string);
  var
    s: array[1..3] of string;
    i: integer;
    done: boolean;

    function comma(st: string): integer;
    begin
      comma:=pos(',',st);
    end;

  begin
    for i:=1 to 3 do
      s[i]:='';

    i:=1;
    done:=false;
    repeat
      if comma(sx)<>0 then
      begin
        s[i]:=copy(sx,1,comma(sx)-1);
        delete(sx,1,comma(sx));
      end
      else
      begin
        s[i]:=sx;
        done:=true;
      end;
      inc(i);
    until done;

    s1:=s[1];
    s2:=s[2];
    s3:=s[3];
  end;

begin
  openProgramManager;
  createGroup(FprogramGroup);


  with FprogramItems do
  for i:=0 to count-1 do
  begin
    parseLine(strings[i],s1,s2,s3);
    addItem(s1,s2,s3);
  end;

  closeProgramManager;
end;


end.
