{Record Label Component

 Designer:  Craig Ward, 100554.2072@compuserve.com
 Date:      2/1/96
 Version:   1.3


 Function:  Displays the current position in a data-set


 Calling:   To update the current record position the developer must
            call something similar to:

                 recordPosition1.Dataset := Table1;
                 recordPosition1.UpdateLabel;

            Of course, the dataset could be specified in the object-
            inspector. I've found that the best place to call the updateLabel
            procedure from is the OnDataChange event for a TDataSource component.


 Notes:     Some of this code is derived from an un-named source who contributed it
            to a compuserve forum.


 Update:    The component possesses two new properties:
             [1] RecordNo           - this stores the current record number of the datatset
             [2] FilteredTable      - this is a bool used in the determination of the label (see below)


 Filters:   The previous version of the component would not work correctly if used on tables
            that contained a filter (it would still present the full record-count). However,
            the component now possesses a boolean property to take into account filters.

            If the table will contain an active filter, then set this property to true BEFORE
            setting the filter. For example:

             recordPosition1.FilteredTable := true;
             << filter code goes here >>

            If this property is set to true then the component will present only the
            record number (ie: "Record n"). This has been designed to reflect the behaviour of Paradox
            on filtered tables.

            Obviously you will need to set this property to false when the filter is removed:

             recordPostion1.FilteredTable := false;
             << filter code goes here >>


 Thanks:    Thanks to Martin Brekhof (100424.716@compuserve.com) for additions to the component.
*******************************************************************************}
unit Rck;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBiProcs, DBiTypes, StdCtrls, DBTables, DBiErrs;


type
 TRecordPosition = class(TLabel)
  private
    { Private declarations }
    FDataSet: TDataSet;
    FrecordLabel: String;
    FRecordNo : LongInt;
    FIsFilter: boolean;
    procedure SetDataset(Value: TDataset);
    procedure SetrecordLabel(Value: string);
    procedure SetRecordNo( Value : LongInt );
    procedure SetIsFilter(value: boolean);
    function FindRecordNumber: longint;
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure UpdateLabel;
    constructor Create(aOwner: TComponent); override;
  published
    { Published declarations }
    property Dataset: TDataset read FDataset write SetDataset;
    property RecordLabel: string read FRecordLabel write SetRecordLabel;
    property RecordNo: LongInt read FRecordNo write SetRecordNo;
    property FilteredTable: boolean read FIsFilter write SetIsFilter;
end;

procedure Register;

{****************************************************************************}
implementation



{procedure to register this as a custom component}
procedure Register;
begin
  RegisterComponents('Samples', [TRecordPosition]);
end;

constructor TRecordPosition.Create(aOwner: TComponent);
begin
 inherited Create(aOwner);
  FRecordLabel := 'Record';
  Caption := FRecordLabel;
  FRecordNo := 0;
  FIsFilter := false;
end;

{set the table name}
procedure TRecordPosition.SetDataset(Value: TDataset);
begin
 if value <> FDataset then
  begin
   FDataset := Value;
  end;
end;

{set the record type}
procedure TRecordPosition.SetrecordLabel(Value: string);
begin
 if value <> FrecordLabel then
  begin
   FRecordLabel := Value;
  end;
end;

{proc to set Property RecordNo}
procedure TRecordPosition.SetRecordNo(Value : LongInt);
begin
  if value <> FrecordNo then
  begin
   FRecordNo := Value;
  end;
end;

{is the table filtered?}
procedure TRecordPosition.SetIsFilter(value: boolean);
begin
 if value <> FIsFilter then
  begin
   FIsFilter := value;
  end;
end;

{find record number}
function TRecordPosition.FindRecordNumber: longint;
var
 cP: CurProps;
 rP: RECProps;
 DBRes: DBiResult;
begin
 {Return 0 if dataset is not Paradox or dBase}
 Result := 0;

 with FDataset do
  begin

   if state = dsInactive then  exit;

   {we need to make this call to grab the cursor's iSeqNums}
   DBRes := DBiGetCursorProps(Handle,cP);
   if DBRes <> DBIERR_NONE then exit;

   {synchronize the BDE cursor with the dataset's cursor}
   UpdateCursorPos;

   {fill rP with the current record's properties}
   DBRes := DBiGetRecord(Handle,DBiNOLOCK,nil,@rP);
   if DBRes <> DBIERR_NONE then exit;

   {what kind of dataset are we looking at?}
   case cP.iSeqNums of
    0:
     result := rP.iPhyRecNum; {dBase}
    1:
     result := rP.iSeqNum;    {Paradox}
   end;
  end;
end;


{proc to update the label}
procedure TRecordPosition.UpdateLabel;
var
 li: ^longint;
 s: ^string;
begin
 try
  New(s);
  New(li);

 li^ := FindRecordNumber;

 {set record-number property}
 Self.RecordNo := li^;

 {find label}
 s^ := FRecordLabel;

 {display record position}
 case FIsFilter of
  {no filter}
  false:
   self.caption := s^+' '+IntToStr(li^)+' of '+IntToStr(FDataset.RecordCount);
  {filter}
  true:
   self.caption := s^+' '+IntToStr(li^);
 end;

 finally
  Dispose(s);
  Dispose(li);
 end;

end;

end.
