{-----------------------------------------------------------------------------
  Unit     : RU_TEST.PAS

  Author   : Rob McGregor, 73122.3125@compuserve.com

  Comments : Implementation of a modified "Rollup" form class.
 -----------------------------------------------------------------------------}

unit Ru_test;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, Menus, ExtCtrls, StdCtrls, ColorGrd;

const
  ROLL_AMOUNT = 15;          { Sets the rollup / rolldown speed}
  MOUSE_MOVE  = $F012;       { Used for dragging the window from fake caption }

type
  TfrmRollupTest = class(TForm)
    btnClose: TSpeedButton;
    btnRollDown: TSpeedButton;
    btnRollUp: TSpeedButton;
    btnMenu: TSpeedButton;
    PopupMenu1: TPopupMenu;
    mnuChooseColor: TMenuItem;
    mnuChooseFont: TMenuItem;
    ComboBox1: TComboBox;
    Label1: TLabel;
    ColorDialog1: TColorDialog;
    FontDialog1: TFontDialog;
    Edit1: TEdit;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure btnRollDownClick(Sender: TObject);
    procedure btnRollUpClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
                            Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState;
                            X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
                          Shift: TShiftState; X, Y: Integer);
    procedure btnMenuClick(Sender: TObject);
    procedure mnuChooseColorClick(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure mnuChooseFontClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

  private
    { Private Variables }

    rcTitleBar       : TRect;       { Title bar rectangle      }
    nMaxClientHeight : integer;     { Max height for rollup    }
    bDragging        : boolean;     { State variable           }
    szCaption        : PChar;       { The caption string       }
    nCaptionLength   : integer;     { The length of the string }


    { Private Methods }

    function  CreateDCLogFont (dc: HDC; nHeight: integer; yStyle: byte): HFont;
    procedure DrawTitleBar;
    procedure DrawWindowBorder;
    procedure InitRollupData;
    function  IsPointInFakeCaption (x, y: integer): boolean;

  public

  end;

var
  frmRollupTest: TfrmRollupTest;

implementation

{$R *.DFM}

{-----------------------------------------------------------------------------
   Procedure : CreateDCLogFont ()

   Purpose   : Creates a logical font of the typestyle and size desired,
               selects it into the device context.

   Returns   : Handle to the new logical font.

   Comments  : The logical font must be destroyed using DeleteObject after
               use!!
 -----------------------------------------------------------------------------}
function TfrmRollupTest.CreateDCLogFont (dc      : HDC;
                                     nHeight : integer;
                                     yStyle  : byte): HFont;
var
  sz : array [0..75] of char;
  lf : TLogFont;
  hf : HFont;
  i  : integer;

begin
  { select a font and fill in the structure }
  with lf do
  begin
    lfHeight         := nHeight;
    lfWidth          := 0;
    lfEscapement     := 0;
    lfOrientation    := 0;
    lfWeight         := FW_THIN;
    lfItalic         := 0;
    lfUnderline      := 0;
    lfStrikeOut      := 0;
    lfCharSet        := 0;
    lfOutPrecision   := OUT_TT_ONLY_PRECIS;
    lfClipPrecision  := CLIP_TT_ALWAYS;
    lfQuality        := DEFAULT_QUALITY;
    lfPitchAndFamily := DEFAULT_PITCH or yStyle;
  end;

  { Create the font }
  hf := CreateFontIndirect (lf);
  SelectObject (dc, hf);

  result := hf;

end;  { CreateDCLogFont }


{-----------------------------------------------------------------------------
   Procedure : DrawTitleBar ()

   Purpose   : Draws a fake mini-caption bar on the top of the form.

   Comments  :
 -----------------------------------------------------------------------------}
procedure TfrmRollupTest.DrawTitleBar;
var
  dc      : HDC;
  bActive : boolean;
  hf      : HFont;

begin
  with canvas do
  begin
    dc := GetDC (self.handle);

    { Is this rollup active? }
    bActive := (screen.ActiveForm = self);

    if (bActive) then
    begin
      brush.color := GetSysColor (COLOR_ACTIVECAPTION);
      SetTextColor (dc, GetSysColor (COLOR_CAPTIONTEXT));
    end
    else begin
      brush.color := GetSysColor (COLOR_INACTIVECAPTION);
      SetTextColor (dc, GetSysColor (COLOR_INACTIVECAPTIONTEXT));
    end;

    { Draw the fake caption bar }
    FillRect (rcTitleBar);

    { Draw the caption text }
    hf := CreateDCLogFont (dc, 15, FF_SWISS);
    SetBkMode (dc, TRANSPARENT);
    DrawText (dc, szCaption, nCaptionLength, rcTitleBar, DT_LEFT);

    ReleaseDC (self.handle, dc);
    DeleteObject (hf);       { Kill the logical font }

    { Make sure the titlebar buttons aren't painted over by the caption! }
    btnClose.refresh;
    btnRollup.refresh;
    btnRolldown.refresh;
  end;
end;  { DrawTitleBar }


{-----------------------------------------------------------------------------
   Procedure : DrawWindowBorder ()

   Purpose   : Draws a 3d border around the window.

   Comments  :
 -----------------------------------------------------------------------------}
procedure TfrmRollupTest.DrawWindowBorder;
var
  nCH, nCW: integer;

begin
  with canvas do
  begin
    nCW := clientwidth - 1;
    nCH := clientheight - 1;

    pen.mode  := pmCopy;
    pen.color := clBlack;

    { Outline }
    moveto (nCW, 0); lineto (nCW, nCH + 1);           { Right Side }
    moveto (0, nCH); lineto (nCW, nCH);               { Bottom }

    pen.color := clBtnFace;
    moveto (0, 0); lineto (0, nCH);                   { Left Side }
    moveto (0, 0); lineto (nCW, 0);                   { Top }
    moveto (2, 1); lineto (2, nCH - 1);               { Left Side }
    moveto (2, 2); lineto (nCW - 1, 2);               { Top }

    { Highlight / Shadow }
    pen.color := clWhite;
    moveto (1, 1); lineto (1, nCH);                   { Left Side }
    moveto (1, 1); lineto (nCW, 1);                   { Top }

    pen.color := clGray;
    moveto (nCW - 1, 2); lineto (nCW - 1, nCH);       { Right Side }
    moveto (2, nCH - 1); lineto (nCW - 1, nCH - 1);   { Bottom }
  end;
end;


{-----------------------------------------------------------------------------
   Procedure : InitRollupData ()

   Purpose   : Initializes any custom data for this rollup.
 -----------------------------------------------------------------------------}
 procedure TfrmRollupTest.InitRollupData;
 begin
   ComboBox1.items.add ('German');
   ComboBox1.items.add ('Greek');
   ComboBox1.items.add ('French');
   ComboBox1.items.add ('English');
   ComboBox1.items.add ('Spanish');
   ComboBox1.items.add ('Eskimo');
   ComboBox1.sorted := true;
   ComboBox1.itemindex := 0;
   ComboBox1Change (self);

 end;  { InitRollupData }


{-----------------------------------------------------------------------------
   Function : IsPointInFakeCaption ()

   Returns  : True if point (x, y) is in the caption rect, false otherwise.
 -----------------------------------------------------------------------------}
function TfrmRollupTest.IsPointInFakeCaption (x, y: integer): boolean;
begin
  result := PtInRect (rcTitleBar, Point (x, y));

end;   { IsPointInFakeCaption }



{****************************}
{*                          *}
{*  End of private methods  *}
{*                          *}
{****************************}


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.FormCreate(Sender: TObject);
begin
  nMaxClientHeight := clientheight;
  btnRollUp.left   := btnRollDown.left;

  { Set the dimensions of the fake caption bar }
  with rcTitleBar do
  begin
    left   := 3;
    top    := 3;
    right  := clientwidth - 3; 
    bottom := 20;
  end;

  { Move the titlebar buttons into position (no matter what the form width) }
  btnClose.left    := clientwidth - btnClose.width - 4;
  btnRollUp.left   := btnClose.left - btnRollup.width - 4;
  btnRollDown.left := btnClose.left - btnRollup.width - 4;
  btnMenu.left     := btnClose.left;

  btnClose.top    := 4;
  btnRollUp.top   := 4;
  btnRollDown.top := 4;
  btnMenu.top     := 25;

  { Set the fake titlebar caption }
  nCaptionLength := length (self.caption) + 2;  { We'll add 2 blank spaces at
                                                  the beginning of the string }
  GetMem (szCaption, nCaptionLength);
  StrPCopy (szCaption, '  ' + self.caption);

  { OK, a little Windows API magic... }
  DrawTitleBar;
  DrawWindowBorder;

  InitRollupData;
end;


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.FormActivate(Sender: TObject);
begin
  DrawTitleBar;
  DrawWindowBorder;
end;


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.FormPaint(Sender: TObject);
begin
  DrawTitleBar;
  DrawWindowBorder;
end;


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.btnRollDownClick(Sender: TObject);
var
  nCH, i : integer;

begin
  nCH := clientheight;

  { Cover up window border left after after a roll-up }
  self.canvas.pen.color := clBtnFace;
  self.canvas.moveto (2, rcTitleBar.bottom);
  self.canvas.lineto (clientwidth - 2, rcTitleBar.bottom);
  self.canvas.moveto (2, rcTitleBar.bottom + 1);
  self.canvas.lineto (clientwidth - 2, rcTitleBar.bottom + 1);

  self.canvas.moveto (2, rcTitleBar.bottom + 2);
  self.canvas.lineto (clientwidth - 2, rcTitleBar.bottom + 2);

  while (clientheight < nMaxClientHeight) do
    clientheight := clientheight + ROLL_AMOUNT;

  btnRollUp.visible   := true;
  btnRollDown.visible := false;

  clientheight := nMaxClientHeight;
  DrawTitleBar;
  DrawWindowBorder;

end;  { btnRollDownClick }


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.btnRollUpClick(Sender: TObject);
var
  nCH, i : integer;

begin
  nCH := clientheight;

  while (clientheight > btnRollDown.height + 8) do
    clientheight := clientheight - ROLL_AMOUNT;

  btnRollDown.visible := true;
  btnRollUp.visible   := false;

  clientheight := btnRollDown.height + 8;
  DrawWindowBorder;

end;  { btnRollUpClick }


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.btnCloseClick(Sender: TObject);
begin
  close;
end;


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.FormDeactivate(Sender: TObject);
begin
  DrawTitleBar;
end;


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.FormMouseDown (Sender: TObject;
                                           Button: TMouseButton;
                                           Shift : TShiftState;
                                           X, Y  : Integer);
begin
  if (IsPointInFakeCaption (x, y)) then
    bDragging := true;
end;


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.FormMouseMove (Sender: TObject;
                                           Shift : TShiftState;
                                           X, Y  : Integer);
var
  fRet: real;

begin
  if (bDragging) then
  begin
    fRet := Sendmessage (handle, WM_LBUTTONUP, 0, 0);
    fRet := SendMessage (handle, WM_SYSCOMMAND, MOUSE_MOVE, 0);
  end;
end;



{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.FormMouseUp (Sender: TObject;
                                         Button: TMouseButton;
                                         Shift : TShiftState;
                                         X, Y  : Integer);
begin
  bDragging := false;
end;


{-----------------------------------------------------------------------------}
procedure TfrmRollupTest.btnMenuClick(Sender: TObject);
var
  x1, y1: integer;

begin
  btnMenu.refresh;
  x1 := self.left + btnMenu.left + btnMenu.width;
  y1 := self.top + btnMenu.top;
  PopupMenu1.popup (x1, y1);
end;


procedure TfrmRollupTest.mnuChooseColorClick(Sender: TObject);
begin
  If ColorDialog1.Execute then
  edit1.font.Color := ColorDialog1.Color;
end;

procedure TfrmRollupTest.ComboBox1Change(Sender: TObject);
begin
  edit1.text := ComboBox1.text;
end;

procedure TfrmRollupTest.mnuChooseFontClick(Sender: TObject);
begin
  If FontDialog1.Execute then
  edit1.font := FontDialog1.font;
end;

procedure TfrmRollupTest.FormDestroy(Sender: TObject);
begin
  FreeMem (szCaption, nCaptionLength);
end;

end.  { Unit: RU_TEST }
