unit Timeunit;
{----------------------------------------------------------------------
   Written by Dan Statham, July/August 1995.
   Copyright: Dan Statham, July 1995.
   Updated: A. Arnold October/November 1995.
   The program will keep track of the hours/minutes/seconds that
   you are connected to an Internet provider or an online service.
------------------------------------------------------------------------}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Menus,
  About, PreSetUn, FreeHrs, IniFiles, AnaClock, Balloon,
  AWATimes, CommSpy;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileExitItem: TMenuItem;
    SpeedBar: TPanel;
    TimerInterval: TTimer;
    sbtnStartStop: TSpeedButton;
    TimerMenu: TMenuItem;
    StartMenu: TMenuItem;
    StopMenu: TMenuItem;
    ResetMenu: TMenuItem;
    PreSetMenu: TMenuItem;
    PopupMenu1: TPopupMenu;
    StartStopPUMenu: TMenuItem;
    ResetPUmenu: TMenuItem;
    PreSetPuMenu: TMenuItem;
    N1: TMenuItem;
    AboutPuMenu: TMenuItem;
    N2: TMenuItem;
    ExitPUMenu: TMenuItem;
    SetFreeMenu: TMenuItem;
    SetFreeHoursPuMenu: TMenuItem;
    SetupMenu: TMenuItem;
    AnalogClock1: TAnalogClock;
    BalloonHint1: TBalloonHint;
    KeepLog1: TMenuItem;
    KeepLogPUmenu: TMenuItem;
    SpdBtnExit: TSpeedButton;
    SpdBtnAbout: TSpeedButton;
    WatchPortTimer: TTimer;
    AutoSense1: TMenuItem;
    AutomaticallySenseComPort1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FileExit(Sender: TObject);
    procedure StartMenuClick(Sender: TObject);
    procedure TimerIntervalTimer(Sender: TObject);
    procedure StopMenuClick(Sender: TObject);
    procedure ResetMenuClick(Sender: TObject);
    procedure HelpAboutItemClick(Sender: TObject);
    procedure PreSetMenuClick(Sender: TObject);
    procedure StartStop1Click(Sender: TObject);
    procedure SetFreeMenuClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure AnalogClock1Click(Sender: TObject);
    procedure WinMsg(var Msg:TMsg; var Handled:Boolean);
    procedure KeepLog (sbtnStartStop :TSpeedButton; DiffTime, MyTime :String);
    procedure KeepLog1Click(Sender: TObject);
    procedure CheckPort(Sender: TObject);
    procedure AutoSense1Click(Sender: TObject);
  private
   procedure SetMyTime (xTime : TimeRec; var MyTime : String);
   procedure FindRealHours (var RealHours : Real);  {Needed?}
   Procedure TimeToHMSInt(xTime:TDateTime; var xH:Integer; var xM:Integer; var xS:Integer);
   Procedure UpdateIniFile;
  end;

var
  MainForm                                 : TMainForm;
  {Online Session Times}
  SsnStartTime, SsnStopTime, SsnDurTime    : TimeRec;
  {Time Used from Ini File}
  IniTime                                  : TimeRec;
  {UsedTime := IniTime Plus Current Duration}
  UsedTime                                 : TimeRec;
  {Times used since last Reset command}
  Hours ,Minutes, Seconds                  : Integer;
  Hr, Min, Sec, MyTime                     : String;
  WarnMessage, FrHrs                       : String;
  AddCostText                              : String;
  DiffTime                                 : String;
  NewItem                                  : String;
  DiffH, DiffM, DiffS                      : String;
  AdditionalCost                           : Real;
  RealHours                                : Real;
  TotalSeconds                             : Real;
  NotYetShown                              : Boolean;
  MenuFlag                                 : Boolean;
  KeepLogBool                              : Boolean;
  AutoSenseBool                            : Boolean;
  hSysMenu                                 : HMenu;
  Log                                      : Text;
implementation
{$R *.DFM}
const
  ItemID=99;

  {Added by A. Arnold October/November 1995.}
    IniFileName = 'TimeTrac.Ini' {For Testing};
    (*IniFileName = 'OTT.Ini' {For Production};*)

    IniFileSection = 'OnLineTime Tracker'{Becomes a variable in a
                                          future version};

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnMessage:=WinMsg;
  hSysMenu := GetSystemMenu (MainForm.Handle, False);
  AppendMenu (hSysMenu, MF_SEPARATOR, $A9, nil);
  AppendMenu (hSysMenu, MF_STRING, ItemID, 'Start Timer');
  AppendMenu (hSysMenu, MF_STRING, ItemID+1, 'Reset');
  AppendMenu (hSysMenu, MF_STRING, ItemID+2, 'About');
  RealHours := 0.0;
  TotalSeconds := 0;
  with TIniFile.Create (IniFileName) do
    try
      Hours := ReadInteger (IniFileSection, 'Hours', 0);
      Minutes := ReadInteger (IniFileSection, 'Minutes', 0);
      Seconds := ReadInteger (IniFileSection, 'Seconds', 0);
      FreeHours := ReadInteger (IniFileSection, 'FreeHours', 40);
      WarningLevel := ReadInteger (IniFileSection,
        'WarningLevel', 90);
      PerHourCost := ReadInteger (IniFileSection, 'PerHourCost', 195);
      Left  := ReadInteger (IniFileSection, 'Left', 354);
      Top  := ReadInteger (IniFileSection, 'Top', 118);
      Width := ReadInteger (IniFileSection, 'Width', 247);
      Height := ReadInteger (IniFileSection, 'Height', 102);
      KeepLogBool := ReadBool (IniFileSection, 'KeepLog', True);
      AutoSenseBool := ReadBool (IniFileSection, 'AutoSense', True);
    finally
      Free;
  end;
  TimerInterval.Enabled := False;
  sbtnStartStop.Down := False;
  NotYetShown := True;
  {Initialize TimeRec structures}
  IntToHMS(Hours, Minutes, Seconds, IniTime);
  IntToHMS(0,0,0,SsnDurTime);
  UsedTime := IniTime;
  SetMyTime(UsedTime, MyTime);
  FindRealHours (RealHours);
  KeepLog1.Checked := KeepLogBool;
  AutoSense1.Checked := AutoSenseBool;
  {The next line added by A. Arnold October/November 1995.
     Remove comment markers if you want the program to start minimized.}
  {If AutoSense1.Checked then WindowState := wsMinimized;  }
  {Added by A. Arnold October/November 1995.}
  {Initialize CommPort}
  Port := 2;
  CommInit;
end {TMainForm.FormCreate};

procedure TMainForm.WinMsg(var Msg :TMsg; var Handled :Boolean);
{From: JCIRIELL@physiology.uwo.ca
Subject:       Here is a Tip for Delphi
Date:          Tue, 8 Aug 1995 10:45:07 EDT}
begin 
  if Msg.Message=WM_Syscommand then {if the message is a system one...}
      case MSg.WParam of
        ItemID       :  AnalogClock1Click (nil);
        ItemID+1     :  ResetMenuClick (nil);
        ItemID+2     :  HelpAboutItemClick (nil);
      end;{case}
end;

procedure TMainForm.FindRealHours(var RealHours : Real);
begin
  TotalSeconds := (Hours * 36000)/10;
  TotalSeconds := TotalSeconds + (Minutes * 60);
  TotalSeconds := TotalSeconds + Seconds;
  RealHours := TotalSeconds / 3600;
end;

procedure TMainForm.FileExit(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.StartMenuClick(Sender: TObject);
begin
  AnalogClock1.FaceColor := clBtnFace;
  TimerInterval.Enabled := True;
  sbtnStartStop.Down := True;
  TimeToHMSInt(Time, Hours, Minutes, Seconds);
  IntToHMS(Hours, Minutes, Seconds, SsnStartTime);
  IntToHMS(0,0,0,SsnDurTime);
  UsedTime := IniTime;
  MyTime := IniTime.sTime;
  KeepLog (sbtnStartStop,DiffTime, MyTime);
end {StartMenuClick(};

procedure TMainForm.StopMenuClick(Sender: TObject);
begin
  AnalogClock1.FaceColor := clAqua;
  TimerInterval.Enabled := False;
  sbtnStartStop.Down := False;
  NotYetShown := True;
  AddCostText := '';

  TimerIntervalTimer(Sender); {Recompute duration}
  UpdateIniFile;
  KeepLog (sbtnStartStop, DiffTime, MyTime);
end {TMainForm.StopMenuClick};

procedure TMainForm.StartStop1Click(Sender: TObject);
begin
    if not sbtnStartStop.Down then
      begin
        StartMenu.Click;
        sbtnStartStop.Down := True;
      end
  else
    begin
      StopMenu.Click;
      sbtnStartStop.Down := false;
    end;
end {TMainForm.StartStop1Click};

procedure TMainForm.KeepLog (sbtnStartStop :TSpeedButton;
          DiffTime, MyTime :String);
var
  I  : Integer;
begin
  If not KeepLog1.Checked then
    exit;
  AssignFile (Log, 'TimeTrack.Log');
  try
    Append (Log);
  except
    Rewrite (Log);
    WriteLn (Log, 'OnLineTime Tracker Log':46);
    WriteLn (Log, '~~~~~~~~~~~~~~~~~~~~~~':46);
    WriteLn (Log, 'Connected at':12, 'Disconnected at':25,
      'Connected Time':22, 'Total Time':12);
    for I := 1 to 70 do
      Write (Log, '~');
    WriteLn (Log, '~');
  end;
  if sbtnStartStop.Down then
    Write (Log, DateTimeToStr (Now):20)
  else
    WriteLn (Log, DateTimeToStr (Now):22, SsnDurTime.sTime:11,
             UsedTime.sTime:16);
  CloseFile (Log);
end {TMainForm.KeepLog};

procedure TMainForm.TimerIntervalTimer(Sender: TObject);
{Added by A. Arnold October/November 1995.}
{Compute session duration and added cost}
var
  Hour : String;
  RealPerHourCost, TimeOver   : Real;
  Hi, Mi, Si : Integer;
begin
  TimeToHMSInt(Now, Hi, Mi, Si);
  IntToHMS(Hi, Mi, Si, SsnStopTime);
  HMSSubtract(SsnStopTime, SsnStartTime, SsnDurTime);
  HMSAdd(SsnDurTime, IniTime, UsedTime);
  With UsedTime do
    Begin
      Hours := iHrs;
      Minutes := iMins;
      Seconds := iSecs;
      RealHours := rHrs;
    End;
  if FreeHours > 0 then
    begin
      if Hours >= FreeHours then
        begin
          RealPerHourCost := PerHourCost;
          TimeOver :=RealHours - FreeHours;
          AdditionalCost := TimeOver *
            (RealPerHourCost / 100);

          Str (AdditionalCost:6:2, AddCostText);
        end {Hours >= FreeHours};
      if (Hours >= FreeHours) and NotYetShown then
        begin
          NotYetShown := False;
          Hour := IntToStr (Hours);
          FrHrs := IntToStr (FreeHours);
          MessageBeep (48);
          WarnMessage := 'You have used up your '+ FrHrs +
             ' "free" hours!' + Chr(10) + Chr(13) +
             'Watch the title '+
             'bar to see your additional cost add up!';
          MessageDlg (WarnMessage, mtWarning, [mbOK], 0);
        end {Hours >= FreeHours }
      else if (Hours >= (FreeHours * WarningLevel div 100))
        and (NotYetShown) then
        begin
          NotYetShown := False;
          Hour := IntToStr (Hours);
          FrHrs := IntToStr (FreeHours);
          MessageBeep (48);
          WarnMessage := Hour + ' hours of your '+ FrHrs +
             ' "free" hours have already been used up!';
          MessageDlg (WarnMessage, mtWarning, [mbOK], 0);
        end {Hours >= (FreeHours * WarningLevel) and (NotYetShown)};
    end {FreeHours > 0};
  SetMyTime (UsedTime, MyTime);
end {TMainForm.TimerIntervalTimer};

procedure TMainForm.SetMyTime (xTime: TimeRec; var MyTime : String);
begin
  if FreeHours > 0 then
    if Hours >= FreeHours then
      MyTime := xTime.sTime + ' $' + AddCostText
    else
      MyTime := xTime.sTime
  else
      MyTime := xTime.sTime;
  {Application.Title keeps clock showing when minimized}
  {If sbtnStartStop.Down then
    Begin
      MainForm.Caption := 'OnLineTime Tracker ' + MyTime;
      Application.Title := 'OnLineTime ' + Chr(013) + Chr(010) + MyTime;
    end
  else
    Begin
      MainForm.Caption := 'OnLineTime Tracker' ;
      Application.Title := 'OnLineTime' + Chr(013) + Chr(010) + 'Tracker';
    end; }
    MainForm.Caption := MyTime;
    Application.Title := 'OnLineTime ' + Chr(013) + Chr(010) + MyTime;
end {TMainForm.SetMyTime} ;

procedure TMainForm.ResetMenuClick(Sender: TObject);
begin
  MessageBeep (32);
  if MessageDlg ('Reset your time to 00:00:00?',
    mtConfirmation, mbOKCancel, 0) = mrOK then
      begin
        StopMenuClick(Sender) {Stop timer if it is running};
        IntToHMS(0,0,0,IniTime);
        UsedTime := IniTime;
        SsnDurTime := UsedTime;
        SetMyTime (UsedTime, MyTime);
        UpdateIniFile;
  end;
end {TMainForm.ResetMenuClick};

procedure TMainForm.HelpAboutItemClick (Sender: TObject);
begin
  AboutBox := TAboutBox.Create (Self);
  AboutBox.ShowModal;
  AboutBox.Free;
end;

procedure TMainForm.PreSetMenuClick(Sender: TObject);
begin
  BtnRightDlg := TBtnRightDlg.Create (Self);
  BtnRightDlg.ShowModal;
    if BtnRightDlg.ModalResult = mrOK then
      begin
        IntToHMS(Hrs, Mins, Secs, IniTime);
        UsedTime := IniTime;
        SsnDurTime := UsedTime;
        SetMyTime (UsedTime, MyTime);
        UpdateIniFile;
      end;
  BtnRightDlg.Free;
end {TMainForm.PreSetMenuClick};

procedure TMainForm.SetFreeMenuClick(Sender: TObject);
begin
   FreeHoursDlg := TBtnBottomDlg.Create (Self);
   FreeHoursDlg.ShowModal;
   FreeHoursDlg.Free;
end;

procedure  TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  {If clock is running, stop it.}
  If sbtnStartStop.Down then
    StopMenuClick(Sender);
  UpdateIniFile;
end {TMainForm.FormClose};

Procedure TMainForm.UpdateIniFile;
{Added by A. Arnold October/November 1995.}
{Single routine to update all Ini file parameters at the same time}
begin
  with TIniFile.Create (IniFileName) do
    try
      WriteInteger (IniFileSection, 'Hours', UsedTime.iHrs);
      WriteInteger (IniFileSection, 'Minutes', UsedTime.iMins);
      WriteInteger (IniFileSection, 'Seconds', UsedTime.iSecs);
      WriteInteger (IniFileSection, 'FreeHours', FreeHours);
      WriteInteger (IniFileSection,
        'WarningLevel', WarningLevel);
      WriteInteger (IniFileSection, 'PerHourCost', PerHourCost);
      WriteInteger (IniFileSection, 'Left', Left);
      WriteInteger (IniFileSection, 'Top', Top);
      WriteInteger (IniFileSection, 'Width', Width);
      WriteInteger (IniFileSection, 'Height', Height);
      WriteBool (IniFileSection, 'KeepLog', KeepLogBool);
      WriteBool (IniFileSection, 'AutoSense', AutoSensebool);
    finally
      Free;
  end {With};
  IniTime := UsedTime;
end {TMainForm.UpdateIniFile};

procedure TMainForm.AnalogClock1Click;
begin
  MenuFlag := not MenuFlag;
  if not MenuFlag then
    begin
      DeleteMenu (hSysMenu, ItemID, MF_BYCOMMAND);
      InsertMenu (hSysMenu, ItemID + 1, MF_STRING, ItemID, 'Stop Timer');
    end
  else
    begin
      DeleteMenu (hSysMenu, ItemID, MF_BYCOMMAND);
      InsertMenu (hSysMenu, ItemID + 1, MF_STRING, ItemID, 'Start Timer');
    end;

  if not sbtnStartStop.Down then
    begin
      sbtnStartStop.Down := True;
      StartMenu.Click;
   end
  else
    begin
      sbtnStartStop.Down := False;
      StopMenu.Click;
    end;
end {TMainForm.AnalogClock1Click};

procedure TMainForm.KeepLog1Click(Sender: TObject);
begin
  if not sbtnStartStop.Down then
    begin
      KeepLog1.Checked := not KeepLog1.Checked;
      KeepLogPUMenu.Checked := KeepLog1.Checked;
      KeepLogBool := KeepLog1.Checked;
    end;
end;

Procedure TMainForm.TimeToHMSInt(xTime:TDateTime;
          var xH:Integer; var xM:Integer; var xS:Integer);
{Added by A. Arnold October/November 1995.}
{Convert passed DateTime to Hour Minutes Seconds Integers}
var
  sTime, x : String;
Begin;
  sTime := TimeToStr(xTime);
  if Length (sTime) = 11 then
    begin
      xH := StrToInt(Copy (sTime , 1, 2));
      xM := StrToInt(Copy (sTime, 4, 2));
      xS := StrToInt(Copy (sTime, 7, 2));
    end
  else
    if Length (sTime) = 10 then
      begin
        xH := StrToInt(Copy (sTime , 1, 1));
        xM := StrToInt(Copy (sTime, 3, 2));
        xS := StrToInt(Copy (sTime, 6, 2));
      end;
End {TimeStrToHMSInt};

procedure TMainForm.CheckPort(Sender: TObject);
{Added by A. Arnold October/November 1995.}
begin
  If not AutoSense1.Checked then begin
   WatchPortTimer.Enabled := False;
   exit;
   end;
  {If clock stopped and DCD then start clock}
  If sbtnStartStop.Down = False then begin
      CheckCommPortStatusRegister;
      if psDCD then
        StartMenu.Click;
      end {If};

  {If clock Running and not DCD then stop clock}
  If sbtnStartStop.Down = True then begin
      CheckCommPortStatusRegister;
      if Not psDCD then
          StopMenu.Click;
      end {If};
end {Check Port};

procedure TMainForm.AutoSense1Click(Sender: TObject);
{Added by A. Arnold October/November 1995.}
begin
  AutoSense1.Checked := not AutoSense1.Checked;
  AutomaticallySenseComPort1.Checked := AutoSense1.Checked;
  AutoSenseBool := AutoSense1.Checked;
  WatchPortTimer.Enabled := AutoSenseBool;
end;

initialization
  MenuFlag := True;

end.
