{ ========================================================================
  Unit:    UStrGrid
  VCL:     TXStrGrid
  Version: 1.1
  Copyright (C) 1996, Immo Wache
  ========================================================================}

unit Ustrgrid;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Grids;

type
  TGridColorEvent = procedure (Sender: TObject; ARow, ACol: Longint;
                               AState: TGridDrawState;
                               ABrush: TBrush; AFont: TFont ) of object;
  TGridAlignEvent = procedure (Sender: TObject; ARow, ACol: Longint;
                               var AAlignment: TAlignment) of object;

  TXStrGrid = class(TStringGrid)
  private
    { Private declarations }
    FAlign: TAlignment;
    FGrid3D : boolean;
    FHCol: TStrings;
    FHRow: TStrings;
    FOnGetCellColor: TGridColorEvent;
    FOnGetAlignment: TGridAlignEvent;
    function GetAlign:TAlignment;
    procedure SetAlign(const Value: TAlignment);
    procedure SetGrid3D(Value : boolean);
    procedure InitHCol;
    procedure InitHRow;
    procedure SetHCol(Value: TStrings);
    procedure SetHRow(Value: TStrings);
    procedure DrawCell(ACol, ARow : Longint; ARect : TRect;
      AState : TGridDrawState); override;
  protected
    { Protected declarations }
    procedure Loaded; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure MoveTo(ACol, ARow: longint);
  published
    { Published declarations }
    property HCol: TStrings read FHCol write SetHCol;
    property HRow: TStrings read FHRow write SetHRow;
    property OnGetCellColor: TGridColorEvent read FOnGetCellColor
      write FOnGetCellColor;
    property OnGetAlignment: TGridAlignEvent read FOnGetAlignment
      write FOnGetAlignment;
    property Grid3D : boolean read FGrid3D write SetGrid3D;
    property Alignment: TAlignment read GetAlign
      write SetAlign default taLeftJustify;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MyStuff', [TXStrGrid]);
end;

constructor TXStrGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FHCol:=TStringList.Create;
  FHRow:=TStringList.Create;
  FAlign :=taLeftJustify;
end;

procedure TXStrGrid.MoveTo(ACol, ARow: longint);
begin
  MoveColRow(ACol, ARow, True, True);
end;

procedure TXStrGrid.InitHCol;
var
  I: Integer;
begin
  if (FHCol <> nil) then
    for I :=0 to pred( ColCount) do
      if I <FHCol.Count then Cells[I, 0] :=FHCol[I]
                        else Cells[I, 0] :='';
end;

procedure TXStrGrid.InitHRow;
var
  I: Integer;
begin
  if (FHRow <> nil) then
    for I :=0 to RowCount -2 do
      if I <FHRow.Count then Cells[0, I + 1]:=FHRow[I]
                        else Cells[0, I + 1]:='';
end;

procedure TXStrGrid.Loaded;
begin
  inherited Loaded;
  initHCol;
  initHRow;
end;

procedure TXStrGrid.SetHCol(Value: TStrings);
begin
  FHCol.Assign(Value);
  InitHCol;
  Refresh;
end;

procedure TXStrGrid.SetHRow(Value: TStrings);
begin
  FHRow.Assign(Value);
  InitHRow;
  Refresh;
end;

function TXStrGrid.GetAlign: TAlignment;
begin
   Result := FAlign;
end;

procedure TXStrGrid.SetAlign(const Value: TAlignment);
begin
  if FAlign<>Value then
  begin
    FAlign:=Value;
    Invalidate;
  end;
end;

procedure TXStrGrid.DrawCell(ACol, ARow : Longint; ARect : TRect;
  AState : TGridDrawState);

  procedure DrawCellText;
  var
    Text: array[0..255] of Char;
    l : Integer;
    Left : Integer;
    AlignValue : TAlignment;
    FontHeight: Integer;
    Rect : TRect;
  const
    Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  begin
    {determine text alignment}
    AlignValue :=Alignment;
    if Assigned(FOnGetAlignment) then
      FOnGetAlignment(Self, ARow, ACol, AlignValue);
    {using of Brush for background color}
    Rect :=ARect;
    Canvas.FillRect(Rect);
    {centering text in cell}
    FontHeight := Canvas.TextHeight('W');
    with Rect do
    begin
      Top := ((Bottom + Top) - FontHeight) shr 1;
      Bottom := Top + FontHeight;
      Dec(Right, 2);
      Inc(Left, 2);
    end;
    {drawing of text}
    StrPCopy(Text, Cells[ACol, ARow]);
    DrawText(Canvas.Handle, Text, StrLen(Text), Rect, (DT_EXPANDTABS or
      DT_VCENTER) or Alignments[AlignValue]);
  end;

begin
  {prepare color and font selection}
  if Assigned(FOnGetCellColor) then
    FOnGetCellColor(Self, ARow, ACol, AState, Canvas.Brush, Canvas.Font);
  {text draw with alignment}
  if DefaultDrawing then DrawCellText
                    else inherited DrawCell(ACol, ARow, ARect, AState);
  {3D look cells}
  if FGrid3D and
    ([goHorzLine, goVertLine] *Options =[goHorzLine, goVertLine]) then
    with ARect do
    begin
      Canvas.Pen.Color := clHighLightText;
      Canvas.PolyLine([Point(Left, Bottom - 1), Point(Left, Top),
        Point(Right, Top)]);
    end;
end;

procedure TXStrGrid.SetGrid3D(Value : Boolean);
begin
  if FGrid3D<>Value then
  begin
    FGrid3D:=Value;
    Invalidate;
  end;
end;

end.
