Unit UTqbe; { TQBE and TRemoteQuery components 4.0 Jul/95 }

{ David Berneda 100115,1155@compuserve.com }

{ Version 4.0
  NEW !!! TRemoteQuery to execute QBE & SQL queries remotely
 (needs REMOQUE.EXE on the remote machine)

  NEW !!! Function GetUserName:String (returns IDAPI or Paradox user name)

  NEW !!! TQBE.ExecQBE method to execute INSERT, UPDATE & DELETE queries
      without getting the "Cannot create cursor handle" exception.
}

{ Version 3.0
  NEW !!! AnswerType property allows to PARADOX,DBASE and ASCII answer table types
  NEW !!! Query params can be defined (see demo.pas)
}

interface

Uses Classes,DBTables,DB,SysUtils,DBConsts,LibConst,dbiProcs,dbiTypes,DsgnIntf;

{ WARNING: READ CAREFULLY AND GOOD LUCK USING TQBE and TRemoteQuery }
(*
  Answer table name can be specified with or without alias.
  Eg: :dbdemos:void.db

  Assumed answer table Driver: PARADOX
  The new property: AnswerType allows to PARADOX,DBASE and ASCII answer table types

  If the answer table exists, an atempt to delete it is made before copying
  the result cursor.
  The phisical answer table must be opened in exclusive mode and all related
  family files are erased together with the table.

  Other functions in this unit:

**  Function GetAliasPath(Const Alias:String):String;
      Returns the path for the "alias" or a empty string if not found.
      Eg: ('dbdemos') returns 'c:\delphi\demos\data'

**  Function GetDBTablePath(Const TableName:String):String;
      Returns the TableName with path instead of alias (if it has an alias).
      Eg: (':dbdemos:customer.db') returns 'c:\delphi\demos\data\customer.db'

*)

Const MaxParam   =15; { max number of query parameters (freely modify) }
      MaxParamLen=30; { max length of a substituted param (freely modify) }

Type TCustomQBE=Class(TDBDataSet)
     private
       FAnswerTable:String;       { the answer table name }
       FAnswerType:TTableType;    { the answer table type (Paradox,DBase,ASCII)}
       FBlankasZero,              { Treat blank fields as zero ( 0 ) }
       FAuxTables:Boolean;        { Tell BDE to generate auxiliary tables }
       FRequestLive:Boolean;      { We want the query to be "live" }
       FQueryLang:DBIQryLang;     { QBE or SQL language }
     protected
       function CreateHandle: HDBICur; override; { the inherited... }
       procedure GenerateAnswer(Var p:HDbiCur);  { creates the answer table }
       Function CreateSubstituted:TStrings;      { does param stuff }
       property RequestLive: Boolean read FRequestLive write FRequestLive;
     public
       FQBE: TStrings;   { here goes the query statement (QBE or SQL) }
       NumParam:Integer;  { number of parameter substitutions }
       Param,
       Subst:Array[0..MaxParam] of String[MaxParamLen];  { the params... }
       procedure SetQBE(QBE: TStrings);
       Constructor Create(AOwner:TComponent); override;
       destructor Destroy; override;
       Procedure AddParam(Const tmpParam,tmpSubst:String); { add a param  }
       Function ReplaceString(s:String):String;
       Procedure ClearParams;                             { clear all params }
       Function GetAnswerName:String; { valid only in QBE language }
       Procedure ExecQBE;
     published
       property QBE: TStrings read FQBE write SetQBE;
       property AnswerTable: String read FAnswerTable write FAnswerTable;
       property BlankasZero: Boolean read FBlankasZero write FBlankAsZero;
       property AuxTables: Boolean read FAuxTables write FAuxTables;
       property AnswerType:TTableType read FAnswerType write FAnswerType;
       property QueryLang:DBIQryLang read FQueryLang write FQueryLang;
     end;

     TQBE=Class(TCustomQBE) { local QBE query component }
     published
       property RequestLive;
     End;

     { These are the posible states while we are waiting the server to
       execute the query remotely }

     TRemoteStatus=( rsStart,
                     rsWaitingBegin,
                     rsWaitingExecution,
                     rsOK,
                     rsError,
                     rsTimeout,
                     rsIdle,
                     rsCancel,
                     rsRefreshing
                     );

     TNotifyRemote=procedure(Sender:TObject; what:TRemoteStatus) of object;

     TRemoteQuery=Class(TCustomQBE) { The remote QBE or SQL query component }
     Private
       FAskMoreWait,
       FCancel:Boolean;
       FOnWaiting:TNotifyRemote;
       FSecondsTimeout:Longint;
       FDoRemote:Boolean;
       FServerAlias:String;
       Procedure Notify(what:TRemoteStatus);
     protected
       function CreateHandle: HDBICur; override;
     public
       Constructor Create(AOwner:TComponent); override;
       Procedure OpenRemoteOrLocal; { try to execute remote and local if fails }
     published
       property OnWaiting:TNotifyRemote read FOnWaiting write FOnWaiting;
       property SecondsTimeout:Longint read FSecondsTimeout write FSecondsTimeout default 60;
       property DoRemote:Boolean read FDoRemote write FDoRemote default TRUE;
       property Cancel:Boolean read FCancel write FCancel;
       property AskMoreWait:Boolean read FAskMoreWait write FAskMoreWait;
       property ServerAlias:String read FServerAlias write FServerAlias;
     End;

{ given an alias (ie: DBDEMOS), return the path: (ie: c:\delphi\demos\data) }
Function GetAliasPath(Const Alias:String):String;

{ given a table name (ie: :DBDEMOS:CUSTOMER.DB), return the complete path:
  (ie: c:\delphi\demos\data\customer.db) }
Function GetDBTablePath(Const TableName:String):String;


Procedure Register; { registers the TQBE and TRemoteQuery components }

implementation

Uses WinProcs,IniFiles,Dialogs,dbiErrs,Forms,Controls;

{ TCUSTOMQBE base class }
Constructor TCustomQBE.Create(AOwner:TComponent);
Begin
  inherited Create(AOwner);
  FQBE := TStringList.Create;
  NumParam:=0;
  FQueryLang:=qryLangQBE;
  FAnswerType:=ttParadox; { by default, Paradox answer tables }
end;

Destructor TCustomQBE.Destroy;
Begin
  FQBE.Free;
  inherited Destroy;
End;

Function TCustomQBE.GetAnswerName:String; { valid only in QBE language }
var i,t:Integer;
Begin
  result:='';
  for t:=0 to QBE.Count-1 do
  Begin
    i:=Pos('ANSWER:',Uppercase(QBE.Strings[t]));
    if i>0 then
    Begin
      result:=Copy(QBE.Strings[t],i+7,255);
      break;
    end;
  end;
End;

Procedure TCustomQBE.ClearParams;
Begin
  NumParam:=0; { reset params to zero (no params) }
End;

Procedure TCustomQBE.AddParam(Const tmpParam,tmpSubst:String);
Begin
  if (tmpParam<>'') and (tmpParam<>tmpSubst) then
  Begin
    if NumParam<MaxParam then
    Begin
      Inc(NumParam);
      Param[NumParam]:=tmpParam;
      Subst[NumParam]:=tmpSubst;
    End
    Else Raise Exception.Create('Max number of query parameters achieved');
  End;
End;

Function TCustomQBE.ReplaceString(s:String):String;
Var t,i:Integer;
Begin
  for t:=1 to NumParam do
  Repeat
    i:=Pos(Param[t],s);
    if i>0 then s:=Copy(s,1,i-1)+Subst[t]+Copy(s,i+Length(Param[t]),255);
  Until i=0;
  result:=s;
End;

Function TCustomQBE.CreateSubstituted:TStrings;
Var NewQBE:TStrings;
    t:Integer;
Begin
  NewQBE:=TStringList.Create;
  if Assigned(NewQBE) then
  Begin
    With FQBE do
    for t:=0 to Count-1 do NewQBE.Add(ReplaceString(Strings[t]));
  End;
  result:=NewQBE;
End;

function TCustomQBE.CreateHandle:HDBICur;
Var p:HDbiCur;
    Stmt:hDBIStmt;
    NewQBE:TStrings;
Begin
  p:=nil;
  NewQBE:=CreateSubstituted;
  try
    Check(dbiQPrepare(DBHandle,FQueryLang,NewQBE.GetText,Stmt));
    if FRequestLive then Check(dbiSetProp(hDBIObj(Stmt),stmtLIVENESS,Longint(wantLive)))
                    Else Check(dbiSetProp(hDBIObj(Stmt),stmtLIVENESS,Longint(wantDefault)));
    if FBlankAsZero then Check(dbiSetProp(hDBIObj(Stmt),stmtBLANKS,1));
    if FAuxTables then Check(dbiSetProp(hDBIObj(Stmt),stmtAUXTBLS,1));
    Check(dbiQExec(Stmt,@p));
    Check(dbiQFree(Stmt));
    GenerateAnswer(p);
  finally
    NewQBE.Free;
  end;
  Result:=p;
End;

{ Dont return cursor, just do the query (use this on INSERT,DELETE,UPDATE) }
Procedure TCustomQBE.ExecQBE;
Begin
  CheckInActive;
  SetDBFlag(dbfExecSQL, True);
  try
    CreateHandle;
  finally
    SetDBFlag(dbfExecSQL, False);
  end;
End;

procedure TCustomQBE.GenerateAnswer(Var p:HDbiCur);
Var aBatTblDesc:BATTblDesc;
    tmpType:String;
    r:Longint;
    dbiErr:DBIRESULT;
Begin
  if (FAnswerTable<>'') And Assigned(p) then
  try
    Check(DbiSetToBegin(p));
    With aBatTblDesc do
    Begin
      hDB:=DBHandle;
      StrPCopy(szTblName,GetDBTablePath(FAnswerTable));
      Case FAnswerType of
        ttParadox: tmpType:=szParadox;
        ttDbase  : tmpType:=szDbase;
        ttAscii  : tmpType:=szAscii;
      else
        tmpType:=szParadox;
      end;
      StrPCopy(szTblType,tmpType);
      szUsername[0]:=#0;
      szPassword[0]:=#0;
    End;
    r:=0;
    dbiErr:=dbiDeleteTable(DBHandle,aBatTblDesc.szTblName,aBatTblDesc.szTblType);
    if dbiErr<>DBIERR_NOSUCHTABLE then Check(dbiErr);
    Check(DbiBatchMove(nil,p,@aBatTblDesc,nil,batchCOPY,0,
                            nil, nil, nil, 0, nil, nil,
                            nil, nil, nil, nil, TRUE, TRUE,
                            r, TRUE));
    Check(dbiCloseCursor(p));
    Check(DbiOpenTable(DBHandle, aBatTblDesc.szTblName, aBatTblDesc.szTblType,
                       nil, nil, 0,
                       dbiReadWrite,
                       dbiOpenShared,
                       xltField,
                       False,
                       nil,
                       p));
  finally
  End;
end;

procedure TCustomQBE.SetQBE(QBE: TStrings);
begin
  FQBE.Assign(QBE);
end;

Function HasAlias(Const TableName:String):Boolean;
Begin
  Result:=Pos(':',TableName)>0;
End;

Function GetAliasPath(Const Alias:String):String;
Var AliasList:TStringList;
    i:Longint;
    DBPath:String;
Begin
  Result:='';
  AliasList:=TStringList.Create;
  try
    Session.GetAliasNames(AliasList);
    i:=AliasList.IndexOf(Alias);
    if i<0 then raise EDatabaseError.Create('Alias '+Alias+' doesnt exist')
    else
    Begin
      Session.GetAliasParams(Alias,AliasList);
      DBPath := AliasList.Values['PATH'];
      if DBPath='' then raise EDatabaseError.Create('Alias path from '+Alias+' invalid')
                   else Result:=DBPath;
    end;
  finally
    AliasList.Free;
  end;
End;

Procedure SplitTableName(Const TableName:String; Var Alias,Name:String);
Var p1,p2:Integer;
Begin
  Name:=TableName;
  Alias:='';
  p1:=Pos(':',TableName);
  if p1>0 then
  Begin
    p2:=Pos(':',Copy(TableName,p1+1,255));
    if p2>0 then
    Begin
      Alias:=Copy(TableName,p1+1,p2-1);
      Name:=Copy(TableName,p1+p2+1,255);
    End;
  End;
End;

Function GetDBTablePath(Const TableName:String):String;
Var Alias,Name:String;
Begin
  if not HasAlias(TableName) then Result:=TableName
  else
  Begin
    SplitTableName(TableName,Alias,Name);
    if Alias<>'' then Result:=GetAliasPath(Alias)+'\'+Name
                 else Result:=TableName;
  End;
End;

Type TString=Array[0..255] of char;

Function GetUserName:String;
Var St:TString;
Begin
  result:='Unknown';
  if DbiGetNetUserName(St)=DBIERR_NONE then
     result:=StrPas(St)
  else
  With TIniFile.Create('win.ini') do
  try
    result:=ReadString('PDOXWIN','USERNAME','UNKNOWN');
  finally
    Free;
  End;
end;

{ TREMOTE QBE }
Constructor TRemoteQuery.Create(AOwner:TComponent);
Begin
  inherited Create(AOwner);
  FSecondsTimeout:=60;
  FDoRemote:=TRUE;
  FCancel:=FALSE;
  FAskMoreWait:=FALSE;
  FServerAlias:='REMOQUE'; { default Alias }
End;

{ notify the user the remote query execution status }
Procedure TRemoteQuery.Notify(what:TRemoteStatus);
Begin
  if Assigned(FOnWaiting) then
  try
    FOnWaiting(Self,what);
  except
   on Exception do ;
  end;
End;

function TRemoteQuery.CreateHandle: HDBICur;

  Function WaitServer(Table:TTable; NumQuery:Longint; Const WaitFor:String):String;

    Function SetTimeout:Longint;
    Begin
      if FSecondsTimeout=0 then result:=GetTickCount+86400000
                           else result:=GetTickCount+FSecondsTimeout*1000;
    end;

  Var NumTries,
      Timeout,
      TimeToRefresh,
      TickCount:Longint;
  Begin
    result:='';
    { Timeout=0 means wait a whole day !!! }
    Timeout:=SetTimeout;
    NumTries:=1;
    TimeToRefresh:=GetTickCount+NumTries*100;
    Repeat
      TickCount:=GetTickCount;
      Notify(rsIdle);
      Application.ProcessMessages;
      if TickCount>TimeToRefresh then
      try
        Notify(rsRefreshing);
        Table.Refresh;
        Application.ProcessMessages;
        if Table.FindKey([NumQuery]) then
           result:=Table.FieldByName('Estado').AsString
        else
           result:='';
      finally
        if NumTries<15 then Inc(NumTries); { up to 15*100 millisecs }
        TimeToRefresh:=GetTickCount+NumTries*100;
      End
      Else
      if TickCount>Timeout then
      Begin
        if FAskMoreWait And
           (MessageDlg('Timeout while waiting Query. Do you want to wait more?',
            mtInformation, [mbYes, mbNo], 0) = mrYes) then
              Timeout:=SetTimeout;
      End;
    Until (Pos(result,WaitFor)>0) or
          (TickCount>Timeout) or
          FCancel or
          Application.Terminated;
    if FCancel or Application.Terminated then result:='C'; { canceled }
  End;

Var Status:String;
    SAnswer:TString;
    t:TTable;
    NewQBE:TStrings;
    NumQuery:Longint;
Begin
  FCancel:=False;
  if not FDoRemote then
  Begin
    result:=Inherited CreateHandle;
    exit;
  end;
  result:=nil;
  Notify(rsStart);
  t:=TTable.Create(Self);
  with t do
  try
    DatabaseName:=FServerAlias;
    TableName:='remoque.db';
    IndexFieldNames:='NumeroConsulta';
    Open;
    Append;
    FieldByName('Fecha').AsDateTime:=SysUtils.Date;
    FieldByName('Hora').AsDateTime:=Now;
    FieldByName('Cliente').AsString:='DELPHI';
    FieldByName('Usuario').AsString:=GetUserName;
    Case FQueryLang of
      qryLangQBE: FieldByName('TipoConsulta').AsString:='Q';
      qryLangSQL: FieldByName('TipoConsulta').AsString:='S';
    end;
    NewQBE:=CreateSubstituted;
    try
      FieldByName('Consulta').Assign(NewQBE);
    finally
      NewQBE.Free;
    end;
    FieldByName('Estado').AsString:='P';
    FieldByName('Error').AsString:='';
    Post;
    NumQuery:=FieldByName('NumeroConsulta').AsInteger;
    Notify(rsWaitingBegin);
    Status:=WaitServer(t,NumQuery,'EOM');
    if Status='E' then
    Begin
      Notify(rsWaitingExecution);
      Status:=WaitServer(t,NumQuery,'OM');
    end;
    if Status='O' then
    Begin
      Notify(rsOK);
      StrPCopy(SAnswer,GetAliasPath(FServerAlias)+'\PERSONAL\'+FieldByName('NumeroConsulta').AsString);
      Check(DbiOpenTable(DBHandle, SAnswer, szParadox,
                                     nil, nil, 0,
                                     dbiReadWrite,
                                     dbiOpenShared,
                                     xltField,
                                     False,
                                     nil,
                                     Result));
      GenerateAnswer(result);
    end
    else
    if Status='M' then
    Begin
      Notify(rsError);
      raise Exception.Create(FieldByName('Error').AsString);
    end
    else
    if (Status='P') Or (Status='E') then
    Begin
      Notify(rsTimeout);
      try
        if FindKey([NumQuery]) then
        Begin
          Edit;
          FieldByName('Estado').AsString:='T';
          FieldByName('Error').AsString:='Timeout';
          Post;
        end;
      finally
        raise Exception.Create('Timeout while waiting RemoQUE Server');
      end;
    end
    else
    if Status='C' then { cancelled query }
    try
      Notify(rsCancel);
      if FindKey([NumQuery]) then
      try
        Edit;
        FieldByName('Estado').AsString:='C';
        FieldByName('Error').AsString:='Canceled';
        Post;
      finally
      end;
    finally
      SysUtils.Abort;
    End;
  finally
    Free;
  end;
End;

{ try to execute the query in the user-defined mode (local or remote).
  If fails, try to execute in the other mode (remote if local, local if remote
}
Procedure TRemoteQuery.OpenRemoteOrLocal;
Begin
  try
    Open;
  except
    on Exception do
    Begin
      try
        DoRemote:=not DoRemote;
        Open;
      finally
        DoRemote:=not DoRemote;
      end;
    end;
  end;
end;

{ TQBE and TRemoteQuery components }
Procedure Register;
Begin
  RegisterComponents(LoadStr(srDAccess),[TQBE,TRemoteQuery]);
End;

end.
