{$DEFINE DEBUG}
{
             Copyright (C) 1996 Word in Action.  All Rights Reserved.

    Project : Tile Component
    Company : Word in Action
    File    : wiaTile.pas
    Overview: This file is a control that allows an image to be tiled and
              is released into the public domain for your own useage although
              Word in Action retains the Copyright

    Author  : Jay Giganti
    EMail   : Jay Giganti -- CompuServe ID: [72037,2225]

    72037.2225@compuserve.com -- http://ourworld.compuserve.com/homepages/jrg
    wordinaction@hypercon.com -- http://www.hypercon.com/wordinaction/

          Please inform me of any comments, suggestions or improvements:
       *** Now for the stuff that we have unfortunately been driven to ***

    RIGHTS:     Title, ownership rights, and intellectual property rights in
                and to the Software shall remain in Word in Action and/or
                its suppliers. The Software is protected by the copyright
                laws of the United States and international copyright
                treaties. This License gives you no rights to such content.

    DISCLAIMER: This code is for demonstration purposes only and should
                be used at your own risk. Since the Software is provided free
                of charge, the Software is provided on an "AS IS" basis,
                without warranty of any kind, including without limitation
                the warranties of merchantability, fitness for a particular
                purpose and non-infringement. The entire risk as to the
                quality and performance of the Software is borne by you.
                Should the Software prove defective, you and not Word in Action
                assume the entire cost of any service and repair. In
                addition, you must determine that the Software sufficiently
                meets your requirements.  This disclaimer of warranty
                constitutes an essential part of the agreement. SOME STATES
                DO NOT ALLOW EXCLUSIONS OF AN IMPLIED WARRANTY, SO THIS
                DISCLAIMER MAY NOT APPLY TO YOU AND YOU MAY HAVE OTHER LEGAL
                RIGHTS THAT VARY FROM STATE TO STATE OR BY JURISDICTION.

    LIMITATIONS: LIABILITY -- UNDER NO CIRCUMSTANCES AND UNDER NO
                 LEGAL THEORY, TORT, CONTRACT, OR OTHERWISE, SHALL
                 Word in Action OR ITS SUPPLIERS OR RESELLERS BE LIABLE TO
                 YOU OR ANY OTHER PERSON FOR ANY INDIRECT, SPECIAL,
                 INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER
                 INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL,
                 WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND
                 ALL OTHER COMMERCIAL DAMAGES OR LOSSES. IN NO EVENT WILL Word
                 in Action BE LIABLE FOR ANY DAMAGES IN EXCESS OF Word in
                 Action's LIST PRICE FOR A LICENSE TO THE SOFTWARE, EVEN IF
                 Word in Action SHALL HAVE BEEN INFORMED OF THE POSSIBILITY
                 OF SUCH DAMAGES, OR FOR ANY CLAIM BY ANY OTHER PARTY. THIS
                 LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR
                 DEATH OR PERSONAL INJURY TO THE EXTENT APPLICABLE LAW
                 PROHIBITS SUCH LIMITATION. FURTHERMORE, SOME STATES DO NOT
                 ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR
                 CONSEQUENTIAL DAMAGES, SO THIS LIMITATION AND EXCLUSION MAY
                 NOT APPLY TO YOU.

                              Version Information
      Date    Ver  Comments
    ======== ===== ===========================================================
    12/26/96  1.00 Created this component
}
{$A+,B-,F-,G+,I+,K+,N+,P+,T+,V+,W-,X+}
{$IFNDEF DEBUG}
{$D-,L-,Q-,R-,S-,Y-}
{$ELSE}
{$D+,L+,Q+,R+,S+,Y+}
{$ENDIF}

unit wiaTile;

interface
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls;

type
  TwiaTile = class(TGraphicControl)
  protected
    FImage      : TBitmap;
    FTransparent: Boolean;
    FTranColor  : TColor;

    TrnBmp      : TBitmap;

    procedure ChangedImage(Sender: TObject);
    function  GetPalette: HPALETTE; override;
    procedure Paint; override;
    procedure SetImage(Value: TBitMap);
    procedure SetTran(Value: Boolean);
    
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

  published
    property Image    :TBitMap read FImage write SetImage;
    property TranColor:TColor read FTranColor write FTranColor default clBlack;
    property Transparent:Boolean read FTransparent write SetTran default FALSE;

    property Height default 64;
    property Width  default 64;
    property Visible;
    property Align;
    property Enabled;
    property ParentShowHint;
    property ShowHint;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;
  
  procedure Register;
  
implementation
procedure Register;
begin
  RegisterComponents('Graphics', [TwiaTile]);
end;

{==============================================================================
                                 Create
Purpose: To Initialize the Object
Details: Let the base class get a creack at initialization and then set up the
         default values and create the bitmap place holders.
==============================================================================}
constructor TwiaTile.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FImage          := TBitMap.Create;
  TrnBmp          := TBitMap.Create;
  FImage.OnChange := ChangedImage;
  FTranColor      := clBlack;
  FTransparent    := False;
  SetBounds(0, 0, 64, 64);
end;
{==============================================================================
                                 Destroy
Purpose: To Free any memory associated with this object
==============================================================================}
destructor TwiaTile.Destroy;
begin
  FImage.Free;
  TrnBmp.Free;
  inherited Destroy;
end;
{==============================================================================
                                 ChangedImage
==============================================================================}
procedure TwiaTile.ChangedImage(Sender: TObject);
var
 clrOld: TColor;
begin
  FTranColor := FImage.TransparentColor;
  clrOld     := GetBkColor(FImage.Canvas.Handle);
  if FTransparent then
  begin
    TrnBmp.Width      := FImage.Width;
    TrnBmp.Height     := FImage.Height;
    TrnBmp.MonoChrome := True;

    SetBkColor(FImage.Canvas.Handle, FTranColor);
    BitBlt(TrnBmp.Canvas.Handle, 0, 0, FImage.Width, FImage.Height,
           FImage.Canvas.Handle, 0, 0, SRCCOPY);

    SetBkColor(FImage.Canvas.Handle, clBlack);
    SetTextColor(FImage.Canvas.Handle, clWhite);
    BitBlt(FImage.Canvas.Handle, 0, 0, FImage.Width, FImage.Height,
           TrnBmp.Canvas.Handle, 0, 0, SRCAND);
    SetBkColor(FImage.Canvas.Handle, clrOld);
  end;
  Invalidate;
end;
{==============================================================================
                                 GetPalette
==============================================================================}
function TwiaTile.GetPalette: HPALETTE;
begin
  if (FImage.Height > 0)  AND (FImage.Width > 0) then
    Result := FImage.Palette;
end;
{==============================================================================
                                 Paint
==============================================================================}
procedure TwiaTile.Paint;
var
  X, Y, W, H: LongInt;
begin
  if (FImage.Height > 0) AND (FImage.Width > 0) then
  begin
    W     := FImage.Width;
    H     := FImage.Height;
    Y     := 0;
    while Y < Height do
    begin
      X := 0;
      while X < Width do
      begin
        if not FTransparent then
          Canvas.Draw(X, Y, FImage)
        else
        begin
          BitBlt(Canvas.Handle, X, Y, W, H,
                 TrnBmp.Canvas.Handle, 0, 0, SRCAND);
          BitBlt(Canvas.Handle, X, Y, W, H,
                 FImage.Canvas.Handle, 0, 0, SRCPAINT);
        end;
        Inc(X, W);
      end;
      Inc(Y, H);
    end;
  end
  else if (csDesigning in ComponentState) then
  begin
    Canvas.Brush.Color := clGray;
    Canvas.FillRect(Rect(0, 0, Width, Height));
  end;
end;
{==============================================================================
                                 SetImage
==============================================================================}
procedure TwiaTile.SetImage(Value: TBitMap);
begin
  if Assigned(Value) then
    FImage.Assign(Value)
  else
  begin
    FImage.Height := 0;
    FImage.Width  := 0;
  end;
end;
{==============================================================================
                                 SetTran
==============================================================================}
procedure TwiaTile.SetTran(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent := Value;
    ChangedImage(Self);
  end;
end;
end.
