unit Win95gau;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

type
  TWin95Gauge = class(TGraphicControl)
  private
    FMinValue: Longint;
    FMaxValue: Longint;
    FCurValue: Longint;
    FBoxes	 : Byte;
    procedure SetMinValue(Value: Longint);
    procedure SetMaxValue(Value: Longint);
    procedure SetProgress(Value: Longint);
    function GetPercentDone: Longint;
    procedure SetBoxes(Value:byte);
    procedure CheckBounds;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure AddProgress(Value: Longint);
    property PercentDone: Longint read GetPercentDone;
  published
    property Align;
    property Boxes : byte read FBoxes write SetBoxes;
    property Enabled;
    property MinValue: Longint read FMinValue write SetMinValue default 0;
    property MaxValue: Longint read FMaxValue write SetMaxValue default 100;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property Progress: Longint read FCurValue write SetProgress;
    property ShowHint;
    property Visible;
  end;

procedure Register;

implementation
const FboxW=12; FBoxH=18;

type
  TBltBitmap = class(TBitmap)
    procedure MakeLike(ATemplate: TBitmap);
  end;

	procedure TBltBitmap.MakeLike(ATemplate: TBitmap);
	begin
	  Width := ATemplate.Width;
	  Height := ATemplate.Height;
	  Canvas.Brush.Color := clWindowFrame;
	  Canvas.Brush.Style := bsSolid;
	  Canvas.FillRect(Rect(0, 0, Width, Height));
	end;


{ This function solves for x in the equation "x is y% of z". }
function SolveForX(Y, Z: Longint): Integer;
begin
  SolveForX := Trunc( Z * (Y * 0.01) );
end;

{ This function solves for y in the equation "x is y% of z". }
function SolveForY(X, Z: Longint): Integer;
begin
  if Z = 0 then SolveForY := 0
  else SolveForY := Trunc( (X * 100) / Z );
end;

{ TGauge }
constructor TWin95Gauge.Create(AOwner: TComponent);
begin
  	inherited Create(AOwner);
  	ControlStyle := ControlStyle + [csFramed, csOpaque];
  	FMinValue := 0;
  	FMaxValue := 100;
  	FCurValue := 0;
    FBoxes := 15;
    Width := 100;
    Height := 30;
end;

function TWin95Gauge.GetPercentDone: Longint;
begin
	GetPercentDone := SolveForY(FCurValue - FMinValue, FMaxValue - FMinValue);
end;

procedure TWin95Gauge.SetBoxes;
begin
	if FBoxes<>Value then begin
    	if Value<1 then Value:=1;
    	FBoxes:=Value;
    	Width := FBoxes*(FBoxW)+4;
	    Height := FBoxH+3;
	    SetBounds(Left,Top,Width,Height);
        Invalidate;
    end;
end;

(*		*)
procedure TWin95Gauge.CheckBounds;
var maxboxes:word;
begin
	maxboxes := (Width-4) div (FBoxW);
    Width := maxboxes*(FBoxW)+4;
    if maxboxes<1 then maxboxes:=1;
    Height := FBoxH+3;
    FBoxes := maxboxes;
    SetBounds(Left,Top,Width,Height);
end;

(*		*)
procedure TWin95Gauge.Paint;
var FillSize,k: byte;
    tr : TRect;
    TheRect : TRect;
    bitmap : TBitmap;
    W,H : integer;
begin
	CheckBounds;
  	with Canvas do begin
    	TheRect := ClipRect;
    	Frame3D(Canvas,TheRect,clGray,clWhite,1);
        bitmap := TBitmap.Create;
        W := TheRect.Right-TheRect.Left+1;
        H := TheRect.Bottom-TheRect.Top+1;
        bitmap.Width := W;
        bitmap.Height := H;
        bitmap.canvas.Brush.Color := clSilver;
  		bitmap.Canvas.Brush.Style := bsSolid;
        bitmap.canvas.FillRect(Rect(0,0,W,H));
        if Enabled then
        	bitmap.canvas.Brush.Color := clNavy
        else bitmap.canvas.Brush.Color := clGray;
    	(**)
	    FillSize := SolveForX(PercentDone, FBoxes);
        if FillSize>FBoxes then FillSize := FBoxes;
        tr := Rect(TheRect.Left,TheRect.Top,FBoxW,FBoxH);
        for k:=1 to FillSize do begin
            bitmap.canvas.FillRect(tr);
        	OffsetRect(tr,FBoxW,0);
        end;
	    (**)
        Canvas.CopyRect(TheRect,bitmap.Canvas,Bounds(0,0,W,H));
        Bitmap.Free;
    end;
end;


procedure TWin95Gauge.SetMinValue(Value: Longint);
begin
  if Value <> FMinValue then
  begin
    FMinValue := Value;
    Refresh;
  end;
end;

procedure TWin95Gauge.SetMaxValue(Value: Longint);
begin
  if Value <> FMaxValue then
  begin
    FMaxValue := Value;
    Refresh;
  end;
end;

procedure TWin95Gauge.SetProgress(Value: Longint);
begin
	if (FCurValue <> Value) and (Value >= FMinValue) and (Value <= FMaxValue) then
  	begin
    	FCurValue := Value;
    	Refresh;
  	end;
end;

procedure TWin95Gauge.AddProgress(Value: Longint);
begin
	Progress := FCurValue + Value;
  	Refresh;
end;

(*			*)
procedure Register;
begin
  RegisterComponents('ASD', [TWin95Gauge]);
end;

end.
