{*******************************************************
                GrepEng Component

This unit contains the code for the the DBModeIndicator.
DBModeIndicator appears the same as the mode indicator in
the Delphi demo MastApp. Unlike the mode indicator in
MastApp DBModeIndicator attatches to a TDataSource and
responds automatically to DataSet state changes.
Highlight, Dimmed and Active colors can be set in the
object inspector.

                Paul Warren
       HomeGrown Software Development
     (c) 1995 Langley British Columbia.
              (604) 530-9097
       e-mail:  hg_soft@uniserve.com
  Home page: http://haven.uniserve.com/~hg_soft

********************************************************}

unit Dbmode;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, DB, DBTables;

const
  DatasetStates: array[TDataSetState] of string[11] =
    ('Not active', 'Browsing', 'Editing', 'Inserting',
    'Setting key', 'Calculating');

type
  TDBModeIndicator = class(TCustomLabel)
  private
    { Private declarations }
    FDataLink: TFieldDataLink;
    FActive: TColor;
    FDimmed: TColor;
    FHighlight: TColor;
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure SetActive(Value: TColor);
    procedure ActiveChange(Sender: TObject);
    procedure UpdateLabel;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Align;
    property Alignment;
    property AutoSize;
  {  property Caption; }
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
  {  property FocusControl; }
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
  {  property ShowAccelChar; }
    property ShowHint;
    property Transparent;
    property Visible;
  {  property WordWrap; }
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property Active: TColor read FActive write SetActive default clBlue;
    property Dimmed: TColor read FDimmed write FDimmed default clGray;
    property Highlight: TColor read FHighlight write FHighlight default clRed;
  end;

procedure Register;

implementation

{ construct TDBModeIndicator }
constructor TDBModeIndicator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FActive := clBlue;      { default colors Blue }
  FDimmed := clGray;      { gray }
  FHighlight := clRed;    { red }
  FDataLink := TFieldDataLink.Create;   { create the DataLink }
  FDataLink.OnActiveChange := ActiveChange; { attach method to event }
  FDataLink.OnDataChange := ActiveChange; { attach method to event }
end;

{ destroy TDBModeIndicator }
destructor TDBModeIndicator.Destroy;
begin
  FDataLink.Free;     { free the DataLink resources }
  FDataLink := nil;   { set to nil }
  inherited Destroy;
end;

{ method to get DataSource from DataLink }
function TDBModeIndicator.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{ method to set DataSource in DataLink }
procedure TDBModeIndicator.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

{ method to set active color - so the color changes
  when changed at design time }
procedure TDBModeIndicator.SetActive(Value: TColor);
begin
  FActive := Value;
  Self.Font.Color := Active;
end;

{ method to update the unpublished Caption property in
  response to a change in the DataSource }
procedure TDBModeIndicator.ActiveChange(Sender: TObject);
begin
  if not (FDataLink.DataSource = nil) then UpdateLabel; { check DataSource NOT nil }
end;

{procedure to update the label}
procedure TDBModeIndicator.UpdateLabel;
begin
  {update state caption}
  with FDataLink.DataSource do
  begin
    if Dataset <> nil then Self.Caption :=
      Format('[%S: %S]', [Dataset.Name, DatasetStates[State]]);
    if State in [dsEdit, dsInsert] then
      Self.Font.Color := Highlight
    else if State in [dsBrowse, dsSetKey, dsCalcfields] then
      Self.Font.Color := Active
    else Self.Font.Color := Dimmed;
  end;
end;

{ register component }
procedure Register;
begin
  RegisterComponents('Data Controls', [TDBModeIndicator]);
end;

end.
