{*******************************************************
                DBRecNum Component

This unit contains the code for the DBRecNum component.
DBRecNum attaches to a TDataSource and displays the record
number for DBase and Paradox tables. Unlike similar
components, DBRecNum responds automatically to changes
in the DataSource.

                Paul Warren
       HomeGrown Software Development
     (c) 1995 Langley British Columbia.
              (604) 530-9097
       e-mail:  hg_soft@uniserve.com
  Home page: http://haven.uniserve.com/~hg_soft

********************************************************}

unit DBRecNum;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBiProcs, DBiTypes, StdCtrls, DBTables, Menus;

type
 TDBRecNum = class(TCustomLabel)
  private
    { Private declarations }
    FDataLink: TFieldDataLink;
    FActive: TColor;
    FDimmed: TColor;
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure SetActive(Value: TColor);
    procedure DataChange(Sender: TObject);
    function FindRecordNumber: longint;
    procedure UpdateLabel;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Align;
    property Alignment;
    property AutoSize;
  {  property Caption; }
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
  {  property FocusControl; }
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
  {  property ShowAccelChar; }
    property ShowHint;
    property Transparent;
    property Visible;
  {  property WordWrap; }
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property Active: TColor read FActive write SetActive default clBlue;
    property Dimmed: TColor read FDimmed write FDimmed default clGray;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

procedure Register;

{****************************************************************************}
implementation

{ construct the component }
constructor TDBRecNum.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FActive := clBlue;
  FDimmed := clGray;
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange; { attach method to event }
  FDataLink.OnActiveChange := DataChange; { attach method to event }
end;

{ destroy the component }
destructor TDBRecNum.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

{ method to get DataSource from DataLink }
function TDBRecNum.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

{ method to set DataSource in DataLink }
procedure TDBRecNum.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

{ method to set active color - so the color changes
  when changed at design time }
procedure TDBRecNum.SetActive(Value: TColor);
begin
  FActive := Value;
  Self.Font.Color := Active;
end;

{ method to update the unpublished Caption property in
  response to a change in the DataSource }
procedure TDBRecNum.DataChange(Sender: TObject);
begin
  if not (FDataLink.DataSource = nil) then UpdateLabel; { check DataSource NOT nil }
end;

{ method to find record number }
function TDBRecNum.FindRecordNumber: Longint;
var
  cP: CurProps;
  rP: RECProps;
begin
  {Return 0 if dataset is not Paradox or dBase}
  Result := 0;

  with FDataLink.DataSource.Dataset do
  begin
    try
      {we need to make this call to grab the cursor's iSeqNums}
      Check(DBiGetCursorProps(Handle,cP));
      {synchronize the BDE cursor with the dataset's cursor}
      UpdateCursorPos;
      {fill rP with the current record's properties}
      Check(DBiGetRecord(Handle,DBiNOLOCK,nil,@rP));
      {what kind of dataset are we looking at?}
      case cP.iSeqNums of
        0: Result := rP.iPhyRecNum; {dBase}
        1: Result := rP.iSeqNum;    {Paradox}
      end;
    except
      Result := -1;
    end;
  end;
end;

{procedure to update the label}
procedure TDBRecNum.UpdateLabel;
var
 i: longint;
begin
  with FDataLink.DataSource do
  {check if the database is active}
  if (DataSet <> nil) and (DataSet.State <> dsInActive) and (DataSet.RecordCount <> 0) then
  begin
    Self.Font.Color := Active;
    i := FindRecordNumber;
    if i >= 0 then
      Self.caption := 'Record '+IntToStr(i)+' of '+IntToStr(DataSource.Dataset.RecordCount)
    else Self.caption := 'Record '+IntToStr(0)+' of '+IntToStr(0);
  end else
  begin
    Self.Font.Color := Dimmed;
    Self.Caption := 'Record '+IntToStr(0)+' of '+IntToStr(0);
  end;
end;

{procedure to register this as a custom component}
procedure Register;
begin
  RegisterComponents('Data Controls', [TDBRecNum]);
end;

end.
