{
LMD-Tools (Free Components)
---------------------------

These components are Freeware and were originally part
of the LMD-Tools component-package. Included is

TLMDAPP
TLMDDirDlg
TLMDSysInfo

Property-Editor for HelpFiles
Procedure CenterChild
Procedure CheckPath

Use the components at your own risk. You are free to use these
components in commercial or non-commercial products, as long as
they will not be distributed as part of a component-collection
or -library.

We include a Helpfile (LMDFREE.HLP - an extract of the original
LMD-Tools hepfile).

Any comments or suggestions are we welcome.

Contact:
LMD Innovative, CIS-ID: 100416,376
}
unit Lmdfree;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, StdCtrls, ExtCtrls, dsgnintf, printers, filectrl;

type

  TLanguageMode=(lgEnglish, lgGerman);

  TWinFlag=(wf80x87, wfCPU286, wfCPU386, wfCPU486, wfENHANCED,
            wfPAGING, wfPMODE, wfSTANDARD,  wfWIN286, wfWIN386);
  TWinFlags=set of TWinFlag;


  {----------------------TLMDApp ----------------------------------}
  TLMDApp = class(TComponent)
  private
    FTitle:String;
    FHelpFile:TFileName;
    FHintColor:TColor;
    FHintPause:Integer;
    FShowHint:Boolean;
    FIcon:TIcon;
    FOnHint:TNotifyEvent;
    FOnShowHint:TShowHintEvent;
    FOnActivate:TNotifyEvent;
    FOnDeactivate:TNotifyEvent;
    FOnException:TExceptionEvent;
    FOnMessage:TMessageEvent;
    FOnHelp:THelpEvent;
    FOnIdle:TIdleEvent;
    FOnRestore:TNotifyEvent;
    FOnMinimize:TNotifyEvent;
    procedure SetIcon(aValue:TIcon);
    procedure SetHelpFile(aValue:TFileName);
    procedure SetHintColor(aValue:TColor);
  protected
    procedure Loaded;override;
  public
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
  published
    property HintColor:TColor read FHintColor write SetHintColor default $0080FFFF;
    property HintPause:Integer read FHintPause write FHintPause default 800;
    property Icon:TIcon read FIcon write SetIcon;
    Property HelpFile:TFilename read FHelpFile write SetHelpFile;
    property ShowHint:Boolean read FShowHint write FShowHint default true;
    property Title:String read FTitle write FTitle;
    property OnActivate:TNotifyEvent read FOnActivate write FOnActivate;
    property OnDeactivate:TNotifyEvent read FOnDeactivate write FOnDeactivate;
    property OnException:TExceptionEvent read FOnException write FOnException;
    property OnHelp:THelpEvent read FOnHelp write FOnHelp;
    property OnIdle:TIdleEvent read FOnIdle write FOnIdle;
    property OnMessage:TMessageEvent read FOnMessage write FOnMessage;
    property OnMinimize: TNotifyEvent read FOnMinimize write FOnMinimize;
    property OnHint: TNotifyEvent read FOnHint write FOnHint;
    property OnRestore: TNotifyEvent read FOnRestore write FOnRestore;
    property OnShowHint:TShowHintEvent read FOnShowHint write FOnShowHint;
  end;

  {$R LMDDIR.DFM}
  {----------------------Dialog for TLMDDirDlg -------------------------------}
  TDirBox = class(TForm)
    okbtn: TButton;
    DirectoryListBox1: TDirectoryListBox;
    Button1: TButton;
    DriveComboBox1: TDriveComboBox;
    pfad: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    procedure DriveComboBox1Change(Sender: TObject);
  private
    FShow:Boolean;
  public
    AName:String;
    Language:TLanguageMode;
  end;

  {------------------------- TLMDDirDlg --------------------------------------}
  TLMDDirDlg = class(TComponent)
  private
    FCaptionTitle:String;
    FLanguage:TLanguageMode;
    FPath:String;
    FShowDir:Boolean;
  protected
  public
    constructor Create(AOwner:TComponent);override;
    function Execute:Boolean;
    function ExecuteEnh(Main:TForm):Boolean;
    Procedure GetDir(const title:String; var aPath:String);
  published
    { Published-Deklarationen }
    property CaptionTitle:String read FCaptionTitle write FCaptionTitle;
    property Language:TLanguageMode read FLanguage write FLanguage default lgEnglish;
    Property Path:String read FPath write FPath;
    property ShowLabelDir:Boolean read FShowDir write FShowDir default false;
  end;

{------------------------- TLMDSysInfo ----------------------------------------}
  TLMDSysInfo = class(TComponent)
  private
    FEnvironment:TStrings;
    procedure Dummy(aString:String);
    procedure DummyInt(AInt:LongInt);
    procedure DummyFlag(AFlag:TWinFlags);
    procedure DummyStrings(AList:TStrings);
    function GetVersionWin:String;
    function GetVersionDOS:String;
    function GetWindowsDir:String;
    function GetWindowsSysDir:String;
    function GetLargestBlock:LongInt;
    function GetMemFree:LongInt;
    function GetSSDelay:LongInt;
    function GetFreeSys:String;
    function GetFreeUsr:String;
    function GetFreeGdi:String;
    function GetWinflag:TWinFlags;
    function GetUsername:String;
    function GetUserCompany:String;
    function GetFileName:String;
    function GetFonts:TStrings;
    function GetPrinters:TStrings;
  protected
  public
    constructor Create(AOwner:TComponent);override;
    destructor destroy;override;

    function FreeSys:Word;
    function FreeGDI:Word;
    function FreeUSR:Word;

    property TempFileName:string read GetFileName;
  published
    property AllFonts:TStrings read GetFonts write DummyStrings stored false;
    property AllPrinters:TStrings read GetPrinters write DummyStrings stored false;
    property Environment:TStrings read FEnvironment write DummyStrings stored false;
    property FreeSysRes: String read GetFreeSys write Dummy stored false;
    property FreeGdiRes:String read GetFreeGDi write Dummy stored false;
    property FreeUsrRes:String read GetFreeUsr write Dummy stored false;
    property MemMaxBlock: longint read GetLargestBlock write DummyInt stored false;
    property MemFree: longint read GetMemFree write DummyInt stored false;
    property ScreenSaverDelay: Longint read GetSSDelay write DummyInt Stored false;
    property WinFlags:TWinFlags read GetWinFlag write DummyFlag stored false;
    property Username:String read GetUserName write Dummy stored false;
    property UserCompany:string read GetUserCompany write Dummy stored false;
    property VersionDOS:string read GetVersionDOS write Dummy stored false;
    property VersionWIN:string read GetVersionWin write Dummy stored false;
    property WinPath:string read GetWindowsDir write Dummy stored false;
    property WinSysPath:string read GetWindowsSysDir write Dummy stored false;
  end;

  {---------------------- Property-Editor for HelpFile ------------------------}
  THelpFileNameProperty = class (TStringProperty)
      function GetAttributes: TPropertyAttributes; override;
      procedure Edit; override;
  end;

  {----------------------------------------------------------------------------}
  procedure centerchild(frmParent:TForm;frmchild:TForm;flag:boolean);
  function checkPath(path:string; flag:boolean):String;
  procedure Register;

const  {Constants for Property-Editor}
   HELPFILTER='Helpfiles (*.hlp)|*.hlp';  {Hilfedateien (*.hlp)|*.hlp';}
   HELPTITLE='Specify HelpFile';  {Hilfedatei bestimmen';}

implementation
{*************************** Class TLMDApp **********************************}
{----------------------------  Private --------------------------------------}
procedure TLMDAPP.SetIcon(aValue:TIcon);
begin
  FIcon.assign(aValue);
end;

{---------------------------------------------------------------------------}
procedure TLMDAPP.SetHelpFile(aValue:TFileName);
begin
  FHelpFile:=aValue;
end;

{---------------------------------------------------------------------------}
procedure TLMDAPP.SetHintColor(aValue:TColor);
begin
  FHintColor:=aValue;
end;

{---------------------------- Protected-------------------------------------}
Procedure TLMDAPP.Loaded;
begin

  inherited loaded;
  with application do
    begin

      if fileexists(FHelpFile) then
        HelpFile:=FHelpFile
      else
        Helpfile:=ExtractFileName(HelpFile);

      if assigned(Ficon) then Icon.Assign(FIcon);
      ShowHint:=FShowHint;
      Title:=FTitle;
      HintPause:=FHintPause;
      HintColor:=FHintColor;
      if assigned(FOnHint) then OnHint:=FOnHint;
      if assigned(FOnShowHint) then OnShowHint:=FOnShowHint;
      if assigned(FOnActivate) then OnActivate:=FOnActivate;
      if assigned(FOnDeactivate) then OnDeActivate:=FOnDeactivate;
      if assigned(FOnException) then OnException:=FOnException;
      if assigned(FOnMessage) then OnMessage:=FOnMessage;
      if assigned(FOnHelp) then OnHelp:=FOnHelp;
      if assigned(FOnIdle) then OnIdle:=FOnIdle;
      if assigned(FOnRestore) then OnRestore:=FOnRestore;
      if assigned(FOnMinimize) then OnMinimize:=FOnMinimize;
    end;
end;

{---------------------------- Public  -------------------------------------}
constructor TLMDAPP.Create(aOwner:TComponent);
begin
  inherited Create(aOwner);
  FIcon:=TIcon.Create;
  with application do
    begin
      FHelpFile:=HelpFile;
      if icon<>nil then FIcon.Assign(Icon);
      FShowHint:=ShowHint;
      FTitle:=Title;
      FHintPause:=HintPause;
      FHintColor:=HintColor;
    end;
end;

{---------------------------------------------------------------------------}
Destructor TLMDAPP.Destroy;
begin
  FIcon.Free;
  inherited Destroy;
end;

{*************************** Class TDirBox **********************************}
{----------------------------------------------------------------------------}
procedure TDirBox.DriveComboBox1Change(Sender: TObject);
begin
   DirectoryListBox1.Drive := DriveComboBox1.Drive;
end;

{*************************** Class TLMDDirDlg *******************************}
{------------------------------- Public -------------------------------------}
constructor TLMDDirDlg.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
end;

{----------------------------------------------------------------------------}
function TLMDDirDlg.ExecuteEnh(main:TForm):Boolean;
var
  DirBox:TDirBox;

begin

  Dirbox:=TDirBox.Create(self);
  try
    with DirBox do begin
      if length(FCaptionTitle)>0 then
        Caption:=FCaptionTitle
      else
        case Flanguage of
          lgEnglish:begin
                      Caption:='Select Path...';
                      pfad.caption:='Path:';
                      label1.caption:='Drive:';
                      button1.Caption:='&Cancel';
                    end;
          lgGerman :begin
                      Caption:='Pfad auswhlen';
                      pfad.caption:='Pfad:';
                      label1.caption:='Laufwerk:';
                      button1.Caption:='&Abbruch';
                    end;
        end;

      if FShowdir then label2.visible:=true else label2.visible:=false;
      if FPath>'' then
        DirectoryListBox1.Directory:=FPath
      else
        DirectoryListBox1.Directory:=Extractfilepath(application.Exename);
      If assigned(main) then
        begin
          centerchild(main, dirbox,false);
        end
      else
        position:=poScreenCenter;

      if showmodal=mrOK then
        begin
          FPath:=DirectoryListBox1.Directory;
	  Result:=true;
	end;
    end;
  finally
    dirbox.free;
  end;

end;

{----------------------------------------------------------------------------}
Function TLMDDirDlg.Execute:Boolean;
begin
  result:=ExecuteEnh(nil);
end;

{----------------------------------------------------------------------------}
Procedure TLMDDirDlg.GetDir(const title:String; var aPath:String);
begin

   FCaptionTitle:=title;
   FPath:=apath;
   if ExecuteEnh(nil) then aPath:=FPath;
end;

{*************************** Class TLMDSysInfo ******************************}
{------------------------- Private ------------------------------------------}
function TLMDSysInfo.GetVersionWin:String;
begin
  result:=format('%d.%d', [LoByte(LoWord(Getversion)), HiByte(LoWord(Getversion))]);
end;

{---------------------------------------------------------------------------}
Function TLMDSysInfo.GetVersionDOS:String;
begin
  result:=format('%d.%d', [HiByte(HiWord(Getversion)), LoByte(HiWord(Getversion))]);
end;

{---------------------------------------------------------------------------}
Function TLMDSysInfo.GetSSDelay:LongInt;
var temp:integer;
begin
  SystemParametersInfo(SPI_GETSCREENSAVETIMEOUT,0, @temp,0);
  result:=temp Div 60;
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetWindowsDir:String;
var
  buffer:array[0..255] of char;
begin
  GetWindowsDirectory(buffer, 255);
  result:=checkpath(StrPas(buffer),false);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetWindowsSysDir:String;
var
  buffer:array[0..255] of Char;
begin
  GetSystemDirectory(buffer, 255);
  result:=checkpath(StrPas(buffer),false);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetLargestBlock:Longint;
begin
   result:=MaxAvail div 1024 ;
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetMemFree:Longint;
begin
   result:=GetFreeSpace(0) div 1024 ;
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.FreeSys:Word;
begin
  result:=GetFreeSystemResources(GFSR_SYSTEMResources);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.FreeGDI:Word;
begin
  result:=GetFreeSystemResources(GFSR_GDIResources);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.FreeUSR:Word;
begin
  result:=GetFreeSystemResources(GFSR_USERResources);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetFreeSys:String;
begin
  result:=format('%d %%',[FreeSys]);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetFreeGDI:String;
begin
  result:=format('%d %%',[FreeGDI]);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetFreeUSR:String;
begin
  result:=format('%d %%',[FreeUSR]);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetWinFlag:TWinFlags;
begin
   result:=[];
   if (GetWinFlags and  WF_80x87)=WF_80x87 then result:=[wf80x87];
   if (GetWinFlags and WF_CPU286)=WF_CPU286 then result:=result+ [wfCPU286];
   if (GetWinFlags and WF_CPU386)=WF_CPU386 then result:=result+ [wfCPU386];
   if (GetWinFlags and WF_CPU486)=WF_CPU486 then result:=result+ [wfCPU486];
   if (GetWinFlags and WF_ENHANCED)=WF_ENHANCED then result:=result+ [wfEnhanced];
   if (GetWinFlags and WF_PAGING)=WF_PAGING then result:=result+ [wfPaging];
   if (GetWinFlags and WF_PMODE)=WF_PMODE then result:=result+ [wfPMODE];
   if (GetWinFlags and WF_Standard)=WF_Standard then result:=result+ [wfStandard];
   if (GetWinFlags and WF_WIN286)=WF_WIN286 then result:=result+ [wfwin286];
   if (GetWinFlags and WF_WIN386)=WF_WIN386 then result:=result+ [wfwin386];
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetUsername:String;
var Handle:THandle;
    buffer:array[0..32] of char;
begin
    Handle:= LoadLibrary('User');
    if LoadString(handle, 514,buffer,31)>0 then result:=StrPas(buffer);
    FreeLibrary(Handle);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetUserCompany:String;
var Handle:THandle;
    buffer:array[0..32] of char;
begin
    Handle:= LoadLibrary('User');
    if LoadString(handle, 515,buffer,31)>0 then result:=StrPas(buffer);
    FreeLibrary(Handle);
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetFileName:String;
var
  buffer:array[0..255] of char;
begin
 if GetTempFileName(GetTempDrive(chr(0)),'DEL',0, buffer)>0 then result:=StrPas(buffer)
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetFonts:TStrings;
begin
  result:=screen.Fonts;
end;

{---------------------------------------------------------------------------}
function TLMDSysInfo.GetPrinters:TStrings;
begin
  result:=printer.printers;
end;

{---------------------------------------------------------------------------}
Procedure TLMDSysInfo.Dummy(aString:String);
begin
end;

{---------------------------------------------------------------------------}
Procedure TLMDSysInfo.DummyInt(AInt:LongInt);
begin
end;

{---------------------------------------------------------------------------}
Procedure TLMDSysInfo.DummyFlag(aFlag:TWinFlags);
begin
end;

{---------------------------------------------------------------------------}
Procedure TLMDSysInfo.DummyStrings(aList:TStrings);
begin
end;

{--------------------  Public ----------------------------------------------}
constructor TLMDSysInfo.Create(AOwner:TComponent);
var temp:PChar;

begin
  inherited Create(AOwner);
  FEnvironment:=TStringList.Create;

  temp:=GetDosEnvironment;
    while temp[0]<>#0 do
      begin
        FEnvironment.add(strpas(temp));
        inc(temp, strlen(temp)+1);
      end;
end;

{---------------------------------------------------------------------------}
destructor  TLMDSysInfo.Destroy;
begin
  FEnvironment.free;
  inherited destroy;
end;

{********************HelpFileNameProperty**************************}
{------------------- THelpFileNameProperty ------------------------}
function THelpFileNameProperty.GetAttributes: TPropertyAttributes;
begin
  result := [paDialog];
end;

{------------------------------------------------------------------}
Procedure THelpFileNameProperty.Edit;
var OpenDialog:TOpenDialog;

  begin
    OpenDialog := TOpenDialog.Create(Application);
    with opendialog do
      try
         filter:=HELPFILTER;
         title:=HELPTITLE;
         options:=[ofHideReadOnly,ofExtensionDifferent,ofPathMustExist,ofFileMustExist];
         defaultext:='hlp';filename:=GetStrValue;
         if execute then SetStrValue(filename);
      finally
        Free;
      end;
  end;

{-----------------------------------------------------------------------}
procedure centerchild (frmParent:TForm; frmchild: TForm; flag:boolean);
var l,t:integer;
begin

  frmchild.position:=poDefault;

  l:= frmParent.Left + ((frmParent.Width - frmchild.Width) div 2);
  If (l + frmchild.Width) > screen.Width Then l:= screen.Width - frmchild.Width;

  t := frmParent.Top + ((frmParent.Height - frmchild.Height) div 2);
  If (t + frmchild.Height) > screen.Height Then t:= screen.Height - frmchild.Height;

  frmchild.Left := l; frmchild.Top := t;

  {Soll modal angezeigt werden?}
  if flag then frmchild.showmodal;

end;

{-----------------------------------------------------------------------}
function checkPath(path:string; flag:boolean):String;
{Funktion prft, ob letztes Zeichen in Pfadangabe ein '\' ist
 Flag:
        TRUE - '\' Zeichen erwnscht
        FALSE - '\' Zeichen unerwnscht}

begin
  if (flag=true) then
    begin
      if path[length(path)]<>'\' then path:=path+'\'
    end
  else
    if path[length(path)]='\' then path:=copy(path,1,length(path)-1);

  result:=path;
end;

{******************** Register ***********************************************}
procedure Register;
begin
  {HelpFileName}
  RegisterComponents('LMDFree', [TLMDApp, TLMDDirDlg, TLMDSysInfo]);
  RegisterPropertyEditor( TypeInfo( TFileName ), TLMDAPP, 'HelpFile', THelpFileNameProperty );
end;
end.
